// Import TableMaster
var TableMaster = require('../table/tables');

// Provide session type
exports['session'] = function(session, msg)
{
	// No type specified yet?
	if(!session.type)
	{
		// Only allow 'table' or 'player'
		if(msg.type == 'table' || msg.type == 'player')
		{
			session.type = msg.type;
		}
		else
		{
			console.log(session + ' specified invalid type "' + msg.type + '"?!');
			session.end();
		}
	}
}

// Provide device info
exports['device'] = function(session, msg)
{
	console.log(session + ' is using ' + msg.userAgent);
	/*console.log('Window: ' + msg.width + 'x' + msg.height);
	console.log('Screen: ' + msg.screenWidth + 'x' + msg.screenHeight);*/
}

// Selecting a table to play on (or create it)
exports['selectTable'] = function(session, msg)
{
	// Not already table selected?
	if(session.table)
	{
		session.end();
		return;
	}

	// Specified table not already in TableMaster?
	var table = TableMaster.getTable(msg.id);
	if(!table)
	{
		// Can create table?
		if(session.type == 'table')
		{
			// Create table...
			table = TableMaster.createTable(msg.id, session);
			if(table)
			{
				// Table was just created! This *should* always evaluate to true
				if(session == table.host)
				{
					session.registerEvents('host');
				}
			}
			else
			{
				session.send('tableError', { error: 'creationFailed' });
			}
		}
		else
		{
			session.send('tableError', { error: 'notAllowed' });
			console.log(session + ' (type="player") cannot create a table...');
		}
	}


	// Table exists now?
	if(table)
	{
		table.addSession(session);
	}
	else
	{
		console.log(session + ' could not select table #' + msg.id);
	}
}

// Handle identification of a player
exports['identify'] = function(session, msg)
{
	// Valid table selected?
	var table = session.table;
	if(!table)
	{
		console.log(session + ' cannot identify: no table selected');
		session.end();
		return;
	}

	// Session type is 'player', and not identified already?
	if(session.type != 'player' || session.player != null)
	{
		console.log(session + ' cannot identify: not a player session, or already identified');
		session.end();
		return;
	}

	// Check nickname
	var check = table.isValidNickname(msg.nick);
	if(check.error)
	{
		session.send('nickError', check);
	}
	else
	{
		// Build the player info
		var info =
		{
			'nick': msg.nick,
			'color': msg.color,
			'avatarId': msg.avatarId
		};

		// Added successfully?
		if(table.addPlayer(session, info))
		{
			session.registerEvents('chat');
			session.registerEvents('cards');
			session.send('welcome', { msg: 'Welcome to #' + table.id + ', ' + session.player.nick + '!' });
		}
	}
}