// Constructor
var SessionManager = function()
{
	// Map with sessions
	this._sessions = {};

	// Use start() to start listening
	this._listener = null;
}

// Expose a singleton
module.exports = new SessionManager();

// Start listening
SessionManager.prototype.createServer = function(port)
{
	// Not already listening?
	if(!this._listener)
	{
		// Setup socket.io server, disable debug listening
		this._listener = require('socket.io').listen(port, {'log level': 1});
		this._listener.on('connection', this._onConnectionRequest.bind(this));

		// Up and running!
		console.log('SessionManager: listening for sessions on TCP/' + port);
	}
}

// Import Session
var Session = require('./session');

// Accept new connections
SessionManager.prototype._onConnectionRequest = function(socket)
{
	// Create Session and register event listeners
	var sessionId = socket.id.substr(0, 6);
	var session = new Session(sessionId, socket);

	// Session started!
	this._sessions[session.id] = session;
	console.log(session + ' started from ' + session.remoteAddress);
}

// Removes a session from the manager
SessionManager.prototype.removeSession = function(session)
{
	// Session active?
	if(session.id in this._sessions)
	{
		delete this._sessions[session.id];
		console.log(session + ' ended');
	}
}