package org.openhabbo.catalog;

import org.openhabbo.items.ItemDefinition;
import org.openhabbo.net.messages.SerializableObject;
import org.openhabbo.net.messages.ServerMessage;
import org.openhabbo.storage.DatabaseResult;

public class CatalogProduct implements SerializableObject
{
	public int id;
	public int pageId;
	public CatalogPage page;

	public String name;
	public int itemDefinitionId;
	public ItemDefinition itemDefinition;
	public int itemAmount;

	public int costCredits;
	public int costPixels;
	public int costQuestPoints;

	public final boolean isItem()
	{
		return (this.itemDefinition != null);
	}
	
	public final boolean isNonItem()
	{
		return !this.isItem();
	}
	
	public final char getType()
	{
		if(this.isItem())
		{
			return this.itemDefinition.getType();
		}
		else
		{
			return 'e'; // Or 'h' for subscriptions!
		}
	}
	
	@Override
	public void serialize(ServerMessage msg)
	{
		msg.appendInt(this.id);
		msg.appendStr(this.name);
		msg.appendInt(this.costCredits);
		msg.appendInt(this.costPixels);
		msg.appendInt(this.costQuestPoints);
		msg.appendInt(1);
		msg.appendChr(this.getType());
		msg.appendInt(this.itemDefinition.id);
		msg.appendStr("");
		msg.appendInt(this.itemAmount);
		msg.appendInt(-1);
	}

	public static CatalogProduct parse(DatabaseResult row)
	{
		CatalogProduct product = new CatalogProduct();
		product.id = row.getInt("id");
		product.pageId = row.getInt("page_id");
		product.name = row.getString("name");
		product.itemDefinitionId = row.getInt("definition_id");
		product.itemAmount = row.getInt("amount");
		product.costCredits = row.getInt("cost_credits");
		product.costPixels = row.getInt("cost_pixels");
		product.costQuestPoints = row.getInt("cost_questpoints");
		return product;
	}
}
