package org.openhabbo.catalog;

import org.openhabbo.items.ItemDefinition;
import org.openhabbo.users.User;

public final class CatalogProduct
{
	public int id;
	public int pageId;
	public CatalogPage page;

	public String name;
	public int itemDefinitionId;
	public ItemDefinition itemDefinition;
	public int itemAmount;

	public int costCredits;
	public int costPixels;
	public int costQuestPoints;

	public final boolean isItem()
	{
		return (this.itemDefinition != null);
	}
	
	public final boolean isNonItem()
	{
		return !this.isItem();
	}
	
	public final char getType()
	{
		if(this.isItem())
		{
			return this.itemDefinition.getType();
		}
		else
		{
			return 'e'; // Or 'h' for subscriptions!
		}
	}
	
	public boolean isPurchasableBy(User user)
	{
		// Can user access page?
		if(this.page == null || !this.page.isAccessibleTo(user))
		{
			return false;
		}
		
		// Can user actually afford this?
		if(user.credits < this.costCredits || user.pixels < this.costPixels)
		{
			return false;
		}

		// Purchasable!
		return true;
	}
}
