package org.openhabbo.core;

import java.util.HashMap;
import java.util.Map;

public abstract class AttachmentBox
{
	private final Map<String, Object> attachments;

	protected AttachmentBox()
	{
		this(3);
	}

	protected AttachmentBox(int initialAttachments)
	{
		this.attachments = new HashMap<String, Object>();
	}

	public final <T> void set(String key, Object attachment)
	{
		synchronized(this.attachments)
		{
			this.attachments.put(key, attachment);
		}
	}

	public final void unset(String key)
	{
		synchronized(this.attachments)
		{
			this.attachments.remove(key);
		}
	}

	public final boolean has(String key)
	{
		synchronized(this.attachments)
		{
			return this.attachments.containsKey(key);
		}
	}

	@SuppressWarnings("unchecked")
	public final <T> T get(String key)
	{
		synchronized(this.attachments)
		{
			Object attachment = this.attachments.get(key);
			if(attachment != null)
			{
				return (T) attachment;
			}
			else
			{
				return null;
			}
		}
	}
}
