package org.openhabbo.core;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

public abstract class DynamicObject
{
	private final Map<String, Object> attachments;

	protected DynamicObject()
	{
		this(3);
	}

	protected DynamicObject(int initialAttachments)
	{
		this.attachments = new ConcurrentHashMap<String, Object>(initialAttachments);
	}

	public final void set(String key)
	{
		this.set(key, "");
	}
	
	public final <T> void set(String key, T attachment)
	{
		this.attachments.put(key, attachment);
	}

	public final boolean unset(String key)
	{
		return (this.attachments.remove(key) != null);
	}

	public final boolean has(String key)
	{
		return this.attachments.containsKey(key);
	}

	@SuppressWarnings("unchecked")
	public final <T> T get(String key)
	{
		Object attachment = this.attachments.get(key);
		if(attachment != null)
		{
			return (T) attachment;
		}
		else
		{
			return null;
		}
	}
}
