package org.openhabbo.modules;

import java.lang.annotation.*;

/**
 * A bundle of meta information to be attached to a Module. Please construct this meta-information exactly like the documentation specifies, or the module will be seen as corrupt!
 * 
 * @see Module
 */
@Retention(RetentionPolicy.RUNTIME)
@Target({ ElementType.TYPE })
@Documented
public @interface ModuleInfo
{
	/**
	 * The name of the module, entirely lowercase, starting with "mod_". Modules starting with names other than 'mod_' cannot be used.
	 */
	String name();

	/**
	 * A brief and friendly description of the module and its purpose.
	 */
	String description();

	/**
	 * The author of the module. This can be a full name, a nickname, etc.
	 */
	String author();

	/**
	 * The URL of the module. Please specify null when this module does not have a webpage.
	 */
	String url();

	/**
	 * The version string of the module. Examples of correctly-formatted version strings are '1.0.0' (default), '1.0.1', '2.5.0', '3.5.5-beta' etc. Always specify three version digits (major, minor, build). The suffix is optional.
	 */
	String version();

	/**
	 * A string array containing dependencies for this module. Every element is a concatenation of the module name and the minimum required version string. If the minimum version string is omitted, then any version will be accepted. Examples: mod_helloworld/2.0.0, mod_hamsterdance/1.0.0, mod_meow, etc.
	 * 
	 * @see #name()
	 * @see #version()
	 * 
	 */
	String[] dependencies();
}