package org.openhabbo.rooms.instances;

import java.util.ArrayList;
import java.util.List;

import org.openhabbo.items.Item;

public final class RoomMap
{
	public final static byte OPEN = 0;
	public final static byte CLOSED = 1;
	public final static byte INTERACTIVE = 2;

	public byte[][] state;
	public float[][] height;
	public boolean[][] avatars;

	public RoomMap(int maxX, int maxY)
	{
		this.state = new byte[maxX][maxY];
		this.height = new float[maxX][maxY];
		this.avatars = new boolean[maxX][maxY];
	}

	public boolean exists(RoomTile tile)
	{
		return this.exists(tile.x, tile.y);
	}

	public boolean exists(int x, int y)
	{
		return(x >= 0 && x < this.state.length && y >= 0 && y < this.state[0].length);
	}

	public boolean walkable(RoomTile tile, boolean lastStep)
	{
		return this.walkable(tile.x, tile.y, lastStep);
	}

	public boolean walkable(int x, int y)
	{
		return this.walkable(x, y, false);
	}

	public boolean walkable(int x, int y, boolean lastStep)
	{
		// Tile exists?
		if(!this.exists(x, y))
		{
			return false;
		}

		// User here?
		if(this.avatars[x][y])
		{
			return false;
		}

		// Blocked?
		if(this.state[x][y] == RoomMap.CLOSED)
		{
			return false;
		}

		// Allow laststep?
		if(this.state[x][y] == RoomMap.INTERACTIVE && !lastStep)
		{
			return false;
		}

		// Walkable
		return true;
	}

	public static List<RoomTile> getAffectedTiles(Item obj, boolean includeRoot)
	{
		return getAffectedTiles(obj.definition.length, obj.definition.width, obj.x, obj.y, obj.rotation, includeRoot);
	}

	public static List<RoomTile> getAffectedTiles(byte length, byte width, int X, int Y, byte rotation, boolean includeRoot)
	{
		List<RoomTile> tiles = new ArrayList<RoomTile>(0);

		// Is this a non-square item?
		if(length != width)
		{
			// Flip rotation
			if(rotation == 0 || rotation == 4)
			{
				byte tmpL = length;
				length = width;
				width = tmpL;
			}
		}

		/*for(int iX = X; iX < X + width; iX++)
		{
			for(int iY = Y; iY < Y + length; iY++)
			{
				tiles.add(new RoomTile(iX, iY));
			}
		}*/
		
		for(int iX = X; iX < X + length; iX++)
		{
			for(int iY = Y; iY < Y + width; iY++)
			{
				tiles.add(new RoomTile(iX, iY));
			}
		}

		return tiles;
	}

	public static boolean tilesTouch(int x, int y, int x2, int y2)
	{
		return(Math.abs(x - x2) <= 1 && Math.abs(y - y2) <= 1);
	}
}
