package org.openhabbo.sessions.handlers;

import java.util.Collection;
import java.util.List;

import org.openhabbo.events.Event;
import org.openhabbo.items.EItemFlags;
import org.openhabbo.items.Item;
import org.openhabbo.rooms.instances.RoomInstance;
import org.openhabbo.rooms.instances.RoomAvatar;
import org.openhabbo.sessions.EServerMessage;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionMessageHandler;
import org.openhabbo.sessions.messages.ClientMessage;
import org.openhabbo.sessions.messages.ServerMessage;

public class GetRoomContentDataHandler implements SessionMessageHandler
{
	public final static Event onSendRoomContent = new Event("OnSendRoomData");
	public final static Event onSendUserInfo = new Event("OnSendUserInfo");

	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Shorthand variable
		RoomInstance instance = session.getRoomInstance();
		if(instance == null)
		{
			return;
		}
		
		// If entering a guestroom, then send more data, client doesn't ask for it
		if(instance.getInfo().isGuestRoom())
		{
			// Send wallpaper/floor/landscape if not default data
			if(!instance.getInfo().wallpaper.isEmpty())
			{
				response.init(EServerMessage.RoomProperty);
				response.appendStr("wallpaper");
				response.appendStr(instance.getInfo().wallpaper);
				session.send(response);
			}
			if(!instance.getInfo().floor.isEmpty())
			{
				response.init(EServerMessage.RoomProperty);
				response.appendStr("floor");
				response.appendStr(instance.getInfo().floor);
				session.send(response);
			}
			if(!instance.getInfo().landscape.isEmpty())
			{
				response.init(EServerMessage.RoomProperty);
				response.appendStr("landscape");
				response.appendStr(instance.getInfo().landscape);
				session.send(response);
			}
		}
		
		// Passive objects
		List<Item> items = instance.getItems(EItemFlags.PASSIVEOBJECT);
		response.init(30);
		response.appendInt(items.size());
		for(Item item : items)
		{
			response.appendObject(item);
		}
		session.send(response);

		// Public rooms have a different 'interface'
		if(instance.getInfo().isPublicRoom())
		{
			response.init(471);
			response.appendBool(false);
			response.appendStr(instance.getInfo().modelName);
			response.appendBool(false);
			session.send(response);
		}
		else
		{
			// Hide walls
			response.init(472);
			response.appendBool(instance.getInfo().hideWalls);
			session.send(response);

			// Send floor item list
			items = instance.getItems(EItemFlags.FLOORITEM);
			response.init(EServerMessage.FloorItems);
			response.appendInt(items.size());
			for(Item item : items)
			{
				response.appendObject(item);
			}
			session.send(response);

			// Send wall item list
			items = instance.getItems(EItemFlags.WALLITEM);
			response.init(EServerMessage.WallItems);
			response.appendInt(items.size());
			for(Item item : items)
			{
				response.appendObject(item);
			}
			session.send(response);

			// Interface info (edit room etc)
			response.init(471);
			response.appendBool(true);
			response.appendInt(instance.getInfo().id);
			response.appendBool(instance.getInfo().ownerId == session.getUser().id);
			session.send(response);
		}
		
		// Other room content
		GetRoomContentDataHandler.onSendRoomContent.fire(session);

		// Send avatars
		Collection<RoomAvatar> avatars = instance.getAllAvatars();
		response.init(EServerMessage.Users);
		response.appendInt(avatars.size());
		for(RoomAvatar avatar : avatars)
		{
			response.appendObject(avatar);
		}
		session.send(response);

		// Try spawning room user
		if(session.enterRoom())
		{
			session.sendRoomPrivileges();
		}
		else
		{
			return;
		}

		// Send statuses of all avatars
		response.init(EServerMessage.UserUpdate);
		response.appendInt(avatars.size());
		for(RoomAvatar avatar : avatars)
		{
			avatar.appendStatusString(response);
		}
		session.send(response);

		// Send other special user statuses
		for(RoomAvatar avatar : avatars)
		{
			if(avatar.danceId >= 1)
			{
				response.init(EServerMessage.Dance);
				response.appendInt(avatar.id);
				response.appendInt(avatar.danceId);
				session.send(response);
			}
			if(avatar.carryItemId != 0 && avatar.carryItemTimer != 0)
			{
				response.init(EServerMessage.CarryObject);
				response.appendInt(avatar.id);
				response.appendInt(avatar.carryItemTimer);
				session.send(response);
			}

			GetRoomContentDataHandler.onSendUserInfo.fire(session, avatar);
		}
	}
}