package org.openhabbo.sessions.handlers;

import org.openhabbo.catalog.Catalog;
import org.openhabbo.catalog.CatalogProduct;
import org.openhabbo.core.Logger;
import org.openhabbo.net.messages.ClientMessage;
import org.openhabbo.net.messages.ServerMessage;
import org.openhabbo.sessions.EServerMessage;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionMessageHandler;
import org.openhabbo.users.UserManager;
import org.openhabbo.util.SecurityUtil;

public class PurchaseGiftHandler implements SessionMessageHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		int pageId = request.readInt();
		int productId = request.readInt();
		String customData = request.readStr();
		String giftUsername = SecurityUtil.filterStr(request.readStr());
		String giftNote = SecurityUtil.filterStr(request.readStr(), false); // Don't filter linebreaks

		// Valid product?
		CatalogProduct product = Catalog.getInstance().getProduct(productId);
		if(product == null || product.page == null || product.pageId != pageId || !product.page.isVisible || product.page.minAccessLevel > session.getUser().role.level)
		{
			Logger.getInstance().warning(this, String.format("User #%d tried to buy purchase product #%d from the Catalog, which is not existent / not available to this user.", session.getUser().id, productId));
			return;
		}

		// Can user actually afford this?
		if(product.costCredits > session.getUser().credits || product.costPixels > session.getUser().pixels)
		{
			return;
		}

		// Receiver exists?
		int receiverId = UserManager.getInstance().getUserId(giftUsername);
		if(receiverId == 0)
		{
			response.init(EServerMessage.PurchaseGiftError);
			response.appendInt(1); // "User does not exist"
			response.appendStr(giftUsername);
			session.send(response);
			return;
		}

		// Issue the purchase
		if(!Catalog.getInstance().purchase(product, customData, session.getUser(), receiverId, true, giftNote))
		{
			session.sendNotif(String.format("Purchase of catalog product #%d failed. Please try again in a few minutes. If this keeps happening, please contact Support via the Yellow Questionmark on the left of your screen.", product.id));
			return;
		}

		// Payday!
		if(product.costCredits > 0)
		{
			session.getUser().credits -= product.costCredits;
			session.sendCreditBalance();
		}
		if(product.costPixels > 0)
		{
			session.getUser().pixels -= product.costPixels;
			session.sendPixelBalance();
		}
		session.getUser().saveCreditsAndPixels();

		// Purchase okay!
		session.sendNotif("Gift purchased and sent!");
		/*
		 * response.init(EServerMessage.PurchaseOK);
		 * response.appendInt(product.itemDefinition.id);
		 * response.appendStr(product.name);
		 * response.appendInt(product.costCredits);
		 * response.appendInt(0);
		 * response.appendInt(product.costPixels);
		 * response.appendInt(product.costQuestPoints);
		 * response.appendInt(1);
		 * response.appendChr(product.getType());
		 * response.appendInt(product.itemDefinition.id);
		 * response.appendStr("");
		 * response.appendInt(1);
		 * response.appendInt(-1);
		 * response.appendStr("");
		 * session.send(response);
		 */
	}
}
