package org.openhabbo.sessions.handlers;

import org.openhabbo.rooms.instances.RoomUser;
import org.openhabbo.sessions.EServerMessage;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionMessageHandler;
import org.openhabbo.sessions.messages.ClientMessage;
import org.openhabbo.sessions.messages.ServerMessage;

public class TakeRightsHandler implements SessionMessageHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Room owner?
		if(session.getRoomUser() == null && !session.getRoomUser().hasRoomOwner)
		{
			return;
		}

		// Parse all the user IDs
		int amount = request.readInt();
		for(int x = 0; x < amount; x++)
		{
			int userId = request.readInt();

			// Attempt removing the rights
			if(session.getRoomInstance().removeUserRights(userId))
			{
				// Notify issuer that this user's rights were removed successfully
				response.init(511);
				response.appendInt(session.getRoomInstance().getInfo().id);
				response.appendInt(userId);
				session.send(response);

				// User is in the room at this very moment?
				RoomUser user = session.getRoomInstance().getUserByUserId(userId);
				if(user != null)
				{
					// Update the user
					user.hasRoomRights = false;
					user.removeStatus("flatctrl");

					// Notify of 'lost room rights'
					response.init(EServerMessage.YouAreNotController);
					user.getSession().send(response);
				}
			}
		}
	}
}
