package org.openhabbo.sessions.messages;

import org.openhabbo.util.Base64Encoding;
import org.openhabbo.util.WireEncoding;

public final class ClientMessage
{
	private final static int ID_LENGTH = 2;

	public final int msgId;
	private final byte[] payload;
	private final int length;
	private int pointer;

	public ClientMessage(byte[] data, int length)
	{
		this.pointer = 0;
		this.length = length;
		if(data != null)
		{
			this.payload = data;
			this.msgId = this.readFixedInt();
		}
		else
		{
			this.payload = new byte[0];
			this.msgId = -1;
		}
	}

	public final void reset()
	{
		this.pointer = ClientMessage.ID_LENGTH;
	}

	public final void advance(int n)
	{
		this.pointer += n;
	}

	public final boolean contains(byte b)
	{
		for(int x = 0; x < this.payload.length; x++)
		{
			if(this.payload[x] == b)
			{
				return true;
			}
		}
		return false;
	}

	private final byte[] readBytes(int n)
	{
		byte[] data = new byte[n];
		for(int x = 0; x < n; x++, this.pointer++)
		{
			data[x] = this.payload[this.pointer];
		}
		return data;
	}

	public final boolean readFixedBool()
	{
		return(this.payload[this.pointer++] == 'A');
	}

	public final int readFixedInt()
	{
		return Base64Encoding.decodeInt(new byte[] { this.payload[this.pointer++], this.payload[this.pointer++] });
	}

	public final String readStr()
	{
		return new String(this.readBytes(this.readFixedInt()));
	}

	public final int readParseFixedInt()
	{
		try
		{
			return Integer.parseInt(this.readStr());
		}
		catch(NumberFormatException ex)
		{
			return 0;
		}
	}

	public final Boolean readBool()
	{
		return(this.remaining() > 0 && this.payload[this.pointer++] == WireEncoding.POSITIVE);
	}

	public final int readInt()
	{
		byte[] bytes = new byte[this.remaining() < WireEncoding.MAX_INTEGER_BYTE_AMOUNT ? this.remaining() : WireEncoding.MAX_INTEGER_BYTE_AMOUNT];
		for(int x = 0, y = this.pointer; x < bytes.length; x++, y++)
		{
			bytes[x] = this.payload[y];
		}
		int[] ret = WireEncoding.decodeInt(bytes);
		this.pointer += ret[1];
		return ret[0];
	}

	public final String getBody()
	{
		return new String(this.payload);
	}

	public final int length()
	{
		return this.length;
	}

	public final int remaining()
	{
		return this.length - this.pointer;
	}

	public final String header()
	{
		return new String(Base64Encoding.encodeInt(this.msgId, ClientMessage.ID_LENGTH));
	}

	@Override
	public final String toString()
	{
		return this.header() + new String(this.payload, ClientMessage.ID_LENGTH, this.length - ClientMessage.ID_LENGTH);
	}
}
