package org.openhabbo.storage;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.openhabbo.core.Logger;
import org.openhabbo.core.PropertiesBox;

public class SQLDatabase
{
	private DatabaseEndpoint endpoint;
	private DatabaseConnectionManager connMgr;
	private DatabaseQueryQueue queryQueue;

	public SQLDatabase(PropertiesBox props)
	{
		this.endpoint = new DatabaseEndpoint(props.get("db.uid"), props.get("db.pwd"), props.get("db.url"));
		this.connMgr = new DatabaseConnectionManager(this.endpoint, props.get("db.driver"), props.getInt("db.maxconnections", 0));
		this.queryQueue = new DatabaseQueryQueue(this);
	}

	public boolean test()
	{
		return this.connMgr.test();
	}

	public void onError(SQLException ex)
	{
		Logger.getInstance().error("SQLDatabase", "Incoming error!", ex);
	}

	public void execute(String sql)
	{
		this.queryQueue.enqueue(sql);
	}

	public int executeReturnKey(String sql)
	{
		DatabaseConnection conn = this.connMgr.getConnection();
		if(conn != null)
		{
			try
			{
				Statement st = conn.getInnerConnection().createStatement();
				st.execute(sql, Statement.RETURN_GENERATED_KEYS);
				ResultSet keys = st.getGeneratedKeys();
				if(keys.next())
				{
					return keys.getInt(1);
				}
				else
				{
					return 0;
				}
			}
			catch(SQLException ex)
			{
				this.onError(ex);
			}
			finally
			{
				this.connMgr.releaseConnection(conn);
			}
		}
		return -1;
	}

	public DatabaseResult query(String sql)
	{
		DatabaseConnection conn = this.connMgr.getConnection();
		if(conn != null)
		{
			try
			{
				ResultSet rs = conn.getInnerConnection().createStatement().executeQuery(sql);
				return new DatabaseResult(conn, rs);
			}
			catch(SQLException ex)
			{
				this.onError(ex);
				this.connMgr.releaseConnection(conn);
			}
		}
		return DatabaseResult.INVALID_RESULT;
	}

	public String queryStr(String sql)
	{
		DatabaseConnection conn = this.connMgr.getConnection();
		if(conn != null)
		{
			try
			{
				ResultSet rs = conn.getInnerConnection().createStatement().executeQuery(sql);
				if(rs.next())
				{
					return rs.getString(1);
				}
			}
			catch(SQLException ex)
			{
				this.onError(ex);
			}
			finally
			{
				this.connMgr.releaseConnection(conn);
			}
		}

		// No result
		return null;
	}

	public DatabaseEndpoint getEndpoint()
	{
		return this.endpoint;
	}

	public DatabaseConnectionManager getConnMgr()
	{
		return this.connMgr;
	}

	public DatabaseQueryQueue getQueryQueue()
	{
		return this.queryQueue;
	}
}
