package org.openhabbo.users;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.openhabbo.OpenHABBO;
import org.openhabbo.access.AccessControl;
import org.openhabbo.access.UserRole;
import org.openhabbo.storage.DatabaseResult;
import org.openhabbo.storage.SQLHelper;

public final class User
{
	// Core
	public int id;
	public String username;
	public String realName;
	public UserRole role;

	// Statistics
	public long registeredTime;
	public boolean onlineNow;
	public long lastOnlineTime;

	// Appearance
	public String figure;
	public char gender;
	public String motto;

	// Valueables
	public int credits;
	public int pixels;

	// Preferences
	public int homeRoomId;

	// Respecting
	public int respect;
	public int dailyRespectPoints;
	public int dailyPetRespectPoints;

	// Aggregrated data
	public List<String> rights;
	public byte specialRightsLevel;
	public Map<Integer, Integer> achievements;
	public List<Integer> ratedRooms = new ArrayList<Integer>(0);
	
	public void saveAppearance()
	{
		OpenHABBO.getInstance().getDb().execute("UPDATE users SET figure='" + SQLHelper.escape(this.figure) + "',gender='" + this.gender + "',motto='" + SQLHelper.escape(this.motto) + "' WHERE id=" + this.id);
	}

	public void saveCreditsAndPixels()
	{
		OpenHABBO.getInstance().getDb().execute("UPDATE users SET credits=" + this.credits + ",pixels=" + this.pixels + " WHERE id=" + this.id);
	}

	public void reloadCreditsAndPixels()
	{
		DatabaseResult db = OpenHABBO.getInstance().getDb().query("SELECT credits,pixels FROM users WHERE id=" + this.id);
		if(db.row())
		{
			this.credits = db.getInt("credits");
			this.pixels = db.getInt("pixels");
		}
		db.release();
	}

	public void saveMisc()
	{
		OpenHABBO.getInstance().getDb().execute("UPDATE users SET homeroom_id=" + this.homeRoomId + ",respect=" + this.respect + ",daily_respect_points=" + this.dailyRespectPoints + ",daily_petrespect_points=" + this.dailyPetRespectPoints + " WHERE id=" + this.id);
	}

	public void updateLastOnline(boolean onlineNow)
	{
		this.onlineNow = onlineNow;
		this.lastOnlineTime = System.currentTimeMillis();
		OpenHABBO.getInstance().getDb().execute("UPDATE users SET online=" + (onlineNow ? '1' : '0') + ",lastonline='" + SQLHelper.formatDate(this.lastOnlineTime) + "' WHERE id=" + this.id);
	}

	public static User parse(DatabaseResult row)
	{
		User user = new User();
		user.id = row.getInt("id");
		user.username = row.getString("username");
		user.realName = row.getString("realname");
		user.role = AccessControl.getInstance().getUserRole(row.getInt("role_id"));
		user.registeredTime = row.getTime("registered");
		user.lastOnlineTime = row.getTime("lastonline");
		user.onlineNow = row.getBoolean("online");
		user.figure = row.getString("figure");
		user.gender = row.getString("gender").equals("F") ? 'F' : 'M';
		user.motto = row.getString("motto");
		user.credits = row.getInt("credits");
		user.pixels = row.getInt("pixels");
		user.homeRoomId = row.getInt("homeroom_id");
		user.respect = row.getInt("respect");
		user.dailyRespectPoints = row.getInt("daily_respect_points");
		user.dailyPetRespectPoints = row.getInt("daily_petrespect_points");
		return user;
	}
}
