package mod_corechatcmds;

import java.lang.management.ManagementFactory;
import java.lang.management.OperatingSystemMXBean;
import java.lang.management.ThreadMXBean;

import org.openhabbo.OpenHABBO;
import org.openhabbo.chatcommands.ChatCommandHandler;
import org.openhabbo.rooms.RoomManager;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionManager;
import org.openhabbo.util.GameError;

public class StatsCommandHandler implements ChatCommandHandler
{
	@Override
	public String getDescription()
	{
		return "Shows some server stats";
	}

	@Override
	public String getRequiredRight()
	{
		return "default";
	}
	
	@Override
	public String[] getSyntax()
	{
		return null;
	}

	@Override
	public void handle(Session session, String[] args) throws GameError
	{
		// Gather the required information
		Runtime runtime = Runtime.getRuntime();
		OperatingSystemMXBean os = ManagementFactory.getOperatingSystemMXBean();
		ThreadMXBean threads = ManagementFactory.getThreadMXBean();
		OpenHABBO instance = OpenHABBO.getInstance();

		// Calculate uptime
		long uptime = (System.currentTimeMillis() - instance.getStartTime());
		long days = (uptime / (1000 * 60 * 60 * 24));
		long hours = (uptime - days * (1000 * 60 * 60 * 24)) / (1000 * 60 * 60);
		long minutes = (uptime - days * (1000 * 60 * 60 * 24) - hours * (1000 * 60 * 60)) / (1000 * 60);
		long seconds = (uptime - days * (1000 * 60 * 60 * 24) - hours * (1000 * 60 * 60) - minutes * (1000 * 60)) / (1000);

		// Build notification
		StringBuilder notif = new StringBuilder();

		// Uptime
		notif.append(instance.versionString());
		notif.append("\r");
		notif.append(String.format("Server uptime is %d day(s), %d hour(s), %d minute(s) and %d second(s).\r", days, hours, minutes, seconds));
		notif.append(String.format("There are %d user(s) online, and your session ID is #%d.\r", SessionManager.getInstance().authedSessions(), session.id));
		notif.append(String.format("There are %d room instance(s) loaded.\r\r", RoomManager.getInstance().getAllRoomInstances().size()));

		// Environment
		notif.append(String.format("OS: %s %s (%s)\r", os.getName(), os.getArch(), os.getVersion()));
		notif.append(String.format("CPU cores: %d (active server threads: %d)\r", runtime.availableProcessors(), (threads.getThreadCount() - threads.getDaemonThreadCount())));
		notif.append(String.format("JVM memory usage: %d KB\r\r", (runtime.totalMemory() / 1024)));

		// Footer
		notif.append("Type :about for more server information.");

		// Send it
		session.sendSystemNotif(notif.toString());
	}
}
