package mod_effects;

import org.openhabbo.net.messages.ClientMessage;
import org.openhabbo.net.messages.ServerMessage;
import org.openhabbo.rooms.instances.RoomUser;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionMessageHandler;

public class ApplyEffectHandler implements SessionMessageHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Get effect inventory
		AvatarEffectInventory effects = session.get("effects");
		if(effects == null)
		{
			return;
		}

		// Get effect from inventory
		int effectId = request.readInt();
		AvatarEffect effect = effects.getEffect(effectId);

		// User has this effect enabled? (-1 is no effect)
		RoomUser user = session.getRoomUser();
		if(effectId == -1)
		{
			user.unset("effect");
		}
		else
		{
			if(effect != null && effect.enabled)
			{
				user.set("effect", effectId);
			}
			else
			{
				// Invalid effect
				return;
			}
		}
		effects.setCurrentEffect(effectId);

		// Notify the room (-1 = remove effect)
		response.init(EEffectMessage.AvatarEffect);
		response.appendInt(user.virtualId);
		response.appendInt(effectId);
		session.getRoomInstance().broadcast(response);
	}
}
