package mod_effects;

import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

import org.openhabbo.OpenHABBO;
import org.openhabbo.rooms.instances.RoomUser;
import org.openhabbo.sessions.messages.ServerMessage;
import org.openhabbo.storage.DatabaseResult;

public class AvatarEffectInventory
{
	private final int userId;
	private int currentEffect;
	private List<AvatarEffect> effects;

	public AvatarEffectInventory(int userId)
	{
		this.userId = userId;
		this.currentEffect = -1;
		this.effects = new CopyOnWriteArrayList<AvatarEffect>();
	}

	public void clear()
	{
		this.effects.clear();
	}

	public void reload()
	{
		// Clear old effects
		this.clear();

		// Load new ones from database
		DatabaseResult result = OpenHABBO.getInstance().getDb().query("SELECT * FROM user_effects WHERE user_id=" + this.userId);
		while(result.row())
		{
			AvatarEffect effect = AvatarEffect.parse(result);
			if(effect.hasExpired())
			{
				effect.delete();
			}
			else
			{
				this.effects.add(effect);
			}
		}
		result.release();
	}

	public void addEffect(int effectId, int totalDuration)
	{
		// Create effect
		AvatarEffect effect = new AvatarEffect();
		effect.userId = this.userId;
		effect.effectId = effectId;
		effect.totalDuration = totalDuration;
		effect.insert();

		// Add to list
		this.effects.add(effect);
	}

	public int getCurrentEffect()
	{
		return this.currentEffect;
	}

	public void setCurrentEffect(RoomUser user, int effectId)
	{
		// Set current effect serverside
		this.currentEffect = effectId;
		if(effectId == AvatarEffect.NO_EFFECT)
		{
			user.unset("effect");
		}
		else
		{
			user.set("effect", effectId);
		}

		// Notify the room (-1 = remove effect)
		ServerMessage msg = new ServerMessage(EEffectMessage.AvatarEffect);
		msg.appendInt(user.virtualId);
		msg.appendInt(effectId);
		user.room.broadcast(msg);
	}

	public void checkAndStopExpiredEffects(RoomUser user)
	{
		if(!this.effects.isEmpty())
		{
			for(AvatarEffect effect : this.effects)
			{
				if(effect.hasExpired())
				{
					// Delete data
					effect.delete();

					// Notify the user that the effect expired
					ServerMessage msg = new ServerMessage(EEffectMessage.AvatarEffectExpired);
					msg.appendInt(effect.effectId);
					user.getSession().send(msg);

					// Current effect? Disable it!
					if(effect.effectId == this.currentEffect)
					{
						this.setCurrentEffect(user, -1);
					}
				}
			}
		}
	}

	public AvatarEffect getEffect(int effectId)
	{
		for(AvatarEffect effect : this.effects)
		{
			if(effect.effectId == effectId && !effect.hasExpired())
			{
				return effect;
			}
		}
		return null;
	}

	public List<AvatarEffect> getEffects()
	{
		return this.effects;
	}
}
