package mod_messenger;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.openhabbo.OpenHABBO;
import org.openhabbo.core.Logger;
import org.openhabbo.storage.DatabaseResult;
import org.openhabbo.storage.SQLHelper;

public final class InstantMessengerService
{
	private Map<Integer, MessengerBuddy> register;

	private InstantMessengerService()
	{
		this.register = new HashMap<Integer, MessengerBuddy>();
	}

	public void clear()
	{
		synchronized(register)
		{
			register.clear();
		}
	}

	public MessengerBuddy getBuddy(int userId, boolean attemptDbLoad)
	{
		// Lookup the register
		MessengerBuddy buddy = this.register.get(userId);

		// Not found? Attempt load from database, then add to register
		if(buddy == null && attemptDbLoad)
		{
			DatabaseResult result = OpenHABBO.getInstance().getDb().query("SELECT id,username,figure,motto,lastonline,im_blockrequests FROM users WHERE id=" + userId);
			if(result.row())
			{
				buddy = MessengerBuddy.parse(result);
				synchronized(this.register)
				{
					this.register.put(buddy.userId, buddy);
				}
			}
			result.release();
		}

		// Update last ref time
		if(buddy != null)
		{
			buddy.updateRefLastAccessTime();
		}
		return buddy;
	}

	public Map<Integer, MessengerBuddy> getBuddyListForUser(int userId)
	{
		HashMap<Integer, MessengerBuddy> buddies = new HashMap<Integer, MessengerBuddy>();

		DatabaseResult result = OpenHABBO.getInstance().getDb().query("SELECT u.id,u.username,u.figure,u.motto,u.lastonline,u.im_blockrequests FROM users u INNER JOIN user_imbuddies rel ON u.id=rel.user_id WHERE rel.user2_id=" + userId + " AND rel.accepted=1 UNION ALL SELECT u.id,u.username,u.figure,u.motto,u.lastonline,u.im_blockrequests FROM users u INNER JOIN user_imbuddies rel ON u.id=rel.user2_id WHERE rel.user_id=" + userId + " AND rel.accepted=1");
		while(result.row())
		{
			// First lookup the register...
			int buddyUserId = result.getInt("id");
			MessengerBuddy buddy = this.getBuddy(buddyUserId, false);

			// Not in register?
			if(buddy == null)
			{
				// Accept the database data and add to the register
				buddy = MessengerBuddy.parse(result);
				synchronized(this.register)
				{
					this.register.put(buddy.userId, buddy);
				}
			}

			// Add to the buddylist
			buddies.put(buddy.userId, buddy);
		}
		result.release();

		return buddies;
	}

	public Map<Integer, String> getBuddyRequestsForUser(int userId)
	{
		Map<Integer, String> requests = new HashMap<Integer, String>();

		DatabaseResult result = OpenHABBO.getInstance().getDb().query("SELECT u.id,u.username FROM users u JOIN user_imbuddies rel ON u.id=rel.user_id WHERE rel.user2_id=" + userId + " AND rel.accepted=0");
		while(result.row())
		{
			requests.put(result.getInt("id"), result.getString("username"));
		}
		result.release();

		return requests;
	}

	public List<MessengerBuddy> getUserSearchResults(String criteria)
	{
		List<MessengerBuddy> buddies = new ArrayList<MessengerBuddy>();

		DatabaseResult result = OpenHABBO.getInstance().getDb().query("SELECT id,username,figure,motto,lastonline,im_blockrequests FROM users WHERE username LIKE '" + SQLHelper.escape(criteria) + "%' LIMIT 30;");
		while(result.row())
		{
			// Lookup register first, optionally accept database data
			MessengerBuddy buddy = this.getBuddy(result.getInt("id"), false);
			if(buddy == null)
			{
				buddy = MessengerBuddy.parse(result);
			}
			buddies.add(buddy);
		}
		result.release();

		return buddies;
	}

	public MessengerBuddy getBuddyInfoByUsername(String username)
	{
		// Fetch user info by username, or null if user does not exist
		DatabaseResult result = OpenHABBO.getInstance().getDb().query("SELECT id,username,figure,motto,lastonline,im_blockrequests FROM users WHERE username='" + SQLHelper.escape(username) + "'");
		MessengerBuddy buddy = result.row() ? MessengerBuddy.parse(result) : null;
		result.release();

		return buddy;
	}

	public void createBuddyRequest(int fromUserId, String fromUsername, int toUserId)
	{
		// Delete old requests and create new unaccepted relation in database
		OpenHABBO.getInstance().getDb().execute("DELETE FROM user_imbuddies WHERE ((user_id=" + fromUserId + " AND user2_id=" + toUserId + ") OR (user_id=" + toUserId + " AND user2_id=" + fromUserId + ")) AND accepted=0 LIMIT 1");
		OpenHABBO.getInstance().getDb().execute("INSERT INTO user_imbuddies(user_id,user2_id) VALUES (" + fromUserId + "," + toUserId + ")");

		// Notify buddy if online
		MessengerBuddy receiver = this.getBuddy(toUserId, true);
		if(receiver != null && receiver.online())
		{
			receiver.messenger.addRequest(fromUserId, fromUsername);
		}
	}

	public void acceptBuddyRequest(int toUserId, int fromUserId)
	{
		// Update the relation in the database
		OpenHABBO.getInstance().getDb().execute("UPDATE user_imbuddies SET accepted=1 WHERE user_id=" + fromUserId + " AND user2_id=" + toUserId);

		MessengerBuddy buddy;

		// to-user online?
		if((buddy = this.getBuddy(toUserId, false)) != null)
		{
			// Add the from-user to it's buddy list
			buddy.messenger.addBuddy(getBuddy(fromUserId, true));
		}

		// from-user online?
		if((buddy = this.getBuddy(fromUserId, false)) != null)
		{
			// Add the to-user to it's buddylist
			buddy.messenger.addBuddy(getBuddy(toUserId, true));
		}
	}

	public void deleteRelation(int userId1, int userId2)
	{
		OpenHABBO.getInstance().getDb().execute("DELETE FROM user_imbuddies WHERE (user_id=" + userId1 + " AND user2_id=" + userId2 + ") OR (user_id=" + userId2 + " AND user2_id=" + userId1 + ") LIMIT 1");
	}

	public void cleanup()
	{
		// Remove offline buddies that haven't been accessed recently
		int removed = 0;
		synchronized(this.register)
		{
			Iterator<MessengerBuddy> it = this.register.values().iterator();
			while(it.hasNext())
			{
				if(it.next().requiresUnload())
				{
					it.remove();
					removed++;
				}
			}
		}

		// Anything removed?
		if(removed > 0)
		{
			Logger.getInstance().debug("InstantMessengerService", String.format("Cleaned %d buddies.", removed));
		}
	}

	/* Begin singleton */
	private final static InstantMessengerService instance = new InstantMessengerService();

	public final static InstantMessengerService getInstance()
	{
		return instance;
	}
	/* End singleton */
}
