package mod_moderation;

import java.util.ArrayList;
import java.util.List;

import org.openhabbo.OpenHABBO;
import org.openhabbo.access.AccessControl;
import org.openhabbo.access.UserRole;
import org.openhabbo.rooms.RoomManager;
import org.openhabbo.rooms.instances.RoomInstance;
import org.openhabbo.sessions.EServerMessage;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionManager;
import org.openhabbo.sessions.messages.ServerMessage;
import org.openhabbo.storage.DatabaseResult;
import org.openhabbo.storage.SQLHelper;

public final class ModerationCenter
{
	public boolean issueUserAlert(int issuerId, String username, String message, String extraInfo)
	{
		Session sess = SessionManager.getInstance().getSessionByUsername(username);
		return this.issueUserAlert(issuerId, sess, message, extraInfo);
	}
	
	public boolean issueUserAlert(int issuerId, int userId, String message, String extraInfo)
	{
		Session sess = SessionManager.getInstance().getSessionByUserId(userId);
		return this.issueUserAlert(issuerId, sess, message, extraInfo);
	}
	
	private boolean issueUserAlert(int issuerId, Session session, String message, String extraInfo)
	{
		if(session == null)
		{
			return false;
		}
		else
		{
			session.sendNotif(message);
			AccessControl.getInstance().logUserAction(issuerId, session.getUser().id, "alert", message, extraInfo);
			return true;
		}
	}
	
	public boolean issueUserKick(int issuerId, UserRole issuerRole, String username, String message, String extraInfo)
	{
		Session sess = SessionManager.getInstance().getSessionByUsername(username);
		return this.issueUserKick(issuerId, issuerRole, sess, message, extraInfo);
	}
	
	public boolean issueUserKick(int issuerId, UserRole issuerRole, int userId, String message, String extraInfo)
	{
		Session sess = SessionManager.getInstance().getSessionByUserId(userId);
		return this.issueUserKick(issuerId, issuerRole, sess, message, extraInfo);
	}
	
	private boolean issueUserKick(int issuerId, UserRole issuerRole, Session session, String message, String extraInfo)
	{
		if(session == null || !session.inRoom())
		{
			return false;
		}
		else if((issuerRole != null) && issuerRole.level < session.getUser().role.level)
		{
			return false;
		}
		else
		{
			session.kickFromRoom(message);
			AccessControl.getInstance().logUserAction(issuerId, session.getUser().id, "kick", message, extraInfo);
			return true;
		}
	}

	public boolean issueRoomAlert(int issuerId, int roomId, String message, String extraInfo)
	{
		RoomInstance room = RoomManager.getInstance().getRoomInstance(roomId, false);
		if(room == null)
		{
			return false;
		}
		else
		{
			ServerMessage msg = new ServerMessage(EServerMessage.Notif);
			msg.appendStr(message);
			room.broadcast(msg);

			AccessControl.getInstance().logUserAction(issuerId, roomId, "roomalert", message, extraInfo);
			return true;
		}
	}

	public boolean issueRoomKick(int issuerId, int roomId, String message, String extraInfo)
	{
		RoomInstance room = RoomManager.getInstance().getRoomInstance(roomId, false);
		if(room == null)
		{
			return false;
		}
		else
		{
			room.moderationKick(issuerId, message);
			AccessControl.getInstance().logUserAction(issuerId, roomId, "roomkick", message, extraInfo);
			return true;
		}
	}

	public boolean issueUserBan(int issuerId, int userId, String message, int hours, String extraInfo)
	{
		return false;
	}

	public boolean issueIpBan(int issuerId, String ip, String message, int hours, String extraInfo)
	{
		return false;
	}

	public List<ModerationBan> getBansForIp(String ip, boolean includeExpired)
	{
		List<ModerationBan> bans = new ArrayList<ModerationBan>(0);
		DatabaseResult result = OpenHABBO.getInstance().getDb().query("SELECT * FROM moderation_bans WHERE ip='" + SQLHelper.escape(ip) + "'");
		while(result.row())
		{
			ModerationBan ban = ModerationBan.parse(result);
			if(includeExpired || !ban.hasExpired())
			{
				bans.add(ban);
			}
		}
		return bans;
	}

	public List<ModerationBan> getBansForUserId(int userId, boolean includeExpired)
	{
		List<ModerationBan> bans = new ArrayList<ModerationBan>(0);
		DatabaseResult result = OpenHABBO.getInstance().getDb().query("SELECT * FROM moderation_bans WHERE user_id=" + userId);
		while(result.row())
		{
			ModerationBan ban = ModerationBan.parse(result);
			if(includeExpired || !ban.hasExpired())
			{
				bans.add(ban);
			}
		}
		result.release();
		return bans;
	}
	
	private final static ModerationCenter instance = new ModerationCenter();
	
	public final static ModerationCenter getInstance()
	{
		return instance;
	}
}
