package mod_moderation;

import java.util.ArrayList;
import java.util.List;

import org.openhabbo.OpenHABBO;
import org.openhabbo.access.AccessControl;
import org.openhabbo.rooms.RoomManager;
import org.openhabbo.rooms.instances.RoomInstance;
import org.openhabbo.sessions.EServerMessage;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionManager;
import org.openhabbo.sessions.messages.ServerMessage;
import org.openhabbo.storage.DatabaseResult;
import org.openhabbo.storage.SQLHelper;
import org.openhabbo.users.User;

public final class ModerationCenter
{
	public UserModerationHistory getUserModerationHistory(int userId)
	{
		// If not exists yet, create it
		// Parse and return
		return null;
	}
	
	public void issueUserAlert(int issuerId, String username, String message, boolean isCaution) throws ModerationActionError
	{
		Session sess = SessionManager.getInstance().getSessionByUsername(username);
		this.issueUserAlert(issuerId, sess, message, isCaution);
	}
	
	public void issueUserAlert(int issuerId, int userId, String message, boolean isCaution) throws ModerationActionError
	{
		Session sess = SessionManager.getInstance().getSessionByUserId(userId);
		this.issueUserAlert(issuerId, sess, message, isCaution);
	}
	
	private void issueUserAlert(int issuerId, Session session, String message, boolean isCaution) throws ModerationActionError
	{
		if(session == null)
		{
			throw new ModerationActionError("user not online");
		}
		else if(message.isEmpty())
		{
			throw new ModerationActionError("message cannot be blank");
		}
		else
		{
			session.sendNotif(message);
			AccessControl.getInstance().logUserAction(issuerId, session.getUser().id, "alert", message);
		}
	}
	
	public void issueUserKick(int issuerId, User issuerInfo, String username, String message) throws ModerationActionError
	{
		Session sess = SessionManager.getInstance().getSessionByUsername(username);
		this.issueUserKick(issuerId, issuerInfo, sess, message);
	}
	
	public void issueUserKick(int issuerId, User issuerInfo, int userId, String message) throws ModerationActionError
	{
		Session sess = SessionManager.getInstance().getSessionByUserId(userId);
		this.issueUserKick(issuerId, issuerInfo, sess, message);
	}
	
	private void issueUserKick(int issuerId, User issuerInfo, Session session, String message) throws ModerationActionError
	{
		if(session == null || !session.inRoom())
		{
			throw new ModerationActionError("user not online / not in room");
		}
		else if((issuerInfo != null) && issuerInfo.role.level < session.getUser().role.level)
		{
			throw new ModerationActionError("insufficient permissions");
		}
		else
		{
			if(message != null && message.isEmpty())
			{
				message = null;
			}
			session.kickFromRoom(message);
			AccessControl.getInstance().logUserAction(issuerId, session.getUser().id, "kick", message);
		}
	}

	public void issueRoomAlert(int issuerId, int roomId, String message) throws ModerationActionError
	{
		RoomInstance room = RoomManager.getInstance().getRoomInstance(roomId, false);
		if(room == null)
		{
			throw new ModerationActionError("room not loaded");
		}
		else if(message.isEmpty())
		{
			throw new ModerationActionError("message cannot be blank");
		}
		else
		{
			ServerMessage msg = new ServerMessage(EServerMessage.Notif);
			msg.appendStr(message);
			room.broadcast(msg);

			AccessControl.getInstance().logUserAction(issuerId, roomId, "roomalert", message);
		}
	}

	public void issueRoomKick(int issuerId, int roomId, String message) throws ModerationActionError
	{
		RoomInstance room = RoomManager.getInstance().getRoomInstance(roomId, false);
		if(room == null)
		{
			throw new ModerationActionError("room not loaded");
		}
		else
		{
			if(message != null && message.isEmpty())
			{
				message = null;
			}
			room.moderationKick(issuerId, message);
			AccessControl.getInstance().logUserAction(issuerId, roomId, "roomkick", message);
		}
	}

	public void issueUserBan(int issuerId, int userId, String message, int hours)
	{
		
	}

	public void issueIpBan(int issuerId, String ip, String message, int hours)
	{
		
	}

	public List<ModerationBan> getActiveBansForIp(String ip)
	{
		List<ModerationBan> bans = new ArrayList<ModerationBan>(0);
		DatabaseResult result = OpenHABBO.getInstance().getDb().query("SELECT * FROM moderation_bans WHERE ip='" + SQLHelper.escape(ip) + "'");
		while(result.row())
		{
			ModerationBan ban = ModerationBan.parse(result);
			if(!ban.hasExpired())
			{
				bans.add(ban);
			}
		}
		return bans;
	}

	public List<ModerationBan> getActiveBansForUserId(int userId)
	{
		List<ModerationBan> bans = new ArrayList<ModerationBan>(0);
		DatabaseResult result = OpenHABBO.getInstance().getDb().query("SELECT * FROM moderation_bans WHERE user_id=" + userId);
		while(result.row())
		{
			ModerationBan ban = ModerationBan.parse(result);
			if(!ban.hasExpired())
			{
				bans.add(ban);
			}
		}
		result.release();
		return bans;
	}
	
	private final static ModerationCenter instance = new ModerationCenter();
	
	public final static ModerationCenter getInstance()
	{
		return instance;
	}
}
