package mod_teleporter;

import java.util.List;

import org.openhabbo.items.UsableItem;
import org.openhabbo.items.Item;
import org.openhabbo.items.ItemManager;
import org.openhabbo.rooms.instances.RoomTile;
import org.openhabbo.rooms.instances.RoomUser;
import org.openhabbo.sessions.Session;

public final class TeleporterItem extends UsableItem
{
	private boolean isWaiting = false;
	
	@Override
	public void onPurchase(List<Item> shipping)
	{
		// Store this teleporter, get unique ID
		ItemManager.getInstance().storeItem(this);
		
		// Create new teleporter, link to this teleporter ID
		Item other = this.definition.newInstance();
		other.ownerId = this.ownerId;
		other.customData = Integer.toString(this.id);
		ItemManager.getInstance().storeItem(other);

		// Update this teleporter with ID of other teleporter
		this.customData = Integer.toString(other.id);
		this.update();

		// Add both teleporters to shipping
		shipping.add(this);
		shipping.add(other);
	}

	@Override
	public void onTrigger(Session session, int instruction)
	{
		// Can go in teleporters?
		RoomUser user = session.getRoomUser();
		if(!user.canMove())
		{
			return;
		}

		// Infront of teleporter? Attempt to move to the front
		RoomTile frontTile = this.getTileInFront();
		if(!user.at(frontTile))
		{
			user.startMovingTo(frontTile.x, frontTile.y);
			return;
		}

		// Not in use yet?
		if(!this.inUse())
		{
			// Tadaa
			this.setUser(user);

			// Lock own movement, move into the teleporter
			user.lockMovement();
			user.overrideNextTile = true;
			user.startMovingTo(this.x, this.y);

			// Wait...
			this.setTimeout(3);
		}
	}

	@Override
	protected void onTimeout()
	{
		// User inside?
		if(this.inUse())
		{
			// Get user
			RoomUser user = this.getUser();
			
			// Get tele info
			int otherId = Integer.parseInt(this.customData);
			int otherRoomId = mod_teleporter.getTeleporterRoomId(otherId);
			
			// Waiting for teleport start?
			if(this.isWaiting)
			{
				// Same room?
				if(otherRoomId == this.roomId)
				{
					// Warp to the other tele
					TeleporterItem other = (TeleporterItem)this.room.getItem(otherId);
					user.getRoom().getMap().avatars[user.x][user.y] = false;
					user.x = other.x;
					user.y = other.y;
					user.z = other.z;
					user.getRoom().getMap().avatars[user.x][user.y] = true;
					user.headRotation = user.bodyRotation = other.rotation;
					user.requiresUpdate = true;
					
					// Open the door in the other tele
					((TeleporterItem)other).openDoorTemporarily();
					
					// Get out!
					user.unlockMovement();
					user.requestNewTrip(other.getTileInFront());
				}
				else
				{
					//user.getSession().set("teleporterId", otherId);
					//user.getSession().tryGetRoomAccess(otherRoomId, "");
					
					// Open the door...
					user.getSession().sendNotif("Cross-room teleporting is not implemented in this build of OpenHABBO.");
					this.kickUserOut();
				}
				
				// Waiting done! Done with this teleporter
				this.isWaiting = false;
				this.clearUser();
			}
			
			// User just entered teleporter?
			else if(user.at(this.getTile()))
			{
				// Linked teleporter in room?
				if(otherRoomId == 0)
				{
					user.getSession().sendNotif("This teleporter is currently not linked.");
					this.kickUserOut();
				}
				else
				{
					// Flash this teleporter, wait 2 pulses before the actual teleport
					this.isWaiting = true;
					this.setTemporaryData("2");
					this.setTimeout(2);
				}
			}
		}
	}

	private void openDoorTemporarily()
	{
		this.setTemporaryData("1");
		this.setTimeout(1);
	}

	private void kickUserOut()
	{
		RoomUser user = this.getUser();
		if(user != null)
		{
			// Open the door...
			this.clearUser();
			this.openDoorTemporarily();

			// Start move to tile infront
			user.unlockMovement();
			user.requestNewTrip(this.getTileInFront());
		}
	}
}
