package mod_teleporter;

import org.openhabbo.items.InteractableItem;
import org.openhabbo.items.Item;
import org.openhabbo.items.ItemManager;
import org.openhabbo.rooms.instances.RoomTile;
import org.openhabbo.rooms.instances.RoomUser;
import org.openhabbo.sessions.Session;

public class TeleporterItem extends InteractableItem
{
	private boolean isTeleporting = false;
	private boolean isDoorOpen = false;
	
	@Override
	public void onTrigger(Session session, int newState)
	{
		// Can go in teleporters?
		RoomUser user = session.getRoomUser();
		if(!user.canMove())
		{
			return;
		}
		
		// Infront of teleporter? Attempt to move to the front
		RoomTile frontTile = this.getTileInFront();
		if(!user.at(frontTile))
		{
			user.requestNewTrip(frontTile.x, frontTile.y);
			return;
		}
		
		// Not in use yet?
		if(!this.inUse())
		{
			// Tadaa
			this.setUser(user);

			// Lock own movement, move into the teleporter
			user.lockMovement();
			user.overrideNextTile = true;
			user.requestNewTrip(this.x, this.y);

			// Wait...
			this.setTimeout(3);
		}
	}
	
	@Override
	protected void onTimeout()
	{
		// In use?
		if(this.inUse())
		{
			// Get user
			RoomUser user = this.getUser();
			
			// Get tele info
			Item other = ItemManager.getInstance().getItem(Integer.parseInt(this.customData));
			
			// Teleporting?
			if(this.isTeleporting)
			{
				// Same room?
				if(other.roomId == this.roomId)
				{
					// Warp to the other tele
					user.x = other.x;
					user.y = other.y;
					user.headRotation = user.bodyRotation = other.rotation;
					user.requiresUpdate = true;
					
					// Open the door!
					this.isDoorOpen = true;
					other.setTemporaryState("1");
					
					// Get out!
					user.unlockMovement();
					user.requestNewTrip(other.getTileInFront());
				}
				else
				{
					user.getSession().set("teleporterId", other.id);
					user.getSession().goToRoom(other.roomId);
				}
				
				// Done with teleporting!
				this.clearUser();
				this.isTeleporting = false;
			}
			
			// Waiting for user to get in?
			else if(user.at(this.getTile()))
			{
				// Other tele exists, and in room?
				if(other != null && other.inRoom())
				{
					// Working... (2 pulses)
					this.isTeleporting = true;
					this.setTemporaryState("2");
					this.setTimeout(2);
				}
				else
				{
					// Open the door...
					this.isDoorOpen = true;
					this.setTemporaryState("1");
					this.setTimeout(1);
					
					// Start moving...
					user.unlockMovement();
					user.requestNewTrip(this.getTileInFront());
					
					return;
				}
			}
			
			// Open door to close?
			if(this.isDoorOpen)
			{
				this.clearUser();
			}
		}
	}
}
