package mod_badges;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import org.openhabbo.OpenHABBO;
import org.openhabbo.core.Logger;
import org.openhabbo.storage.DatabaseConnection;
import org.openhabbo.storage.DatabaseResult;

public final class BadgeInventory
{
	private int userId;
	private Map<String, Byte> badges;
	private Map<Byte, String> equipped;
	private boolean requiresSave;

	public BadgeInventory(int userId)
	{
		this.userId = userId;
		this.badges = new HashMap<String, Byte>(3);
		this.equipped = new HashMap<Byte, String>(3);
		this.requiresSave = false;
	}

	public final void clear()
	{
		this.badges.clear();
		this.equipped.clear();
	}
	
	public void reload()
	{
		// Clear badges + equipped maps
		this.clear();

		// Reload available badges from database
		DatabaseResult result = OpenHABBO.getInstance().getDb().query("SELECT badge,slot FROM user_badges WHERE user_id=" + this.userId);
		while(result.row())
		{
			this.badges.put(result.getString("badge"), result.getByte("slot"));
		}
		result.release();
		
		// Re-map the equipped list
		for(Entry<String, Byte> badge : this.badges.entrySet())
		{
			// If this badge is actually in a slot...
			if(badge.getValue() > 0)
			{
				this.equipped.put(badge.getValue(), badge.getKey());
			}
		}
	}
	
	public void clearEquipped()
	{
		for(Entry<String, Byte> badge : this.badges.entrySet())
		{
			badge.setValue((byte) 0);
		}
		this.equipped.clear();
	}
	
	public void equip(byte slot, String badge)
	{
		if(this.badges.containsKey(badge))
		{
			this.badges.put(badge, slot);
			this.equipped.put(slot, badge);
		}
	}

	public void requireSave()
	{
		this.requiresSave = true;
	}
	
	public final boolean requiresSave()
	{
		return this.requiresSave;
	}
	
	public void save()
	{
		// Get dedicated database connection
		DatabaseConnection conn = OpenHABBO.getInstance().getDb().getConnMgr().getConnection();
		if(conn == null)
		{
			return;
		}

		// Create PreparedStatement, batch everything and execute
		try
		{
			PreparedStatement query = conn.getInnerConnection().prepareStatement("UPDATE user_badges SET slot=? WHERE user_id=" + this.userId + " AND badge=?");
			for(Entry<String, Byte> badge : this.badges.entrySet())
			{
				query.setByte(1, badge.getValue());
				query.setString(2, badge.getKey());
				query.addBatch();
			}
			query.executeBatch();
			query.close();
		}
		catch(SQLException ex)
		{
			Logger.getInstance().error("User", "Error saving badges.", ex);
		}

		// Release the dedicated database connection
		OpenHABBO.getInstance().getDb().getConnMgr().releaseConnection(conn);
		
		// Saved!
		this.requiresSave = false;
	}
	
	public final Map<String, Byte> getBadges()
	{
		return this.badges;
	}
	
	public final Map<Byte, String> getEquipped()
	{
		return this.equipped;
	}
}
