package mod_pets;

import org.openhabbo.rooms.instances.RoomAvatar;
import org.openhabbo.rooms.instances.RoomUser;
import org.openhabbo.sessions.messages.ServerMessage;

public final class RoomPet extends RoomAvatar
{
	private Pet pet;

	public RoomPet(Pet pet)
	{
		this.pet = pet;
	}

	public Pet getInfo()
	{
		return this.pet;
	}

	public void syncAndSaveLocationInfo()
	{
		if(this.getRoom() != null)
		{
			this.pet.roomId = this.getRoom().getInfo().id;
			this.pet.x = this.x;
			this.pet.y = this.y;
			this.pet.z = this.z;
		}
		else
		{
			this.pet.roomId = 0;
			this.pet.x = 0;
			this.pet.y = 0;
			this.pet.z = 0;
		}
		this.pet.save();
	}

	@Override
	protected void onChatFromAvatar(RoomAvatar from, String text, boolean isShout)
	{
		// Only respond to users
		if(!(from instanceof RoomUser))
		{
			return;
		}

		// Only respond to owner
		RoomUser user = (RoomUser) from;
		if(user.getInfo().id != this.getInfo().ownerId)
		{
			return;
		}

		// Find out if text starts with pet name (case insensitive)
		boolean isCommand = text.toLowerCase().startsWith(this.getInfo().name.toLowerCase());

		// Pay attention...
		if(!isCommand)
		{
			// Rotate head
			this.lookTo(from.x, from.y, true);
		}
		else
		{
			// Rotate body too (if needed)
			this.lookTo(from.x, from.y, false);

			// Try for matches
			PetUtil.handlePetCommand(this, user, text);
			
			// La la la testing
			this.statuses.clear();
			if(text.contains("free"))
			{
				
			}
			else if(text.contains("sit"))
			{
				this.addStatus("sit", Float.toString(pet.z));
			}
			else if(text.contains("down"))
			{
				this.addStatus("lay", Float.toString(pet.z));
			}
			else if(text.contains("here"))
			{
				this.chat("Lol screw you mate! :)", false);
			}
			else if(text.contains("beg"))
			{
				
			}
			else if(text.contains("play dead"))
			{
				this.addStatus("ded", Float.toString(pet.z));
			}
			else if(text.contains("stay"))
			{
				
			}
		}
	}

	@Override
	public void serialize(ServerMessage msg)
	{
		msg.appendInt(this.pet.id);
		msg.appendStr(this.pet.name);
		msg.appendStr("");
		msg.appendStr(this.pet.getFigure());
		msg.appendInt(this.avatarId);
		msg.appendInt(this.x);
		msg.appendInt(this.y);
		msg.appendStr(Float.toString(this.z));
		msg.appendInt(4);
		msg.appendInt(2);
		msg.appendInt(0);
	}

	@Override
	public String getAvatarKey()
	{
		return "p" + this.pet.id;
	}
}
