package mod_roomdimmer;

import org.openhabbo.items.Item;
import org.openhabbo.sessions.Session;

public class RoomDimmerItem extends Item
{
	private final static char STORAGE_DELIMITER = '|';

	public boolean enabled;
	public int currentPreset;
	public RoomDimmerPreset[] presets;

	@Override
	public boolean onPlace(Session session)
	{
		if(this.room.has("roomDimmer"))
		{
			session.sendNotif("You can only place one roomdimmer in a room!");
			return false;
		}
		else
		{
			this.room.set("roomDimmer", this);
			return true;
		}
	}

	public String getCustomData()
	{
		StringBuilder sb = new StringBuilder();
		sb.append(this.enabled ? '1' : '0');
		sb.append(STORAGE_DELIMITER);
		sb.append(this.currentPreset);
		sb.append(STORAGE_DELIMITER);
		sb.append(this.presets.length);
		sb.append(STORAGE_DELIMITER);
		for(int x = 0; x < this.presets.length; x++)
		{
			sb.append(this.presets[x].color);
			sb.append(STORAGE_DELIMITER);
			sb.append(this.presets[x].intensity);
			sb.append(STORAGE_DELIMITER);
			sb.append(this.presets[x].backgroundOnly ? '1' : '0');
		}
		return sb.toString();
	}

	public static RoomDimmerItem parse(String customData)
	{
		String[] bits = customData.split(String.valueOf(STORAGE_DELIMITER));
		int n = 0;

		RoomDimmerItem data = new RoomDimmerItem();
		data.enabled = (bits[n++].equals("1"));
		data.currentPreset = Integer.parseInt(bits[n++]);
		data.presets = new RoomDimmerPreset[Integer.parseInt(bits[n++])];
		for(int x = 0; x < data.presets.length; x++)
		{
			data.presets[x] = new RoomDimmerPreset();
			data.presets[x].color = bits[n++];
			data.presets[x].intensity = Integer.parseInt(bits[n++]);
			data.presets[x].backgroundOnly = (bits[n++].equals("1"));
		}

		return data;
	}

	public static RoomDimmerItem getDefaultData()
	{
		RoomDimmerItem data = new RoomDimmerItem();
		data.enabled = false;
		data.currentPreset = 0;
		data.presets = new RoomDimmerPreset[3];
		for(int x = 0; x < data.presets.length; x++)
		{
			data.presets[x] = new RoomDimmerPreset();
			data.presets[x].color = RoomDimmerPreset.DEFAULT_COLOR;
			data.presets[x].intensity = 0;
			data.presets[x].backgroundOnly = false;
		}
		return data;
	}
}
