package org.openhabbo.core;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Date;
import java.text.SimpleDateFormat;

public class Logger
{
	public enum ELogLevel
	{
		Debug, Info, Warning, Error
	}

	private BufferedWriter writer;
	private SimpleDateFormat timeformat;
	private ELogLevel minImportance;

	public Logger(String name)
	{
		this.timeformat = new SimpleDateFormat("dd-MM-yyyy @ HH:mm:ss");
		this.minImportance = ELogLevel.Debug;

		String fileName = name + ".log";
		try
		{
			this.writer = new BufferedWriter(new FileWriter(fileName, true));
		}
		catch(IOException ex)
		{
			System.out.println("ERROR creating new FileWriter for filename \"" + fileName + "\"");
		}
	}

	public void setMinImportance(ELogLevel level)
	{
		this.minImportance = level;
	}

	private void writeLog(ELogLevel level, String line)
	{
		if(level.ordinal() < this.minImportance.ordinal())
		{
			return;
		}

		// Log to console
		System.out.println(line);

		// Log to file?
		if(level != ELogLevel.Debug)
		{
			try
			{
				synchronized(this.writer)
				{
					this.writer.write(line);
					this.writer.write("\r\n");
					this.writer.flush();
				}
			}
			catch(IOException ex)
			{
				System.out.println("Error logging to file!");
			}
		}
	}

	public void debug(Object system, String s)
	{
		this.writeLog(ELogLevel.Debug, this.timeformat.format(new Date()) + " - [" + system + "][#debug] " + s);
	}

	public void info(Object system, String s)
	{
		this.writeLog(ELogLevel.Info, this.timeformat.format(new Date()) + " - [" + system + "] " + s);
	}

	public void warning(Object system, String s)
	{
		this.writeLog(ELogLevel.Warning, this.timeformat.format(new Date()) + " - [" + system + "][#warning] " + s);
	}

	public void error(Object system, String s, Exception ex)
	{
		this.writeLog(ELogLevel.Error, this.timeformat.format(new Date()) + " - [" + system + "][#error] " + s + ", stacktrace: \r\n" + Logger.convertStackTraceToString(ex));
	}

	public static String convertStackTraceToString(Exception ex)
	{
		StringWriter sw = new StringWriter();
		if(ex != null)
		{
			ex.printStackTrace(new PrintWriter(sw));
		}
		return sw.toString();
	}

	private static Logger instance;

	public static Logger getInstance()
	{
		return instance;
	}

	public static void createInstance(String name)
	{
		instance = new Logger(name);
	}
}
