package org.openhabbo.items;

import org.openhabbo.OpenHABBO;
import org.openhabbo.items.impl.GenericItem;
import org.openhabbo.registries.ItemImplRegistry;
import org.openhabbo.storage.DatabaseResult;

public final class ItemDefinition
{
	public int id;
	public String sprite;
	public String color;
	public byte length;
	public byte width;
	public float height;
	public char type;
	private char[] behaviourFlags;
	private Class<? extends Item> implClass;
	public String data;
	public String name;
	public String description;

	private ItemDefinition()
	{

	}

	public boolean is(char flag)
	{
		for(int i = 0; i < this.behaviourFlags.length; i++)
		{
			if(this.behaviourFlags[i] == flag)
			{
				return true;
			}
		}
		return false;
	}

	public static ItemDefinition parse(DatabaseResult result)
	{
		ItemDefinition def = new ItemDefinition();
		def.id = result.getInt("id");
		def.sprite = result.getString("sprite");
		def.color = result.getString("color");
		def.length = result.getByte("length");
		def.width = result.getByte("width");
		def.height = result.getFloat("height");
		def.behaviourFlags = result.getString("behaviour").toCharArray();
		def.name = result.getString("name");
		def.description = result.getString("description");
		def.loadImplClass(result.getString("class"));
		def.data = result.getString("classdata");
		return def;
	}

	private void loadImplClass(String name)
	{
		this.implClass = null;
		if(name != null && name.length() > 0)
		{
			this.implClass = ItemImplRegistry.lookup(name);
			if(this.implClass == null)
			{
				OpenHABBO.getInstance().getLogger().warning("ItemDefinition", String.format("Implementation '%s' is not in the registry, any missing modules? Falling back to GenericItem.", name));
			}
		}
	}

	public Item newInstance()
	{
		// Attempt to create an instance of the impl class
		Item item = null;
		if(this.implClass != null)
		{
			try
			{
				item = this.implClass.newInstance();
			}
			catch(Exception ex)
			{
				// Bad impl class?
				ex.printStackTrace();
			}
		}

		// No implementation constructed? Fall back to GenericItem
		if(item == null)
		{
			item = new GenericItem();
		}

		// Wire it together with the definition and return!
		item.definition = this;
		return item;
	}
}
