package org.openhabbo.modules;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import org.openhabbo.OpenHABBO;

public abstract class ModuleResource
{
	private Map<String, Object> m_attachments = new HashMap<String, Object>();
	private Map<Integer, ArrayList<ModuleEventHandler>> m_eventHandlers = new HashMap<Integer, ArrayList<ModuleEventHandler>>();

	public <T> void set(String key, Object attachment)
	{
		synchronized(m_attachments)
		{
			m_attachments.put(key, attachment);
		}
	}

	public void unset(String key)
	{
		synchronized(m_attachments)
		{
			m_attachments.remove(key);
		}
	}

	public boolean has(String key)
	{
		synchronized(m_attachments)
		{
			return m_attachments.containsKey(key);
		}
	}

	@SuppressWarnings("unchecked")
	public <T> T get(String key)
	{
		synchronized(m_attachments)
		{
			Object attachment = m_attachments.get(key);
			if(attachment != null)
			{
				return (T) attachment;
			}
			else
			{
				return null;
			}
		}
	}

	public void addEventHandler(int code, ModuleEventHandler handler)
	{
		synchronized(m_eventHandlers)
		{
			ArrayList<ModuleEventHandler> handlers = m_eventHandlers.get(code);
			if(handlers == null)
			{
				handlers = new ArrayList<ModuleEventHandler>();
				m_eventHandlers.put(code, handlers);
			}
			handlers.add(handler);
		}
	}

	public void removeEventHandler(int code, ModuleEventHandler handler)
	{
		synchronized(m_eventHandlers)
		{
			ArrayList<ModuleEventHandler> handlers = m_eventHandlers.get(code);
			handlers.remove(handler);
			if(handlers.isEmpty())
			{
				m_eventHandlers.remove(code);
			}
		}
	}

	protected void raiseEvent(int code)
	{
		this.raiseEvent(code, null);
	}

	protected void raiseEvent(int code, ModuleEventArgs args)
	{
		synchronized(m_eventHandlers)
		{
			ArrayList<ModuleEventHandler> handlers = m_eventHandlers.get(code);
			if(handlers != null)
			{
				for(ModuleEventHandler handler : handlers)
				{
					try
					{
						handler.handle(this, args);
					}
					catch(Exception ex)
					{
						OpenHABBO.getModHost().getLogger().error("ModuleResource", String.format("Error executing a handler for event #%d.", code), ex);
					}
					finally
					{
						if(args != null)
						{
							ModuleEventArgs.release(args);
						}
					}
				}
			}
		}
	}

	/*
	 * protected void raiseEvent(int code, String argKey1, Object argValue1)
	 * {
	 * this.raiseEvent(name, new ModuleEventArgs(argKey1, argValue1));
	 * }
	 * protected void raiseEvent(int code, String argKey1, Object argValue1, String argKey2, Object argValue2)
	 * {
	 * this.raiseEvent(name, new ModuleEventArgs(argKey1, argValue1, argKey2, argValue2));
	 * }
	 * protected void raiseEvent(int code, String argKey1, Object argValue1, String argKey2, Object argValue2, String argKey3, Object argValue3)
	 * {
	 * this.raiseEvent(name, new ModuleEventArgs(argKey1, argValue1, argKey2, argValue2, argKey3, argValue3));
	 * }
	 */
}
