package org.openhabbo.net.messages;

import org.openhabbo.util.Base64Encoding;
import org.openhabbo.util.WireEncoding;

public final class ClientMessage
{
	private final static int ID_LENGTH = 2;

	public final int msgId;
	private byte[] m_payload;
	private final int m_length;
	private int m_pointer;

	public ClientMessage(byte[] data, int length)
	{
		m_pointer = 0;
		m_length = length;
		if(data != null)
		{
			m_payload = data;
			this.msgId = this.readFixedInt();
		}
		else
		{
			m_payload = new byte[0];
			this.msgId = -1;
		}
	}

	public void reset()
	{
		m_pointer = ClientMessage.ID_LENGTH;
	}

	public void advance(int n)
	{
		m_pointer += n;
	}

	public boolean contains(byte b)
	{
		for(int x = 0; x < m_payload.length; x++)
		{
			if(m_payload[x] == b)
			{
				return true;
			}
		}
		return false;
	}

	private byte[] readBytes(int n)
	{
		byte[] data = new byte[n];
		for(int x = 0; x < n; x++, m_pointer++)
		{
			data[x] = m_payload[m_pointer];
		}
		return data;
	}

	public Boolean readFixedBool()
	{
		return(m_payload[m_pointer++] == 'A');
	}

	public int readFixedInt()
	{
		return Base64Encoding.decodeInt(new byte[] { m_payload[m_pointer++], m_payload[m_pointer++] });
	}

	public String readStr()
	{
		return new String(this.readBytes(this.readFixedInt()));
	}

	public int readParseFixedInt()
	{
		try
		{
			return Integer.parseInt(this.readStr());
		}
		catch(NumberFormatException ex)
		{
			return 0;
		}
	}

	public Boolean readBool()
	{
		return(this.remaining() > 0 && m_payload[m_pointer++] == WireEncoding.POSITIVE);
	}

	public int readInt()
	{
		byte[] bytes = new byte[this.remaining() < WireEncoding.MAX_INTEGER_BYTE_AMOUNT ? this.remaining() : WireEncoding.MAX_INTEGER_BYTE_AMOUNT];
		for(int x = 0, y = m_pointer; x < bytes.length; x++, y++)
		{
			bytes[x] = m_payload[y];
		}
		int[] ret = WireEncoding.decodeInt(bytes);
		m_pointer += ret[1];
		return ret[0];
	}

	public String getBody()
	{
		return new String(m_payload);
	}

	public int length()
	{
		return m_length;
	}

	public int remaining()
	{
		return m_length - m_pointer;
	}

	public String header()
	{
		return new String(Base64Encoding.encodeInt(this.msgId, ClientMessage.ID_LENGTH));
	}

	@Override
	public String toString()
	{
		return this.header() + new String(m_payload, ClientMessage.ID_LENGTH, m_length - ClientMessage.ID_LENGTH);
	}
}
