package org.openhabbo.net.messages;

import java.io.ByteArrayOutputStream;
import java.io.IOException;

import org.openhabbo.util.Base64Encoding;
import org.openhabbo.util.WireEncoding;

public final class ServerMessage
{
	public final static byte MSG_EOF = (byte) 1;

	public int msgId;
	private ByteArrayOutputStream m_buff;

	public ServerMessage()
	{
		m_buff = new ByteArrayOutputStream();
	}

	public ServerMessage(int msgId)
	{
		this();
		this.init(msgId);
	}

	public void init(int msgId)
	{
		this.reset();
		this.msgId = msgId;
		this.append(Base64Encoding.encodeInt(msgId, 2));
	}

	public void reset()
	{
		m_buff.reset();
	}

	public void append(byte b)
	{
		m_buff.write(b);
	}

	public void append(char c)
	{
		this.append((byte) c);
	}

	public void append(byte[] bytes)
	{
		try
		{
			m_buff.write(bytes);
		}
		catch(IOException e)
		{
			e.printStackTrace();
		}
	}

	public void append(String s)
	{
		if(s.length() > 0)
		{
			this.append(s.getBytes());
		}
	}

	public void appendBool(boolean b)
	{
		if(b)
		{
			this.append(WireEncoding.POSITIVE);
		}
		else
		{
			this.append(WireEncoding.NEGATIVE);
		}
	}

	public void appendInt(int i)
	{
		this.append(WireEncoding.encodeInt(i));
	}

	public void appendStr(String s)
	{
		this.appendStr(s, (byte) 2);
	}

	public void appendStr(String s, byte tail)
	{
		this.append(s != null ? s : "");
		this.append(tail);
	}

	public void appendStrLine(String s)
	{
		this.appendStr(s, (byte) 13);
	}

	public void appendObject(SerializableObject obj)
	{
		obj.serialize(this);
	}

	public byte[] getResult()
	{
		m_buff.write(ServerMessage.MSG_EOF);
		return m_buff.toByteArray();
	}

	public String header()
	{
		return new String(Base64Encoding.encodeInt(this.msgId, 2));
	}

	public int length()
	{
		if(m_buff.size() > 0)
		{
			return m_buff.size() - 2;
		}
		else
		{
			return 0;
		}
	}

	@Override
	public String toString()
	{
		return new String(m_buff.toByteArray());
	}
}
