package org.openhabbo.remote;

import java.util.HashMap;
import java.util.Map;

import org.openhabbo.OpenHABBO;
import org.openhabbo.util.PackageClassFinder;

public class RemoteCommandHandlerManager
{
	private Map<String, RemoteCommandHandler> m_handlers;

	public RemoteCommandHandlerManager()
	{
		m_handlers = new HashMap<String, RemoteCommandHandler>();
	}

	public void registerHandler(String cmd, RemoteCommandHandler handler)
	{
		m_handlers.put(cmd, handler);
		OpenHABBO.getInstance().getLogger().info("RemoteCommandHandlerMgr", String.format("Registered remotecommand '%s'.", cmd));
	}

	public void registerHandler(RemoteCommandHandler handler)
	{
		this.registerHandler(handler.getCommand(), handler);
	}

	public void unregisterHandler(String cmd)
	{
		m_handlers.remove(cmd);
	}

	public boolean handleCommand(String cmd, String payload)
	{
		// Handler registered?
		RemoteCommandHandler handler = m_handlers.get(cmd);
		if(handler == null)
		{
			OpenHABBO.getInstance().getLogger().warning("RemoteCommandHandlerMgr", String.format("Unknown command '%s'. Command ignored.", cmd));
			return false;
		}

		// Invoke handler and catch exceptions
		try
		{
			handler.handle(payload);
		}
		catch(Exception ex)
		{
			OpenHABBO.getInstance().getLogger().error("RemoteCommandHandlerMgr", String.format("Error processing command '%s'. (payload=%s)", cmd, payload), ex);
		}

		// Handled
		return true;
	}

	public void registerStandardCommands()
	{
		for(Class<RemoteCommandHandler> cmdClass : PackageClassFinder.getClasses("org.openhabbo.remote.std"))
		{
			try
			{
				this.registerHandler(cmdClass.newInstance());
			}
			catch(Exception ex)
			{
				ex.printStackTrace();
			}
		}
	}
}
