package org.openhabbo.rooms;

import org.openhabbo.storage.DatabaseResult;

public class RoomModel
{
	public String name;
	public int doorX;
	public int doorY;
	public float doorZ;
	public byte doorRotation;
	public boolean hasSwimmingPool;
	public boolean clubOnly;

	private String m_mapStr;
	private char[][] m_map;
	private String m_heightMapStr;
	private String m_floorMapStr;

	public final char[][] getMap()
	{
		return m_map;
	}

	public String getHeightMapString()
	{
		return m_heightMapStr;
	}

	public String getFloorMapString()
	{
		return m_floorMapStr;
	}

	public int getMaxX()
	{
		return m_map.length;
	}

	public int getMaxY()
	{
		return m_map[0].length;
	}

	public void parseMap()
	{
		// Parse the map axes from the string
		String[] axes = m_mapStr.split("\r");
		int maxX = axes[0].length();
		int maxY = axes.length;
		m_map = new char[maxX][maxY];

		// Process the map tiles and build the strings for the client
		StringBuilder hmsb = new StringBuilder();
		StringBuilder fmsb = new StringBuilder();
		for(int y = 0; y < maxY; y++)
		{
			for(int x = 0; x < maxX; x++)
			{
				char tile = axes[y].charAt(x);
				hmsb.append(tile);
				if(x == this.doorX && y == this.doorY)
				{
					tile = Integer.toString((int) this.doorZ).charAt(0);
				}
				m_map[x][y] = Character.toLowerCase(tile);
				fmsb.append(m_map[x][y]);
			}
			hmsb.append((char) 13);
			fmsb.append((char) 13);
		}
		m_heightMapStr = hmsb.toString();
		m_floorMapStr = fmsb.toString();
	}

	public static RoomModel parse(DatabaseResult row)
	{
		RoomModel model = new RoomModel();
		model.name = row.getString("name");
		model.doorX = row.getInt("door_x");
		model.doorY = row.getInt("door_y");
		model.doorZ = row.getFloat("door_z");
		model.doorRotation = row.getByte("door_rotation");
		model.m_mapStr = row.getString("heightmap");
		model.hasSwimmingPool = row.getBoolean("has_pool");
		model.clubOnly = row.getBoolean("club_only");
		return model;
	}
}
