package org.openhabbo.sessions.handlers;

import org.openhabbo.net.messages.ClientMessage;
import org.openhabbo.net.messages.ServerMessage;
import org.openhabbo.rooms.instances.RoomUser;
import org.openhabbo.sessions.EServerMessage;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionMessageHandler;

public class GiveRightsHandler implements SessionMessageHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Get the ID of the user to give rights to
		int userId = request.readInt();

		// This user can give rights?
		if(session.getRoomUser() != null && session.getRoomUser().hasRoomOwner)
		{
			// Get the target
			RoomUser target = session.getRoomInstance().getRoomUserByUserId(userId);
			if(target != null && !target.hasRoomOwner)
			{
				// Update the room
				if(session.getRoomInstance().addRights(target.getUser().id, target.getUser().username))
				{
					// Update the room user
					target.hasRoomRights = true;
					target.statuses.put("flatctrl", null);
					target.requiresUpdate = true;
					target.getSession().sendRoomPrivileges();

					// Update the issuer
					response.init(EServerMessage.FlatControllerAdded);
					response.appendInt(session.getRoomInstance().getInfo().id);
					response.appendInt(target.getUser().id);
					response.appendStr(target.getUser().username);
					session.send(response);
				}
			}
		}
	}
}
