package org.openhabbo.sessions.handlers;

import org.openhabbo.OpenHABBO;
import org.openhabbo.catalog.CatalogProduct;
import org.openhabbo.items.GiftInfo;
import org.openhabbo.items.Item;
import org.openhabbo.net.messages.ClientMessage;
import org.openhabbo.net.messages.ServerMessage;
import org.openhabbo.sessions.EServerMessage;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionMessageHandler;

public class OpenPresentHandler implements SessionMessageHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Allowed to open presentboxes in this room?
		if(!session.getRoomUser().hasRoomOwner)
		{
			return;
		}

		// Find the item
		Item item = session.getRoomInstance().getItem(request.readInt());
		if(item == null || !item.definition.sprite.startsWith("present_"))
		{
			return;
		}

		// Pickup + the item
		session.getRoomInstance().pickupItem(item.id, session);

		// Get giftinfo for the item
		GiftInfo giftInfo = OpenHABBO.getInstance().getItemMgr().getGiftInfoForItem(item.id);
		if(giftInfo == null)
		{
			OpenHABBO.getInstance().getLogger().warning(this, String.format("Opening presentbox #%d failed: gift info missing!", item.id));
			return;
		}

		// Get product
		CatalogProduct product = OpenHABBO.getInstance().getCatalog().getProduct(giftInfo.productId);
		if(product == null)
		{
			session.sendNotif("Invalid presentbox contents, contents have possibly expired.");
			return;
		}

		// 'Purchase' the product for this user
		if(OpenHABBO.getInstance().getCatalog().purchase(product, giftInfo.customData, session.getUser(), session.getUser().id, false, null))
		{
			// 'Present OK'
			response.init(219);
			response.appendInt(item.id);
			session.send(response);

			// Present contents
			response.init(EServerMessage.PresentOpened);
			response.appendStr("s");
			response.appendInt(product.itemDefinition.id);
			response.appendStr(product.itemDefinition.name);
			session.send(response);
		}

		// Delete the item + giftinfo
		item.delete();
		giftInfo.delete();
	}
}
