package org.openhabbo.sessions.handlers;

import java.util.ArrayList;
import java.util.List;

import org.openhabbo.OpenHABBO;
import org.openhabbo.messenger.MessengerBuddy;
import org.openhabbo.net.messages.ClientMessage;
import org.openhabbo.net.messages.ServerMessage;
import org.openhabbo.sessions.EServerMessage;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionMessageHandler;

public class SearchHabboHandler implements SessionMessageHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Requires initialized messenger
		if(!session.getInstantMessenger().initialized()) return;

		// Valid criteria?
		String criteria = request.readStr().trim();
		if(criteria.length() == 0) return;

		// Get results and group them
		List<MessengerBuddy> buddies = new ArrayList<MessengerBuddy>();
		List<MessengerBuddy> nonBuddies = new ArrayList<MessengerBuddy>();
		for(MessengerBuddy user : OpenHABBO.getInstance().getImSvc().getUserSearchResults(criteria))
		{
			// Buddy or non-buddy?
			if(session.getInstantMessenger().hasBuddy(user.userId))
			{
				buddies.add(user);
			}
			else
			{
				nonBuddies.add(user);
			}
		}

		// Serialize the results
		response.init(EServerMessage.HabboSearchResult);
		response.appendInt(buddies.size());
		for(MessengerBuddy user : buddies)
		{
			this.serializeSearchResultUser(user, response);
		}
		response.appendInt(nonBuddies.size());
		for(MessengerBuddy user : nonBuddies)
		{
			this.serializeSearchResultUser(user, response);
		}
		session.send(response);
	}

	private void serializeSearchResultUser(MessengerBuddy user, ServerMessage msg)
	{
		msg.appendInt(user.userId);
		msg.appendStr(user.username);
		msg.appendStr(user.motto);
		msg.appendBool(user.online());
		msg.appendBool(user.inRoom());
		msg.appendStr("");
		msg.appendBool(false);
		msg.appendStr(user.figure);
		msg.appendStr(user.lastOnline);
		msg.appendStr("");
	}
}
