package org.openhabbo.users;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.openhabbo.OpenHABBO;
import org.openhabbo.access.UserRole;
import org.openhabbo.storage.DatabaseConnection;
import org.openhabbo.storage.DatabaseResult;
import org.openhabbo.storage.SQLHelper;

public class User
{
	// Core
	public int id;
	public String username;
	public String realName;
	public UserRole role;

	// Appearance
	public String figure;
	public char gender;
	public String motto;

	// Valueables
	public int credits;
	public int pixels;

	// Preferences
	public int homeRoomId;

	// Respecting
	public int respect;
	public int dailyRespectPoints;
	public int dailyPetRespectPoints;

	// All badges + badge slots in use
	public boolean requiresBadgeUpdate;
	public Map<String, Byte> badges;
	public Map<Byte, String> equippedBadges = new HashMap<Byte, String>();

	// Tags
	public List<String> tags;

	// Achievements
	public Map<Integer, Integer> achievements;

	public void equipBadges()
	{
		// Clear the equipped badges prior to filling the slots again
		this.equippedBadges.clear();

		// Every badge is mapped from badge ID to slot
		for(Entry<String, Byte> badge : this.badges.entrySet())
		{
			// If this badge is actually in a slot...
			if(badge.getValue() > 0)
			{
				this.equippedBadges.put(badge.getValue(), badge.getKey());
			}
		}
	}

	public void saveAppearance()
	{
		OpenHABBO.getInstance().getDb().execute("UPDATE users SET figure='" + SQLHelper.escape(this.figure) + "',gender='" + this.gender + "',motto='" + SQLHelper.escape(this.motto) + "' WHERE id=" + this.id);
	}

	public void saveCreditsAndPixels()
	{
		OpenHABBO.getInstance().getDb().execute("UPDATE users SET credits=" + this.credits + ",pixels=" + this.pixels + " WHERE id=" + this.id);
	}

	public void reloadCreditsAndPixels()
	{
		DatabaseResult db = OpenHABBO.getInstance().getDb().query("SELECT credits,pixels FROM users WHERE id=" + this.id);
		if(db.row())
		{
			this.credits = db.getInt("credits");
			this.pixels = db.getInt("pixels");
		}
		db.release();
	}

	public void saveMisc()
	{
		OpenHABBO.getInstance().getDb().execute("UPDATE users SET homeroom_id=" + this.homeRoomId + ",respect=" + this.respect + ",daily_respect_points=" + this.dailyRespectPoints + ",daily_petrespect_points=" + this.dailyPetRespectPoints + " WHERE id=" + this.id);
	}

	public void saveBadges()
	{
		// Get dedicated database connection
		DatabaseConnection conn = OpenHABBO.getInstance().getDb().getConnMgr().getConnection();
		if(conn == null)
		{
			return;
		}

		// Create PreparedStatement, batch everything and execute
		try
		{
			PreparedStatement query = conn.getInnerConnection().prepareStatement("UPDATE user_badges SET slot=? WHERE user_id=" + this.id + " AND badge=?");
			for(Entry<String, Byte> badge : this.badges.entrySet())
			{
				query.setByte(1, badge.getValue());
				query.setString(2, badge.getKey());
				query.addBatch();
			}
			query.executeBatch();
			query.close();
		}
		catch(SQLException ex)
		{
			OpenHABBO.getInstance().getLogger().error("User", "Error saving badges.", ex);
		}

		// Release the dedicated database connection
		OpenHABBO.getInstance().getDb().getConnMgr().releaseConnection(conn);
	}

	public static User parse(DatabaseResult row)
	{
		User user = new User();
		user.id = row.getInt("id");
		user.username = row.getString("username");
		user.realName = row.getString("realname");
		user.role = OpenHABBO.getInstance().getAccessCtrl().getUserRole(row.getInt("role_id"));
		user.figure = row.getString("figure");
		user.gender = row.getString("gender").equals("F") ? 'F' : 'M';
		user.motto = row.getString("motto");
		user.credits = row.getInt("credits");
		user.pixels = row.getInt("pixels");
		user.homeRoomId = row.getInt("homeroom_id");
		user.respect = row.getInt("respect");
		user.dailyRespectPoints = row.getInt("daily_respect_points");
		user.dailyPetRespectPoints = row.getInt("daily_petrespect_points");
		return user;
	}
}
