package org.openhabbo.users.achievements;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.openhabbo.OpenHABBO;
import org.openhabbo.net.messages.ServerMessage;
import org.openhabbo.sessions.EServerMessage;
import org.openhabbo.sessions.Session;
import org.openhabbo.storage.DatabaseResult;
import org.openhabbo.storage.SQLDatabase;
import org.openhabbo.users.User;

public class AchievementManager
{
	private Map<Integer, Achievement> m_achievements;

	public AchievementManager()
	{
		m_achievements = new HashMap<Integer, Achievement>();
	}

	public void reload()
	{
		synchronized(m_achievements)
		{
			m_achievements.clear();

			DatabaseResult result = OpenHABBO.getInstance().getDb().query("SELECT * FROM achievements");
			while(result.row())
			{
				Achievement ach = Achievement.parse(result);
				m_achievements.put(ach.id, ach);
			}
			result.release();

			OpenHABBO.getInstance().getLogger().info("AchievementMgr", String.format("Reloaded %d achievements.", this.achievements()));
		}
	}

	public Achievement getAchievement(int achievementId)
	{
		return m_achievements.get(achievementId);
	}

	public Collection<Achievement> getAllAchievements()
	{
		return m_achievements.values();
	}

	public Map<Integer, Integer> getAchievementsForUser(int userId)
	{
		HashMap<Integer, Integer> achs = new HashMap<Integer, Integer>();

		DatabaseResult result = OpenHABBO.getInstance().getDb().query("SELECT achievement_id,level FROM user_achievements WHERE user_id=" + userId);
		while(result.row())
		{
			achs.put(result.getInt("achievement_id"), result.getInt("level"));
		}
		result.release();

		return achs;
	}

	public void unlockAchievementForUser(int userId, int achievementId)
	{
		this.unlockAchievementForUser(userId, achievementId, Achievement.START_LEVEL);
	}

	public void unlockAchievementForUser(int userId, int achievementId, int level)
	{
		// User and achievement exists?
		User user = OpenHABBO.getInstance().getUserMgr().getUserById(userId, true);
		Achievement ach = this.getAchievement(achievementId);
		if(user == null || ach == null)
		{
			return;
		}

		// Valid level?
		if(level > ach.maxLevel)
		{
			level = ach.maxLevel;
		}

		// Calculate pixel reward
		int pixels = ach.getPixels(level);

		// Shorthand variable
		SQLDatabase db = OpenHABBO.getInstance().getDb();

		// Delete old achievement level (if any)
		db.execute("DELETE FROM user_achievements WHERE user_id=" + userId + " AND achievement_id=" + achievementId);

		// New achievement level
		db.execute("INSERT INTO user_achievements(user_id,achievement_id,level) VALUES (" + userId + "," + achievementId + "," + level + ")");

		// Give pixels
		user.pixels += pixels;
		user.saveCreditsAndPixels();

		// User online?
		Session session = OpenHABBO.getInstance().getSessionMgr().getSessionByUserId(userId);
		if(session != null)
		{
			// Update achievement list
			session.getUser().achievements.put(achievementId, level);

			// Notify of achievement gain
			ServerMessage msg = new ServerMessage(EServerMessage.AchievementNotification);
			msg.appendInt(achievementId);
			msg.appendInt(level);
			msg.appendStr(ach.getBadge(level));
			if(level == Achievement.START_LEVEL)
			{
				// No previous badge
				msg.appendStr("");
			}
			else
			{
				// Previous badge
				msg.appendStr(ach.getBadge(level - 1));
			}
			session.send(msg);

			// Notify of pixel gain
			session.sendPixelsAddedNotif(pixels);
		}
	}

	public int achievements()
	{
		return m_achievements.size();
	}
}
