package org.openhabbo.communication.requests.catalog;

import org.openhabbo.catalog.Catalog;
import org.openhabbo.catalog.CatalogPage;
import org.openhabbo.catalog.CatalogProduct;
import org.openhabbo.communication.ClientMessage;
import org.openhabbo.communication.RequestHandler;
import org.openhabbo.communication.EServerMessage;
import org.openhabbo.communication.ServerMessage;
import org.openhabbo.sessions.Session;

public class GetCatalogPageHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Fetch the page from the catalog
		int pageId = request.readInt();
		CatalogPage page = Catalog.getInstance().getPage(pageId);

		// Page accessible to the user?
		if(page == null || !page.isEnabled || !page.isAccessibleTo(session.getUser()))
		{
			return;
		}
		
		// Build the response (containing page data + product data)
		response.init(EServerMessage.CatalogPage);
		response.appendInt(page.id);
		response.appendStr(page.layout);

		// Serialize extra msg for the layout
		if(page.layout.equals("frontpage3"))
		{
			response.appendInt(3);
			response.appendStr(page.headlineImage);
			response.appendStr(page.teaserImage);
			response.appendStr("");
			response.appendInt(11);
			response.appendStr(page.text1);
			response.appendStr("");
			response.appendStr(page.text2);
			response.appendStr(page.details);
			response.appendStr("");
			response.appendStr("#FAF8CC");
			response.appendStr("#FAF8CC");
			response.appendStr("Other ways to get more credits >");
			response.appendStr("magic.credits");
		}
		else if(page.layout.equals("recycler_info"))
		{
			response.appendInt(2);
			response.appendStr(page.headlineImage);
			response.appendStr(page.teaserImage);
			response.appendInt(3);
			response.appendStr(page.text1);
			response.appendStr(page.text2);
			response.appendStr(page.details);
		}
		else if(page.layout.equals("recycler_prizes"))
		{
			response.appendInt(1);
			response.appendStr("catalog_recycler_headline3");
			response.appendInt(1);
			response.appendStr(page.text1);
		}
		else if(page.layout.equals("spaces"))
		{
			response.appendInt(1);
			response.appendStr(page.headlineImage);
			response.appendInt(1);
			response.appendStr(page.text1);
		}
		else if(page.layout.equals("recycler"))
		{
			response.appendInt(2);
			response.appendStr(page.headlineImage);
			response.appendStr(page.teaserImage);
			response.appendInt(1);
			response.appendStr(page.text1, (byte) 10);
			response.appendStr(page.text2);
			response.appendStr(page.details);
		}
		else if(page.layout.equals("trophies"))
		{
			response.appendInt(1);
			response.appendStr(page.headlineImage);
			response.appendInt(2);
			response.appendStr(page.text1);
			response.appendStr(page.details);
		}
		else if(page.layout.equals("pets"))
		{
			response.appendInt(2);
			response.appendStr(page.headlineImage);
			response.appendStr(page.teaserImage);
			response.appendInt(4);
			response.appendStr(page.text1);
			response.appendStr("Give a name:");
			response.appendStr("Pick a color:");
			response.appendStr("Pick a race:");
		}
		else
		{
			response.appendInt(3);
			response.appendStr(page.headlineImage);
			response.appendStr(page.teaserImage);
			response.appendStr(page.specialImage);
			response.appendInt(3);
			response.appendStr(page.text1);
			response.appendStr(page.details);
			response.appendStr(page.teaserText);
		}

		// Serialize products
		response.appendInt(page.products.size());
		for(CatalogProduct product : page.products)
		{
			response.appendInt(product.id);
			response.appendStr(product.code);
			response.appendInt(product.costCredits);
			response.appendInt(product.costPixels);
			response.appendInt(product.costQuestPoints);
			response.appendInt(1);
			response.appendStr(product.getType());
			response.appendInt(product.itemDefinition.displayId);
			response.appendStr("");
			response.appendInt(product.itemAmount);
			response.appendInt(-1);
			response.appendInt(0);
		}
		session.send(response);
	}
}
