package org.openhabbo.communication.requests.navigator;

import org.openhabbo.communication.ClientMessage;
import org.openhabbo.communication.RequestHandler;
import org.openhabbo.communication.EServerMessage;
import org.openhabbo.communication.ServerMessage;
import org.openhabbo.rooms.RoomManager;
import org.openhabbo.rooms.RoomModel;
import org.openhabbo.sessions.Session;
import org.openhabbo.util.SecurityUtil;

public class CreateRoomHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Read desired name and modelname of the room to create
		String name = SecurityUtil.filterStr(request.readStr());
		String modelName = request.readStr();

		// Valid room model?
		RoomModel model = RoomManager.getInstance().getModel(modelName);
		if(model == null || (model.clubOnly && false))
		{
			session.sendNotif("Invalid room model.");
			return;
		}

		// Attempt to create the room
		int roomId = RoomManager.getInstance().createGuestRoom(session.getUser().id, name, model.name);
		if(roomId == -1)
		{
			session.sendNotif("The server could not create your new guestroom. Please try again later.");
		}
		else
		{
			response.init(EServerMessage.ROOM_CREATE_RESULT);
			response.appendInt(roomId);
			response.appendStr(name);
			session.send(response);
		}
	}
}
