package org.openhabbo.communication.requests.user;

import mod_messenger.InstantMessenger;

import org.openhabbo.communication.ClientMessage;
import org.openhabbo.communication.RequestHandler;
import org.openhabbo.communication.EServerMessage;
import org.openhabbo.communication.ServerMessage;
import org.openhabbo.sessions.Session;
import org.openhabbo.util.SecurityUtil;

public class UpdateMottoHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Get & filter new motto, ensure it's actually an update
		String motto = SecurityUtil.filterStr(request.readStr());
		if(motto.equals(session.getUser().motto))
		{
			return;
		}

		// Update user object
		session.getUser().motto = motto;
		session.getUser().saveAppearance();

		// Notify user of changed user info
		response.init(EServerMessage.UserMottoUpdate);
		response.appendInt(-1);
		response.appendStr(motto);
		session.send(response);

		// Update on messenger
		InstantMessenger messenger = session.get("messenger");
		if(messenger != null)
		{
			messenger.updateStatus(true);
		}
		
		// If in room, then the room users need an update too!
		if(session.inRoom())
		{
			response.init(EServerMessage.UserChange);
			response.appendInt(session.getRoomUser().avatarId);
			response.appendStr(session.getUser().figure);
			response.appendStr(session.getUser().gender);
			response.appendStr(session.getUser().motto);
			session.getRoomInstance().broadcast(response);
		}
	}
}
