package org.openhabbo.core;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

public abstract class DynamicObject
{
	private Map<String, Object> attachments;
	
	public final void set(String key)
	{
		this.set(key, "");
	}
	
	public final <T> void set(String key, T attachment)
	{
		if(this.attachments == null)
		{
			this.attachments = new ConcurrentHashMap<String, Object>(3);
		}
		this.attachments.put(key, attachment);
	}

	public final boolean unset(String key)
	{
		if(this.attachments != null)
		{
			return (this.attachments.remove(key) != null);
		}
		else
		{
			return false;
		}
	}

	public final boolean has(String key)
	{
		if(this.attachments != null)
		{
			return this.attachments.containsKey(key);
		}
		else
		{
			return false;
		}
	}

	@SuppressWarnings("unchecked")
	public final <T> T get(String key)
	{
		if(this.attachments != null)
		{
			Object attachment = this.attachments.get(key);
			if(attachment != null)
			{
				return (T) attachment;
			}
		}
		return null;
	}
}
