package org.openhabbo.events;

import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

import org.openhabbo.core.Logger;

public final class Event
{
	private final String id;
	private final List<EventListener> listeners;

	public Event(String id)
	{
		this.id = id;
		this.listeners = new CopyOnWriteArrayList<EventListener>();
	}

	public final void addListener(EventListener listener)
	{
		this.listeners.add(listener);
	}

	public final void removeListener(EventListener listener)
	{
		this.listeners.remove(listener);
	}

	public final boolean fire(Object subject)
	{
		return this.fire(subject, null);
	}

	public final boolean fire(Object subject, Object args)
	{
		boolean result = true;
		if(this.hasListeners())
		{
			for(EventListener listener : this.listeners)
			{
				try
				{
					result = (result && listener.handle(subject, args));
				}
				catch(Exception ex)
				{
					Logger.getInstance().error(this.id, String.format("Error executing listener %s.", listener.getClass().getName()), ex);
					return false;
				}
			}
		}

		// 'result' will be false if one or more handlers returned false
		return result;
	}

	public final String id()
	{
		return this.id;
	}

	public final boolean hasListeners()
	{
		return(!this.listeners.isEmpty());
	}

	@Override
	public final String toString()
	{
		return this.id;
	}
}
