package org.openhabbo.items;

import org.openhabbo.rooms.instances.RoomInstance;
import org.openhabbo.rooms.instances.RoomUser;

public abstract class UsableItem extends Item
{
	private RoomUser user = null;

	public final boolean inUse()
	{
		// Actually a user?
		if(this.user == null)
		{
			return false;
		}

		// User invalidated?
		if(this.user.invalidated())
		{
			this.user = null;
			return false;
		}

		// Item in use by a valid roomuser!
		return true;
	}

	public final RoomUser getUser()
	{
		return this.inUse() ? this.user : null;
	}

	public final void setUser(RoomUser user)
	{
		if(this.user != null)
		{
			this.user.unlockMovement();
		}
		this.user = user;
		this.setTempState(user != null ? "1" : "0");
	}

	public final void clearUser()
	{
		this.setUser(null);
	}

	@Override
	public void onRoomRemove(RoomInstance room)
	{
		super.onRoomRemove(room);
		this.clearUser();
	}
}
