package org.openhabbo.items.impl;

import java.util.List;

import mod_teleporter.mod_teleporter;

import org.openhabbo.items.UsableItem;
import org.openhabbo.items.Item;
import org.openhabbo.items.ItemManager;
import org.openhabbo.rooms.instances.RoomTile;
import org.openhabbo.rooms.instances.RoomUser;
import org.openhabbo.util.GameUtil;

public final class TeleporterItem extends UsableItem
{
	private boolean isWaiting = false;
	
	@Override
	public void onPurchase(List<Item> shipping)
	{
		// Store this teleporter, get unique ID
		ItemManager.getInstance().storeItem(this);
		
		// Create new teleporter, link to this teleporter ID
		Item other = this.definition.newInstance();
		other.ownerId = this.ownerId;
		other.data = Integer.toString(this.id);
		ItemManager.getInstance().storeItem(other);

		// Update this teleporter with ID of other teleporter
		this.data = Integer.toString(other.id);
		this.update();

		// Add both teleporters to shipping
		shipping.add(this);
		shipping.add(other);
	}

	@Override
	public void toggle(RoomUser user, int instruction)
	{
		// Can go in teleporters?
		if(user == null || !user.canMove())
		{
			return;
		}

		// Infront of teleporter? Attempt to move to the front
		RoomTile frontTile = this.getTileInFront();
		if(!user.at(frontTile))
		{
			user.startMovingTo(frontTile.x, frontTile.y);
			return;
		}

		// Not in use yet?
		if(!this.inUse())
		{
			// Tadaa
			this.setUser(user);

			// Lock own movement, move into the teleporter
			user.lockMovement();
			user.overrideNextTile = true;
			user.startMovingTo(this.x, this.y);

			// Wait...
			this.setTimeout(3);
		}
	}

	@Override
	protected void onTimeout()
	{
		// User inside?
		if(this.inUse())
		{
			// Get user
			RoomUser user = this.getUser();
			
			// Get out?
			if(user.getSession().authenticatedTeleporterId == this.id)
			{
				user.getSession().authenticatedTeleporterId = 0;
				this.kickUserOut();
				return;
			}
			
			// Get tele info
			int otherId = GameUtil.parseInt(this.data);
			int otherRoomId = mod_teleporter.getTeleporterRoomId(otherId);
			
			// Waiting for teleport start?
			if(this.isWaiting)
			{
				// Same room?
				if(otherRoomId == this.roomId)
				{
					// Warp to the other tele
					TeleporterItem other = (TeleporterItem)this.getRoom().getItem(otherId);
					user.headRotation = user.bodyRotation = other.rotation;
					user.getRoom().updateAvatarPosition(user, other.x, other.y, false);
					
					// Open the door in the other tele
					(other).openDoorTemporarily();
					
					// Get out!
					user.unlockMovement();
					user.startMovingTo(other.getTileInFront());
				}
				else
				{
					// Teleporting!
					user.getSession().authenticatedTeleporterId = otherId;
					user.getSession().tryGetRoomAccess(otherRoomId, "");
				}
				
				// Waiting done! Done with this teleporter
				this.isWaiting = false;
				this.clearUser();
			}
			
			// User just entered teleporter?
			else if(user.at(this.getTile()))
			{
				// Linked teleporter in room?
				if(otherRoomId == 0)
				{
					user.getSession().sendNotif("This teleporter is currently not linked.");
					this.kickUserOut();
				}
				else
				{
					// Flash this teleporter, wait 2 pulses before the actual teleport
					this.isWaiting = true;
					this.setTempState("2");
					this.setTimeout(3);
				}
			}
		}
		else
		{
			this.setState("0");
		}
	}

	private void openDoorTemporarily()
	{
		this.setTempState("1");
		this.setTimeout(2);
	}

	private void kickUserOut()
	{
		RoomUser user = this.getUser();
		if(user != null)
		{
			// Open the door...
			this.clearUser();
			this.openDoorTemporarily();

			// Start move to tile infront
			user.unlockMovement();
			user.startMovingTo(this.getTileInFront());
		}
	}
	
	public void arriveUser(RoomUser user)
	{
		this.setUser(user);
		this.setTimeout(2);
	}
}
