package org.openhabbo.net;

import org.jboss.netty.buffer.ChannelBuffer;
import org.jboss.netty.buffer.ChannelBuffers;
import org.jboss.netty.channel.Channel;
import org.jboss.netty.channel.ChannelFutureListener;
import org.jboss.netty.channel.ChannelHandlerContext;
import org.jboss.netty.channel.ExceptionEvent;
import org.jboss.netty.handler.codec.frame.FrameDecoder;

import org.openhabbo.net.codec.ClientMessageDecoder;
import org.openhabbo.net.codec.ServerMessageEncoder;
import org.openhabbo.sessions.SessionManager;
import org.openhabbo.util.CrossdomainPolicy;

public final class NetHandshakeHandler extends FrameDecoder
{
	private final static char MSG_MAGIC_GAME = '@';
	private final static char MSG_MAGIC_POLICYFILE = '<';
	
	private final static NetHandshakeHandler instance = new NetHandshakeHandler();
	
	public static NetHandshakeHandler getInstance()
	{
		return instance;
	}
	
	private NetHandshakeHandler()
	{
		// Hide constructor
	}
	
	@Override
	protected Object decode(ChannelHandlerContext ctx, Channel channel, ChannelBuffer buffer) throws Exception
	{
		// Switch to requested protocol
		byte magic = buffer.readByte();
		if(magic == MSG_MAGIC_GAME)
		{
			// Configure pipeline
			channel.getPipeline().addLast("decoder", new ClientMessageDecoder());
			channel.getPipeline().addLast("encoder", ServerMessageEncoder.getInstance());
			channel.getPipeline().remove(this);

			// Create connection
			NetConnectionHandler conn = new NetConnectionHandler(channel);
			
			// Create session
			SessionManager.getInstance().startSession(conn);
			
			// Configure pipeline
			channel.getPipeline().addLast("handler", conn);
		}
		else if(magic == MSG_MAGIC_POLICYFILE)
		{
			// Get policy file bytes and fill buffer
			byte[] bytes = CrossdomainPolicy.getXml().getBytes();
			ChannelBuffer data = ChannelBuffers.buffer(bytes.length);
			data.writeBytes(bytes);

			// Write buffer and close connection
			channel.write(data).addListener(ChannelFutureListener.CLOSE);
		}
		else
		{
			// Unknown protocol!
			channel.close();
			return null;
		}
		
		// Drain and forward buffer to other upstream handlers
		buffer.resetReaderIndex();
		return buffer.readBytes(buffer.readableBytes());
	}
	
	@Override
	public void exceptionCaught(ChannelHandlerContext ctx, ExceptionEvent e)
	{
		ctx.getChannel().close();
	}
}
