package org.openhabbo.net;

import java.net.InetSocketAddress;
import java.util.concurrent.Executors;

import org.jboss.netty.bootstrap.ServerBootstrap;
import org.jboss.netty.channel.Channel;
import org.jboss.netty.channel.ChannelPipeline;
import org.jboss.netty.channel.ChannelPipelineFactory;
import org.jboss.netty.channel.Channels;
import org.jboss.netty.channel.group.ChannelGroup;
import org.jboss.netty.channel.group.DefaultChannelGroup;
import org.jboss.netty.channel.socket.nio.NioServerSocketChannelFactory;
import org.jboss.netty.util.ThreadNameDeterminer;
import org.jboss.netty.util.ThreadRenamingRunnable;
import org.openhabbo.core.Logger;

public final class NetServer
{
	private ServerBootstrap server;
	private Channel listener;
	private final ChannelGroup channels;
	
	public NetServer()
	{
		this.server = null;
		this.listener = null;
		this.channels = new DefaultChannelGroup();
		
		// Custom thread naming
		ThreadRenamingRunnable.setThreadNameDeterminer(new ThreadNameDeterminer()
		{
			public String determineThreadName(String current, String proposed) throws Exception
			{
				String[] segments = current.split("-");
				int poolId = Integer.parseInt(segments[1]);
				int threadId = Integer.parseInt(segments[3]);
				if(poolId == 1)
				{
					return "NetBoss";
				}
				else
				{
					return "NetWorker #" + threadId;
				}
			}
		});
	}

	public boolean start(int port)
	{
		// Already running?
		if(this.server != null)
		{
			return false;
		}
		
		// Create server boss and worker threadpools using defaults
		ServerBootstrap server = new ServerBootstrap(new NioServerSocketChannelFactory(Executors.newCachedThreadPool(), Executors.newCachedThreadPool()));
		
		// Setup pipeline factory
		final GroupingHandler grouper = new GroupingHandler(this.channels);
		server.setPipelineFactory(new ChannelPipelineFactory()
		{
			public ChannelPipeline getPipeline() throws Exception
			{
				ChannelPipeline pipeline = Channels.pipeline(grouper, NetHandshakeHandler.getInstance());
				return pipeline;
			}
		});

		// Bind and start listening for connections
		try
		{
			Channel listener = server.bind(new InetSocketAddress(port));
			if(listener.isOpen())
			{
				this.server = server;
				this.listener = listener;
				
				// Yay!
				Logger.getInstance().info("NetServer", String.format("Listening for game connections on TCP port %d.", port));
				return true;
			}
		}
		catch(Exception ex)
		{
			
		}
		
		// Failed whatever
		Logger.getInstance().warning("NetServer", String.format("Could not start listening on TCP port %d! Port is already taken/security error?", port));
		return false;
	}
	
	public void stop()
	{
		if(this.server != null)
		{
			// Close listener
			this.listener.close();
			this.listener = null;
			
			// Done!
			Logger.getInstance().info("NetServer", "Closed listener.");
		}
	}
	
	public void addChannel(Channel channel)
	{
		this.channels.add(channel);
	}
	
	public void removeChannel(Channel channel)
	{
		this.channels.remove(channel);
	}
	
	public void close()
	{
		// Close all connections
		this.channels.close();
		this.channels.clear();
		
		// Destroy bootstrapper
		this.server.releaseExternalResources();
		this.server = null;
	}
	
	/* Begin singleton */
	private static NetServer instance;

	public static NetServer getInstance()
	{
		return instance;
	}

	public static void createInstance()
	{
		instance = new NetServer();
	}
	/* End singleton */
}
