package org.openhabbo.net.codec;

import org.jboss.netty.buffer.ChannelBuffer;
import org.jboss.netty.channel.Channel;
import org.jboss.netty.channel.ChannelHandlerContext;
import org.jboss.netty.handler.codec.replay.ReplayingDecoder;

import org.openhabbo.communication.ClientMessage;
import org.openhabbo.util.Base64Encoding;

public final class ClientMessageDecoder extends ReplayingDecoder<ClientMessageDecoder.DecodingState>
{
	private final static Exception invalidLengthException = new Exception("invalid length");
	
	private ClientMessage msg;
	
	public ClientMessageDecoder()
	{
		this.reset();
	}
	
	@Override
	protected Object decode(ChannelHandlerContext ctx, Channel channel, ChannelBuffer buffer, DecodingState state) throws Exception
	{
		switch(state)
		{
			// Determine length of ID + payload
			case LENGTH:
			{
				// Read in length bytes and decode
				byte[] bytes = new byte[ClientMessage.HEADER_LENGTH];
				buffer.readBytes(bytes);
				int total = Base64Encoding.decodeInt(bytes);
				if(total <= 0 || total > 1024)
				{
					throw ClientMessageDecoder.invalidLengthException;
				}
				
				// Pre-allocate payload
				this.msg.setPayload(new byte[total - ClientMessage.TYPE_LENGTH]);
				this.checkpoint(DecodingState.TYPE);
			}
			
			// Determine message type
			case TYPE:
			{
				// Read in type bytes and decode
				byte[] bytes = new byte[ClientMessage.TYPE_LENGTH];
				buffer.readBytes(bytes);
				
				// Set type
				this.msg.setType(Base64Encoding.decodeInt(bytes));
				this.checkpoint(DecodingState.PAYLOAD);
			}
			
			// Determine message payload
			case PAYLOAD:
			{
				// Read in payload
				buffer.readBytes(this.msg.getPayload());
				
				try
				{
					return this.msg;
				}
				finally
				{
					this.reset();
				}
			}
			default:
			{
				// Unknown decoding state
				throw ClientMessageDecoder.invalidLengthException;
			}
		}
	}
	
	private void reset()
	{
		this.checkpoint(DecodingState.LENGTH);
		this.msg = new ClientMessage();
	}
	
	protected enum DecodingState
	{
		LENGTH,
		TYPE,
		PAYLOAD
	}
}
