package mod_messenger;

import mod_messenger.msgs.AcceptRequestHandler;
import mod_messenger.msgs.DeclineRequestHandler;
import mod_messenger.msgs.FollowBuddyHandler;
import mod_messenger.msgs.FriendsListUpdateHandler;
import mod_messenger.msgs.GetFriendsRoomsHandler;
import mod_messenger.msgs.GetRoomsWithFriendsHandler;
import mod_messenger.msgs.InitMessengerHandler;
import mod_messenger.msgs.RemoveBuddyHandler;
import mod_messenger.msgs.RequestBuddyHandler;
import mod_messenger.msgs.SearchUserHandler;
import mod_messenger.msgs.SendInstantInviteHandler;
import mod_messenger.msgs.SendInstantMessageHandler;
import mod_messenger.msgs.VisitFriendsMeetingRoomHandler;

import org.openhabbo.modules.Module;
import org.openhabbo.modules.ModuleInfo;
import org.openhabbo.sessions.Session;

@ModuleInfo
(
		name = "mod_messenger",
		description = "Enables support for the in-game instant messenger.", 
		author = "Nils 'Nillus'",
		url = "http://modules.openhabbo.org/mod_messenger/",
		version = "1.0.0",
		dependencies = { }
)
public final class mod_messenger extends Module
{
	public mod_messenger()
	{
		// Messenger
		this.getAutoLoader().msgHandler(12, new InitMessengerHandler());
		this.getAutoLoader().msgHandler(15, new FriendsListUpdateHandler());
		this.getAutoLoader().msgHandler(34, new SendInstantInviteHandler());
		this.getAutoLoader().msgHandler(40, new RemoveBuddyHandler());
		this.getAutoLoader().msgHandler(41, new SearchUserHandler());
		this.getAutoLoader().msgHandler(33, new SendInstantMessageHandler());
		this.getAutoLoader().msgHandler(37, new AcceptRequestHandler());
		this.getAutoLoader().msgHandler(38, new DeclineRequestHandler());
		this.getAutoLoader().msgHandler(39, new RequestBuddyHandler());
		this.getAutoLoader().msgHandler(262, new FollowBuddyHandler());
		
		// Navigator
		this.getAutoLoader().msgHandler(432, new GetFriendsRoomsHandler());
		this.getAutoLoader().msgHandler(433, new GetRoomsWithFriendsHandler());
		this.getAutoLoader().msgHandler(490, new VisitFriendsMeetingRoomHandler());
		
		// Event hooks
		this.getAutoLoader().eventListener(Session.onLogout, new LogoutHook());
		this.getAutoLoader().eventListener(Session.onRoomEnter, new ChangeRoomHook());
		this.getAutoLoader().eventListener(Session.onRoomExit, new ChangeRoomHook());
		
		// Crawl idle buddy refs every 30 seconds to clean mem
		this.enablePulseTimer(30);
	}
	
	@Override
	public void onUnload()
	{
		// Clear registry
		InstantMessengerService.getInstance().clearRegistry();
	}

	@Override
	protected void onPulse()
	{
		// Clean the cache/registry of buddy info that hasn't been accessed recently
		InstantMessengerService.getInstance().cleanup();
	}

	@Override
	public String getStatusString()
	{
		return "regsize=" + InstantMessengerService.getInstance().registrySize();
	}
}
