package mod_moderation.msgs;

import org.openhabbo.communication.ClientMessage;
import org.openhabbo.communication.RequestHandler;
import org.openhabbo.communication.EServerMessage;
import org.openhabbo.communication.ServerMessage;
import org.openhabbo.rooms.Room;
import org.openhabbo.rooms.RoomManager;
import org.openhabbo.rooms.instances.RoomInstance;
import org.openhabbo.sessions.Session;

import mod_moderation.mod_moderation;
import mod_moderation.ModerationActionError;
import mod_moderation.ModerationCenter;
import mod_roomevent.RoomEvent;
import mod_roomevent.RoomEventManager;

public class ModPerformRoomActionHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Has mod?
		if(!session.getUser().rights.contains("fuse_mod"))
		{
			return;
		}
		
		// Read info
		int roomId = request.readInt();
		boolean lockRoom = request.readBool();
		boolean inappropriateRoom = request.readBool();
		boolean kickRoom = request.readBool();
		
		// Perform actions
		RoomInstance room = RoomManager.getInstance().getRoomInstance(roomId, false);
		if(room == null)
		{
			return;
		}
		
		// Kick users + stop event?
		if(kickRoom)
		{
			try
			{
				ModerationCenter.getInstance().issueRoomKick(session.getUser().id, room.getInfo().id, null);
			}
			catch(ModerationActionError error)
			{
				session.sendNotif(error.generateNotif());
			}
			
			// Event?
			RoomEvent event = session.getRoomInstance().get("event");
			if(event != null)
			{
				RoomEventManager.getInstance().removeEvent(event);
			}
		}
		
		// Room changes?
		if(lockRoom || inappropriateRoom)
		{
			// Lock room?
			if(lockRoom)
			{
				room.getInfo().state = Room.STATE_DOORBELL;
			}
			
			// Change room to inappropriate?
			if(inappropriateRoom)
			{
				room.getInfo().name = mod_moderation.inappropriateRoomName;
				room.getInfo().description = mod_moderation.inappropriateRoomDescription;
				room.getInfo().tags = new String[0];
			}
			
			// Save room
			room.getInfo().saveSettings();
			
			// Inform all room users to request new room info
			response.init(EServerMessage.RoomInfoUpdated);
			response.appendInt(room.getInfo().id);
			session.getRoomInstance().broadcast(response);
		}
	}
}
