package mod_onewaydoor;

import org.openhabbo.items.UsableItem;
import org.openhabbo.rooms.instances.RoomTile;
import org.openhabbo.rooms.instances.RoomUser;

public final class OneWayDoorItem extends UsableItem
{
	@Override
	public void onToggle(RoomUser user, int instruction)
	{
		// Attempt to move infront of the gate
		RoomTile frontTile = this.getTileInFront();
		if(!user.at(frontTile) && user.canMove())
		{
			user.startMovingTo(frontTile.x, frontTile.y);
			return;
		}

		// Is the other end of the gate blocked, not by an interactive item?
		RoomTile behind = this.getTileBehind();
		if(behind == null || !behind.isOpen())
		{
			// Don't bother entering the gate
			return;
		}

		// Not in use yet?
		if(!this.inUse())
		{
			// Tadaa
			this.setUser(user);

			// Lock own movement, move into the gate
			user.lockMovement();
			user.overrideNextTile = true;
			user.startMovingTo(this.x, this.y);

			// Wait...
			this.setTimeout(2);
		}
	}

	@Override
	protected void onTimeout()
	{
		// In use?
		if(this.inUse())
		{
			// User in the gate?
			RoomUser user = this.getUser();
			if(user.at(this.getTile()))
			{
				// Move to tile behind
				user.startMovingTo(this.getTileBehind());

				// Set a new timeout
				this.setTimeout(1);
			}

			// User exited the gate?
			else if(user.at(this.getTileBehind()))
			{
				user.unlockMovement();
				this.clearUser();
			}
		}
	}
}
