package mod_pingpong;

import java.util.ArrayList;
import java.util.List;

import org.openhabbo.communication.EServerMessage;
import org.openhabbo.communication.ServerMessage;
import org.openhabbo.modules.Module;
import org.openhabbo.modules.ModuleInfo;
import org.openhabbo.sessions.Session;
import org.openhabbo.sessions.SessionManager;

@ModuleInfo
(
		name = "mod_pingpong",
		description = "Ends sessions that haven't sent a PONG response to the PING message in time",
		author = "Nils 'Nillus'",
		url = "http://modules.openhabbo.org/mod_pingpong/",
		version = "1.0.0",
		dependencies = { }
)
public final class mod_pingpong extends Module
{
	public mod_pingpong()
	{
		this.enablePulseTimer(60);
		this.getAutoLoader().msgHandler(196, new PongHandler());
	}

	@Override
	protected void onPulse()
	{
		// Check pong status of every session
		List<Session> timeout = new ArrayList<Session>();
		ServerMessage ping = new ServerMessage(EServerMessage.Ping);
		for(Session session : SessionManager.getInstance().getAllSessions())
		{
			if(session.has("pong"))
			{
				boolean pong = session.get("pong");
				if(pong)
				{
					session.set("pong", false);
				}
				else
				{
					timeout.add(session);
					continue;
				}
			}
			else
			{
				session.set("pong", false);
			}
			
			// Ping!
			session.send(ping);
		}
		
		// End the timed out sessions
		for(Session session : timeout)
		{
			SessionManager.getInstance().endSession(session.id, "ping/pong timeout");
		}
	}
}
