package mod_vendingmachine;

import org.openhabbo.core.Logger;
import org.openhabbo.items.EItemFlags;
import org.openhabbo.items.UsableItem;
import org.openhabbo.rooms.instances.RoomUser;
import org.openhabbo.sessions.Session;
import org.openhabbo.util.GameUtil;

public final class VendingMachineItem extends UsableItem
{
	@Override
	public void onTrigger(Session session, int instruction)
	{
		// Get to the vending machine! (if possible)
		RoomUser user = session.getRoomUser();
		if(this.is(EItemFlags.FLOORITEM) && user.canMove() && !user.at(this.getTileInFront()))
		{
			user.startMovingTo(this.getTileInFront());
		}
		else
		{
			// Not in use?
			if(!this.inUse())
			{
				// Start using the vending machine
				if(this.is(EItemFlags.FLOORITEM))
				{
					user.lockMovement();
					user.lookTo(this.x, this.y, false);
				}
				
				// Vending in 2 pulses!
				this.setUser(user);
				this.setTimeout(2);
			}
		}
	}

	@Override
	protected void onTimeout()
	{
		// In use?
		if(this.inUse())
		{
			// Determine a vending ID
			Integer vendingId = -1;
			try
			{
				vendingId = Integer.parseInt(GameUtil.getRandomElement(this.definition.data.split(",")));
			}
			catch(Exception ex)
			{
				Logger.getInstance().warning("VendingMachineItem", String.format("Invalid vending IDs classdata for '%s': \"%s\", VendingMachineItems should specify atleast one numeric vending ID, and the classdata must be delimited by commas. Example: \"1,6,15,23\"", this.definition.sprite, this.definition.data));
			}
			finally
			{
				// Vend the item
				RoomUser user = this.getUser();
				user.carryItem(vendingId);
				user.unlockMovement();
				this.clearUser();
			}
		}
	}
}