package mod_wired;

import java.util.ArrayList;
import java.util.List;

import org.json.JSONObject;
import org.openhabbo.communication.ServerMessage;
import org.openhabbo.items.Item;
import org.openhabbo.rooms.instances.RoomUser;

public abstract class WiredAction extends WiredItem
{
	public int delay;
	private RoomUser user;
	
	public abstract void execute(RoomUser user);
	protected abstract boolean requiresUser();
	
	@Override
	protected void onWrap(JSONObject data) throws Exception
	{
		super.onWrap(data);
		if(this.delay > 0) data.put("del", this.delay);
	}
	
	@Override
	protected void onUnwrap(JSONObject data) throws Exception
	{
		super.onUnwrap(data);
		this.delay = data.optInt("del");
	}
	
	protected void execute(RoomUser user, int delay)
	{
		this.user = user;
		this.setTimeout(delay);
	}
	
	@Override
	protected void onTimeout()
	{
		// If a user is required, then validate it
		if(!this.requiresUser() || (user != null && !this.user.invalidated()))
		{
			this.flash();
			this.execute(this.user);
		}
		
		// Clear user
		this.user = null;
	}
	
	public List<WiredItem> getIncompatibleTriggers()
	{
		List<WiredItem> incompatible = new ArrayList<WiredItem>(0);
		
		// This item needs a user?
		if(this.requiresUser())
		{
			// Search all items in the stack
			for(Item item : this.getStackedItems())
			{
				// Item is trigger?
				if(item instanceof WiredTrigger)
				{
					// Trigger is incompatible?
					WiredTrigger trigger = (WiredTrigger)item;
					if(!trigger.suppliesUser())
					{
						incompatible.add(trigger);
					}
				}
			}
		}
		
		return incompatible;
	}
	
	@Override
	public ServerMessage serializeDialog()
	{
		// Aggregrate lists
		List<Item> selected = this.getSelectedItems();
		List<WiredItem> conflicts = this.getIncompatibleTriggers();
		
		// Serialize info
		ServerMessage msg = new ServerMessage(651);
		msg.appendBool(false); // Disable advanced selection options
		msg.appendInt(5);
		msg.appendInt(selected.size());
		for(Item item : selected)
		{
			msg.appendInt(item.id);
		}
		msg.appendInt(this.definition.displayId);
		msg.appendInt(this.id);
		msg.appendStr(this.getText());
		msg.appendInt(this.getParams().length);
		for(int param : this.getParams())
		{
			msg.appendInt(param);
		}
		msg.appendInt(this.getSelectionType()); // Item determiner option (not sure)
		msg.appendInt(this.getInterface());
		msg.appendInt(this.delay);
		msg.appendInt(conflicts.size());
		for(WiredItem item : conflicts)
		{
			msg.appendInt(item.definition.displayId);
		}
		msg.appendStr(""); // Unknown
		
		return msg;
	}
}
