package mod_wired;

import java.util.ArrayList;
import java.util.List;

import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;

import org.openhabbo.core.Logger;
import org.openhabbo.items.Item;
import org.openhabbo.rooms.instances.RoomInstance;

@SuppressWarnings({ "unchecked", "rawtypes" })
public abstract class WiredItem extends Item
{
	private JSONObject settings;

	@Override
	public final void onRoomAdd(RoomInstance room)
	{
		this.parseData();
	}
	
	@Override
	public final void onRoomRemove(RoomInstance room)
	{
		// Get rid of all data, nothing is persistent
		this.data = null;
		this.settings = null;
		
		// Cleanup!
		
	}
	
	public final void setSetting(String key, Object value)
	{
		if(key == null)
		{
			this.settings.remove(key);
		}
		else
		{
			this.settings.put(key, value);
		}
	}

	public final <T> T getSetting(String key)
	{
		if(this.settings != null)
		{
			Object value = this.settings.get(key);
			if(value != null)
			{
				return (T) value;
			}
		}
		return (T)null;
	}
	
	public final Item getSelectedItem()
	{
		List<Item> selected = this.getSelectedItems();
		if(selected.size() >= 1)
		{
			return selected.get(0);
		}
		else
		{
			return null;
		}
	}

	public final List<Item> getSelectedItems()
	{
		List<Item> items = new ArrayList<Item>(1);
		
		if(this.settings != null)
		{
			List list = (List) this.settings.get("selected");
			if(list != null)
			{
				for(Object entry : list)
				{
					Item item = this.getRoom().getItem((Integer)entry);
					if(item != null)
					{
						items.add(item);
					}
				}
			}
		}
		
		// TODO: cache this until changed
		return items;
	}
	
	private void parseData()
	{
		try
		{
			this.settings = (JSONObject) new JSONParser().parse(this.data);
		}
		catch (Exception e)
		{
			Logger.getInstance().warning("WiredItem #" + this.id, "Malformed JSON data, clearing all settings.");
			this.settings = null;
			this.updateData();
		}
	}
	
	private void updateData()
	{
		if(this.settings == null)
		{
			this.settings = new JSONObject();
		}
		this.data = this.settings.toJSONString();
		this.requireDbUpdate();
	}
	
	public void saveSettings()
	{
		this.updateData();
	}
}
