package mod_wired;

import java.util.ArrayList;
import java.util.List;

import org.openhabbo.communication.ServerMessage;
import org.openhabbo.items.Item;
import org.openhabbo.rooms.instances.RoomUser;

public abstract class WiredTrigger extends WiredItem
{
	protected abstract boolean suppliesUser();
	
	@Override
	protected void onTimeout()
	{
		this.execute(null);
	}
	
	public final void resetTimer()
	{
		this.clearTimeout();
	}
	
	public final void execute(RoomUser user)
	{
		// Fire!
		this.flash();
		
		// Get all items in the stack
		List<Item> items = this.getStackedItems();

		// Search for Conditions
		boolean failed = false;
		for(Item item : items)
		{
			// Is this a Condition?
			if(item instanceof WiredCondition)
			{
				// Evaluate the Condition
				WiredCondition condition = (WiredCondition)item;
				if(condition.evaluate(user) == true)
				{
					condition.flash();
				}
				else
				{
					failed = true;
				}
			}
		}
		
		// Atleast one condition failed?
		if(failed)
		{
			return;
		}
		
		// Search for Actions
		for(Item item : items)
		{
			// Is this an Action?
			if(item instanceof WiredAction)
			{
				// Flash the action!
				WiredAction action = (WiredAction)item;
				action.flash();
				
				// Execute immediately or delay?
				if(action.delay == 0)
				{
					action.execute(user);
				}
				else
				{
					action.execute(user, action.delay);
				}
			}
		}
	}
	
	@Override
	public ServerMessage serializeDialog()
	{
		// Aggregrate lists
		List<Item> selected = this.getSelectedItems();
		List<WiredItem> conflicts = new ArrayList<WiredItem>();
		
		// Serialize info
		ServerMessage msg = new ServerMessage(650);
		msg.appendBool(false); // Disable advanced selection options
		msg.appendInt(5);
		msg.appendInt(selected.size());
		for(Item item : selected)
		{
			msg.appendInt(item.id);
		}
		msg.appendInt(this.definition.displayId);
		msg.appendInt(this.id);
		msg.appendStr(this.getText());
		msg.appendInt(this.getParams().size());
		for(int param : this.getParams())
		{
			msg.appendInt(param);
		}
		msg.appendInt(this.getSelectionType());
		msg.appendInt(this.getInterface());
		msg.appendInt(conflicts.size());
		for(WiredItem item : conflicts)
		{
			msg.appendInt(item.definition.displayId);
		}
		msg.appendStr(""); // Unknown
		
		return msg;
	}
}
