package mod_wired;

import java.util.List;

import mod_wired.msgs.UpdateSnapshotsHandler;
import mod_wired.msgs.OpenWiredHandler;
import mod_wired.msgs.UpdateActionHandler;
import mod_wired.msgs.UpdateConditionHandler;
import mod_wired.msgs.UpdateTriggerHandler;
import mod_wired.triggers.WiredUserEntersRoomTrigger;
import mod_wired.triggers.WiredUserSaysPhraseTrigger;

import org.openhabbo.communication.ServerMessage;
import org.openhabbo.items.Item;
import org.openhabbo.modules.Module;
import org.openhabbo.modules.ModuleInfo;
import org.openhabbo.sessions.Session;

@ModuleInfo
(
		name = "mod_wired",
		description = "Adds support for 'wired furniture' in guestrooms. Wired furniture consists of Effects, Triggers and Conditions, and users can 'wire them together' to make cool stuff happen in rooms.",
		author = "Nils 'Nillus'",
		url = "http://habbo.nillus.net/modules/mod_wired/",
		version = "1.0.0",
		dependencies = { }
)
public final class mod_wired extends Module
{
	public mod_wired()
	{
		// Message handlers
		this.getAutoLoader().msgHandler(3050, new UpdateTriggerHandler());
		this.getAutoLoader().msgHandler(3051, new UpdateActionHandler());
		this.getAutoLoader().msgHandler(3052, new UpdateConditionHandler());
		this.getAutoLoader().msgHandler(3053, new OpenWiredHandler());
		this.getAutoLoader().msgHandler(3054, new UpdateSnapshotsHandler());
		
		// Events
		this.getAutoLoader().eventListener(Session.onRoomEnter, new WiredUserEntersRoomTrigger.RoomEnterHook());
		this.getAutoLoader().eventListener(Session.onChat, new WiredUserSaysPhraseTrigger.ChatHook());
		
		// Items
		for(Class c : ClassFinder.getClasses("mod_wired.actions"))
		{
			this.getAutoLoader().itemImpl(c.getSimpleName(), c);
		}
		for(Class c : ClassFinder.getClasses("mod_wired.conditions"))
		{
			this.getAutoLoader().itemImpl(c.getSimpleName(), c);
		}
		for(Class c : ClassFinder.getClasses("mod_wired.triggers"))
		{
			this.getAutoLoader().itemImpl(c.getSimpleName(), c);
		}
	}
	
	public final static ServerMessage composeMsg(WiredItem wired)
	{
		ServerMessage msg = new ServerMessage(650); // or 651
		msg.appendBool(false); // Don't offer special item determining options
		msg.appendInt(5); // Max selectable items
		
		// Append the selected items
		List<Item> selected = wired.getSelectedItems();
		msg.appendInt(selected.size());
		for(Item item : selected)
		{
			msg.appendInt(item.id);
		}
		
		// Append info about this item
		msg.appendInt(wired.definition.displayId);
		msg.appendInt(wired.id);
		
		// Append str1
		msg.appendStr("");
		
		// Append parameters
		int[] parameters = new int[] { };
		msg.appendInt(parameters.length);
		for(int parameter : parameters)
		{
			msg.appendInt(parameter);
		}
		
		// Unknown
		msg.appendInt(0);
		
		// If 650 = append one other, if 651 = append two other
		
		// Conflicting triggers
		msg.appendInt(0);
		
		// Append str2
		msg.appendStr("");
		
		// Done!
		return msg;
	}
}
