package mod_wired;

import mod_wired.msgs.UpdateSnapshotsHandler;
import mod_wired.msgs.UpdateActionHandler;
import mod_wired.msgs.UpdateConditionHandler;
import mod_wired.msgs.UpdateTriggerHandler;
import mod_wired.triggers.WiredUserEntersRoomTrigger;
import mod_wired.triggers.WiredUserSaysPhraseTrigger;

import org.openhabbo.modules.Module;
import org.openhabbo.modules.ModuleInfo;
import org.openhabbo.sessions.Session;
import org.openhabbo.util.ClassFinder;

@ModuleInfo
(
		name = "mod_wired",
		description = "Adds support for 'wired furniture' in guestrooms. Wired furniture consists of Effects, Triggers and Conditions, and users can 'wire them together' to make cool stuff happen in rooms.",
		author = "Nils 'Nillus'",
		url = "http://habbo.nillus.net/modules/mod_wired/",
		version = "1.0.0",
		dependencies = { }
)
public final class mod_wired extends Module
{
	public mod_wired()
	{
		// Message handlers
		this.getAutoLoader().msgHandler(3050, new UpdateTriggerHandler());
		this.getAutoLoader().msgHandler(3051, new UpdateActionHandler());
		this.getAutoLoader().msgHandler(3052, new UpdateConditionHandler());
		this.getAutoLoader().msgHandler(3054, new UpdateSnapshotsHandler());
		
		// Events
		this.getAutoLoader().eventListener(Session.onRoomEnter, new WiredUserEntersRoomTrigger.RoomEnterHook());
		this.getAutoLoader().eventListener(Session.onChat, new WiredUserSaysPhraseTrigger.ChatHook());
		
		// Items
		for(Class c : ClassFinder.getClasses("mod_wired.actions"))
		{
			this.getAutoLoader().itemImpl(c.getSimpleName(), c);
		}
		for(Class c : ClassFinder.getClasses("mod_wired.conditions"))
		{
			this.getAutoLoader().itemImpl(c.getSimpleName(), c);
		}
		for(Class c : ClassFinder.getClasses("mod_wired.triggers"))
		{
			this.getAutoLoader().itemImpl(c.getSimpleName(), c);
		}
		for(Class c : ClassFinder.getClasses("mod_wired.addons"))
		{
			this.getAutoLoader().itemImpl(c.getSimpleName(), c);
		}
	}
}
