package net.nillus.habbo.communication.messages.incoming.pets;


import net.nillus.habbo.communication.ClientMessage;
import net.nillus.habbo.communication.RequestHandler;
import net.nillus.habbo.communication.ServerMessage;
import net.nillus.habbo.game.items.pets.Pet;
import net.nillus.habbo.game.items.pets.RoomPet;
import net.nillus.habbo.game.rooms.instances.RoomInstance;
import net.nillus.habbo.game.rooms.instances.RoomTile;
import net.nillus.habbo.game.rooms.instances.RoomUser;
import net.nillus.habbo.sessions.Session;


public class PlacePetHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Read info
		int petId = request.readInt();
		int tileX = request.readInt();
		int tileY = request.readInt();

		// Allowed to place pets?
		RoomInstance room = session.getRoomInstance();
		if(!session.getRoomUser().hasRoomOwner && !room.getInfo().allowPets)
		{
			return;
		}

		// Can place pet here?
		RoomTile tile = room.getMap().getTile(tileX, tileY);
		if(tile == null || !tile.isOpen() || tile.is(RoomTile.WATER))
		{
			return;
		}
		
		// Pet in inventory?
		Pet pet = session.getPetInventory().removePet(petId);
		if(pet == null)
		{
			return;
		}
		
		// Spawn roomavatar
		RoomPet avatar = new RoomPet(pet);
		room.spawnAvatar(avatar, tile);
		
		// Sync location
		avatar.syncAndSaveLocationInfo();
		
		// Look to owner!
		RoomUser me = session.getRoomUser();
		avatar.lookTo(me.getTile(), false);
	}
}
