package net.nillus.habbo.communication.messages.incoming.room.items;

import java.util.List;

import net.nillus.habbo.communication.ClientMessage;
import net.nillus.habbo.communication.EServerMessage;
import net.nillus.habbo.communication.RequestHandler;
import net.nillus.habbo.communication.ServerMessage;
import net.nillus.habbo.core.Logger;
import net.nillus.habbo.game.catalog.Catalog;
import net.nillus.habbo.game.catalog.CatalogProduct;
import net.nillus.habbo.game.items.Item;
import net.nillus.habbo.sessions.Session;
import net.nillus.habbo.util.GameUtil;

public class OpenPresentHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Allowed to open presentboxes in this room?
		if(!session.getRoomUser().hasRoomOwner)
		{
			return;
		}

		// Find the item
		Item item = session.getRoomInstance().getItem(request.readInt());
		if(item == null || !item.definition.sprite.startsWith("present_"))
		{
			return;
		}

		// Pickup + delete the item
		session.getRoomInstance().pickupAndDeleteItem(item.id);

		// Get + delete giftinfo for the item
		String[] giftInfo = item.data.split(GameUtil.DATA_DELIMITER, 2);
		if(giftInfo.length != 2)
		{
			Logger.getInstance().warning(session, String.format("Opening presentbox #%d failed: gift info invalid!", item.id));
			return;
		}

		// 'Present OK'
		response.init(219);
		response.appendInt(item.id);
		session.send(response);

		// Redeem contents
		CatalogProduct product = Catalog.getInstance().getProduct(GameUtil.parseInt(giftInfo[0]));
		List<Item> contents = Catalog.getInstance().purchaseProduct(product, session.getUser(), giftInfo[1], false);
		if(product == null || contents == null)
		{
			session.sendNotif("Invalid presentbox contents, sorry!");
			return;
		}
		
		// Present contents
		response.init(EServerMessage.PresentOpened);
		response.appendStr(product.itemDefinition.getType());
		response.appendInt(product.itemDefinition.displayId);
		response.appendStr(product.itemDefinition.name);
		session.send(response);
		
		// Add to inventory
		for(Item tmpItem : contents)
		{
			session.getItemInventory().addItem(tmpItem, false);
		}
		session.getItemInventory().refresh();
	}
}
