package habbod.game.chatcommands.admin;

import habbod.ServerInstance;
import habbod.game.GameManager;
import habbod.game.access.AccessControl;
import habbod.game.catalog.Catalog;
import habbod.game.chatcommands.ChatCommandHandler;
import habbod.game.items.ItemManager;
import habbod.game.rooms.RoomManager;
import habbod.game.util.GameError;
import habbod.sessions.Session;

public class ReloadCommand extends ChatCommandHandler
{
	@Override
	public void handle(Session session, String[] args) throws GameError
	{
		GameManager game = ServerInstance.get().getGameMgr();
		
		String component = args[0];
		if(component.equals("roles"))
		{
			AccessControl ac = game.getAccessControl();
			ac.reloadUserRoles();
			
			session.sendNotif(String.format("Reloaded %d user roles.", ac.getAllUserRoles().size()));
		}
		else if(component.equals("models"))
		{
			RoomManager rm = game.getRoomMgr();
			rm.reloadModels();
			
			session.sendNotif(String.format("Reloaded %d room models.", rm.getAllRoomModels().size()));
		}
		else if(component.equals("itemdefinitions"))
		{
			ItemManager im = game.getItemMgr();
			im.reloadDefinitions();
			
			session.sendNotif(String.format("Reloaded %d item definitions.", im.getAllDefinitions().size()));
		}
		else if(component.equals("catalog"))
		{
			Catalog c = game.getCatalog();
			c.reload();
			
			session.sendNotif(String.format("Reloaded %d pages and %d products.", c.getAllPages().size(), c.getAllProducts().size()));
		}
		else
		{
			session.sendNotif("Reload 'roles', 'models', 'itemdefinitions', 'catalog'.");
		}
	}
}
