package habbod.game.chatcommands.core;

import habbod.ServerInstance;
import habbod.game.chatcommands.ChatCommandHandler;
import habbod.game.rooms.RoomManager;
import habbod.game.rooms.navigator.RoomNavigator;
import habbod.game.util.GameError;
import habbod.sessions.Session;
import habbod.sessions.SessionManager;

import java.lang.management.ManagementFactory;
import java.lang.management.OperatingSystemMXBean;
import java.lang.management.ThreadMXBean;


public class StatsCommand extends ChatCommandHandler
{
	@Override
	public void handle(Session session, String[] args) throws GameError
	{
		// Gather the required information
		Runtime runtime = Runtime.getRuntime();
		OperatingSystemMXBean os = ManagementFactory.getOperatingSystemMXBean();
		ThreadMXBean threads = ManagementFactory.getThreadMXBean();
		ServerInstance instance = ServerInstance.get();

		// Calculate uptime
		long uptime = (System.currentTimeMillis() - instance.getStartTime());
		long days = (uptime / (1000 * 60 * 60 * 24));
		long hours = (uptime - days * (1000 * 60 * 60 * 24)) / (1000 * 60 * 60);
		long minutes = (uptime - days * (1000 * 60 * 60 * 24) - hours * (1000 * 60 * 60)) / (1000 * 60);
		long seconds = (uptime - days * (1000 * 60 * 60 * 24) - hours * (1000 * 60 * 60) - minutes * (1000 * 60)) / (1000);

		// Build notification
		StringBuilder notif = new StringBuilder();

		// Uptime
		notif.append(instance.versionString());
		notif.append("\r");
		notif.append(String.format("Server uptime is %d day(s), %d hour(s), %d minute(s) and %d second(s).\r", days, hours, minutes, seconds));
		notif.append(String.format("There are %d user(s) online, and your session ID is #%d.\r", instance.getSessionMgr().authedSessions(), session.id));
		notif.append(String.format("There are %d room instance(s) loaded.\r\r", instance.getGameMgr().getRoomMgr().getAllRoomInstances().size()));

		// Environment
		notif.append(String.format("OS: %s %s (%s)\r", os.getName(), os.getArch(), os.getVersion()));
		notif.append(String.format("CPU cores: %d (active server threads: %d)\r", runtime.availableProcessors(), (threads.getThreadCount() - threads.getDaemonThreadCount())));
		notif.append(String.format("JVM memory usage: %d MB\r\r", instance.getMemoryUsage()));

		// Footer
		notif.append("Type :about for more server information.");

		// Send it
		session.sendSystemNotif(notif.toString());
	}
}
