package net.nillus.habbo.game.chatcommands.core;

import net.nillus.habbo.game.access.AccessControl;
import net.nillus.habbo.game.catalog.Catalog;
import net.nillus.habbo.game.chatcommands.ChatCommandHandler;
import net.nillus.habbo.game.items.ItemManager;
import net.nillus.habbo.game.rooms.RoomManager;
import net.nillus.habbo.game.util.GameError;
import net.nillus.habbo.sessions.Session;

public class ReloadCommand implements ChatCommandHandler
{
	@Override
	public String getDescription()
	{
		return "Reloads server components";
	}

	@Override
	public String getRequiredRight()
	{
		return "reload_server";
	}
	
	@Override
	public String[] getSyntax()
	{
		return new String[] { "component" };
	}

	@Override
	public void handle(Session session, String[] args) throws GameError
	{
		String component = args[0];
		if(component.equals("roles"))
		{
			AccessControl ac = AccessControl.getInstance();
			ac.reloadUserRoles();
			
			session.sendNotif(String.format("Reloaded %d user roles.", ac.getAllUserRoles().size()));
		}
		else if(component.equals("models"))
		{
			RoomManager rm = RoomManager.getInstance();
			rm.reloadModels();
			
			session.sendNotif(String.format("Reloaded %d room models.", rm.getAllRoomModels().size()));
		}
		else if(component.equals("itemdefinitions"))
		{
			ItemManager im = ItemManager.getInstance();
			im.reloadDefinitions();
			
			session.sendNotif(String.format("Reloaded %d item definitions.", im.getAllDefinitions().size()));
		}
		else if(component.equals("catalog"))
		{
			Catalog c = Catalog.getInstance();
			c.reload();
			
			session.sendNotif(String.format("Reloaded %d pages and %d products.", c.getAllPages().size(), c.getAllProducts().size()));
		}
	}
}
