package habbod.communication;

import habbod.core.Logger;
import habbod.core.Registry;
import habbod.storage.DatabaseResult;

public final class RequestRegistry
{
	private final static Registry<Integer, RequestHandler> registry = new Registry<Integer, RequestHandler>();

	public final static Registry<Integer, RequestHandler> get()
	{
		return registry;
	}
	
	public static void reload()
	{
		registry.clear();
		
		DatabaseResult result = habbod.ServerInstance.get().getDb().query("SELECT * FROM system_requesthandlers WHERE enabled=1");
		while(result.row())
		{
			int msgId = result.getInt("id");
			String className = String.format("habbod.communication.messages.incoming.%sHandler", result.getString("class"));
			
			try
			{
				// Create instance of handler
				RequestHandler handler = (RequestHandler) Class.forName(className).newInstance();
				
				// Add handler to registry
				registry.register(msgId, handler);
			}
			catch(Exception ex)
			{
				Logger.getInstance().error("RequestRegistry", String.format("Error registering requesthandler #%d ('%s').", msgId, className), ex);
			}
		}
		result.release();
		
		// Log result
		Logger.getInstance().info("RequestRegistry", String.format("Loaded %d entries.", registry.count()));
	}
}
