package habbod.remote;

import habbod.ServerInstance;
import habbod.core.Logger;
import habbod.core.Registry;
import habbod.storage.DatabaseResult;

public final class RemoteCmdRegistry
{
	private final static Registry<String, RemoteCommandHandler> registry = new Registry<String, RemoteCommandHandler>();

	public final static Registry<String, RemoteCommandHandler> get()
	{
		return registry;
	}
	
	public static void reload()
	{
		registry.clear();
		
		DatabaseResult result = ServerInstance.get().getDb().query("SELECT * FROM system_remotecommands WHERE enabled=1");
		while(result.row())
		{
			String className = result.getString("class");
			
			try
			{
				// Create instance of handler
				RemoteCommandHandler handler = (RemoteCommandHandler) Class.forName("habbod.remote.commands." + className).newInstance();
				
				// Add handler to registry
				registry.register(result.getString("command"), handler);
			}
			catch(Exception ex)
			{
				Logger.getInstance().error("RemoteCmdRegistry", String.format("Error registering remotecommand '%s'.", className), ex);
			}
		}
		result.release();
		
		// Log result
		Logger.getInstance().info("RemoteCmdRegistry", String.format("Loaded %d entries.", registry.count()));
	}
}
