package habbod.util;

import habbod.core.Logger;

import org.keplerproject.luajava.LuaState;
import org.keplerproject.luajava.LuaStateFactory;

public final class ScriptHandler
{
	private final LuaState lua;
	
	private ScriptHandler(String file)
	{
		this.lua = LuaStateFactory.newLuaState();
		this.lua.openLibs();
		this.lua.LdoFile(file);
	}
	
	public void destroy()
	{
		this.lua.close();
	}
	
	public void call(String func, Object... args)
	{
		try
		{
			this.lua.getGlobal(func);
			for(Object arg : args)
			{
				this.lua.pushObjectValue(arg);
			}
			this.lua.call(args.length, 0);
		}
		catch(Exception ex)
		{
			Logger.getInstance().error("ScriptHandler", "Error calling function!", ex);
		}
	}
	
	public static ScriptHandler loadScript(String script)
	{
		return new ScriptHandler("scripts/" + script + ".lua");
	}
}
