package net.nillus.habbo.communication;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.List;

import net.nillus.habbo.net.codec.Base64Encoding;
import net.nillus.habbo.net.codec.WireEncoding;

public final class ServerMessage
{
	public final static byte MSG_EOF = (byte) 1;

	public int type;
	private final ByteArrayOutputStream buff;

	public ServerMessage()
	{
		this.buff = new ByteArrayOutputStream();
	}

	public ServerMessage(int type)
	{
		this();
		this.init(type);
	}

	public final void reset()
	{
		this.buff.reset();
	}

	public final void init(int type)
	{
		this.type = type;
		this.reset();
	}

	public final void append(byte b)
	{
		this.buff.write(b);
	}

	public final void append(char c)
	{
		this.append((byte) c);
	}

	public final void append(byte[] bytes)
	{
		try
		{
			this.buff.write(bytes);
		}
		catch(IOException e)
		{
			e.printStackTrace();
		}
	}

	public final void append(String s)
	{
		if(s.length() > 0)
		{
			this.append(s.getBytes());
		}
	}

	public final void appendBool(boolean b)
	{
		if(b)
		{
			this.append(WireEncoding.POSITIVE);
		}
		else
		{
			this.append(WireEncoding.NEGATIVE);
		}
	}
	
	public final void appendBool(Boolean b)
	{
		this.appendBool((b != null && b));
	}

	public final void appendInt(int i)
	{
		if(i == 0)
		{
			this.appendBool(false);
		}
		else if(i == 1)
		{
			this.appendBool(true);
		}
		else
		{
			this.append(WireEncoding.encodeInt(i));
		}
	}
	
	public final void appendStr(char c)
	{
		this.appendStr(c, (byte)2);
	}
	
	public final void appendStr(char c, byte tail)
	{
		this.append(c);
		this.append(tail);
	}
	
	public final void appendStr(int i)
	{
		this.appendStr(i, (byte)2);
	}
	
	public final void appendStr(int i, byte tail)
	{
		this.append(Integer.toString(i));
		this.append(tail);
	}

	public final void appendStr(float f)
	{
		this.appendStr(f, (byte)2);
	}
	
	public final void appendStr(float f, byte tail)
	{
		this.append(Float.toString(f));
		this.append(tail);
	}
	
	public final void appendStr(String s)
	{
		this.appendStr(s, (byte) 2);
	}

	public final void appendStr(String s, byte tail)
	{
		this.append(s != null ? s : "");
		this.append(tail);
	}

	public final void appendStrLine(String s)
	{
		this.appendStr(s, (byte) 13);
	}

	public final void appendStrList(String[] arr)
	{
		this.appendInt(arr.length);
		for(int x = 0; x < arr.length; x++)
		{
			this.appendStr(arr[x]);
		}
	}
	
	public final void appendStrList(List<String> list)
	{
		this.appendInt(list.size());
		for(int x = 0; x < list.size(); x++)
		{
			this.appendStr(list.get(x));
		}
	}
	
	public final void appendObject(SerializableObject obj)
	{
		obj.serialize(this);
	}

	public final byte[] getPayload()
	{
		return this.buff.toByteArray();
	}
	
	public final byte[] getResult()
	{
		this.buff.write(ServerMessage.MSG_EOF);
		return this.buff.toByteArray();
	}

	public final String header()
	{
		return new String(Base64Encoding.encodeInt(this.type, 2));
	}

	public final int length()
	{
		if(this.buff.size() > 0)
		{
			return this.buff.size() - 2;
		}
		else
		{
			return 0;
		}
	}

	@Override
	public final String toString()
	{
		return new String(this.header() + new String(this.buff.toByteArray()));
	}
	
	public final static ServerMessage create(final int type)
	{
		return new ServerMessage(type);
	}
}
