package net.nillus.habbo.tasks;

import java.util.Iterator;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

import net.nillus.habbo.core.Logger;

public final class TaskScheduler implements Runnable
{
	private final List<ScheduledTask> tasks;
	
	public TaskScheduler()
	{
		this.tasks = new CopyOnWriteArrayList<ScheduledTask>();
	}
	
	public void scheduleSingleExecute(Task task, long delayMillis)
	{
		this.schedule(task, delayMillis, false);
	}
	
	public void scheduleRepeater(Task task, long intervalMillis)
	{
		this.schedule(task, intervalMillis, true);
	}
	
	private void schedule(Task task, long intervalMillis, boolean isRepeating)
	{
		ScheduledTask stask = new ScheduledTask(task, intervalMillis, isRepeating);
		stask.lastTime = System.currentTimeMillis();
		this.tasks.add(stask);
	}
	
	@Override
	public void run()
	{
		long nowTime = System.currentTimeMillis();
		Iterator<ScheduledTask> iterator = this.tasks.iterator();
		while(iterator.hasNext())
		{
			// Get the task
			ScheduledTask stask = iterator.next();
			
			// Needs executing?
			if((nowTime - stask.lastTime) >= stask.interval)
			{
				try
				{
					stask.task.executeTask();
				}
				catch(Exception ex)
				{
					Logger.getInstance().error("TaskScheduler", String.format("Error executing task %s.", stask.task.getClass().getName()), ex);
				}
				finally
				{
					if(stask.isRepeating)
					{
						stask.lastTime = nowTime;
					}
					else
					{
						iterator.remove();
					}
				}
			}
		}
	}
}
