package habbod.game.catalog;

import habbod.game.catalog.filters.PrePurchaseFilter;
import habbod.game.items.EItemFlags;
import habbod.game.items.ItemDefinition;
import habbod.game.users.User;

public final class CatalogProduct
{
	public int id;
	public int pageId;
	public CatalogPage page;

	public String code;
	public int itemDefinitionId;
	public ItemDefinition itemDefinition;
	public int itemAmount;

	public int costCredits;
	public int costPixels;
	public int costQuestPoints;

	private PrePurchaseFilter prepurchaseFilter;
	
	public final boolean isPhysicalItem()
	{
		return (this.itemDefinition != null && this.itemDefinition.is(EItemFlags.FLOORITEM) || this.itemDefinition.is(EItemFlags.WALLITEM));
	}
	
	public final char getType()
	{
		return this.itemDefinition.getType();
	}
	
	public boolean isPurchasableBy(User user)
	{
		// Can user access page?
		if(this.page == null || !this.page.isAccessibleTo(user))
		{
			return false;
		}
		
		// Can user actually afford this?
		if(user.credits < this.costCredits || user.pixels < this.costPixels)
		{
			return false;
		}

		// Purchasable!
		return true;
	}
}
