package habbod.game.chatcommands;

import habbod.core.Logger;
import habbod.core.Registry;
import habbod.storage.DatabaseResult;

public final class ChatCmdRegistry
{
	private final static Registry<String, ChatCommandHandler> registry = new Registry<String, ChatCommandHandler>();

	public final static Registry<String, ChatCommandHandler> get()
	{
		return registry;
	}
	
	public static void reload()
	{
		registry.clear();
		
		DatabaseResult result = habbod.ServerInstance.get().getDb().query("SELECT * FROM system_chatcommands WHERE enabled=1");
		while(result.row())
		{
			String className = result.getString("class");
			
			try
			{
				// Create instance of handler
				ChatCommandHandler handler = (ChatCommandHandler) Class.forName("habbod.game.chatcommands." + className).newInstance();
				
				// Add data to handler
				handler.initialize(result.getString("command"), result.getString("description"), result.getString("required_right"), result.getString("syntax"));
				
				// Add handler to registry
				registry.register(handler.getCommand(), handler);
			}
			catch(Exception ex)
			{
				Logger.getInstance().error("ChatCmdRegistry", String.format("Error registering chatcommand '%s'.", className), ex);
			}
		}
		result.release();
	
		// Log result
		Logger.getInstance().info("ChatCmdRegistry", String.format("Loaded %d entries.", registry.count()));
	}
}