package habbod.game.items;

import habbod.ServerInstance;
import habbod.core.Logger;
import habbod.game.rooms.items.RoomFloorItem;
import habbod.game.rooms.items.RoomItem;
import habbod.game.rooms.items.RoomWallItem;
import habbod.storage.DatabaseResult;
import habbod.storage.SQLHelper;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public final class ItemManager
{
	private Map<Integer, ItemDefinition> definitions;

	public ItemManager()
	{
		this.definitions = Collections.emptyMap();
	}

	public void reloadDefinitions()
	{
		// Create new collection
		Map<Integer, ItemDefinition> tmpDefinitions = new HashMap<Integer, ItemDefinition>();

		// Load definitions from database
		DatabaseResult result = ServerInstance.get().getDb().query("SELECT * FROM itemdefinitions");
		while(result.row())
		{
			ItemDefinition definition = ItemDefinition.parse(result);
			tmpDefinitions.put(definition.id, definition);
		}
		result.release();

		// Set the instance field
		this.definitions = Collections.unmodifiableMap(tmpDefinitions);

		// Done!
		Logger.getInstance().info("ItemMgr", String.format("Reloaded %d item definitions.", this.definitions.size()));
	}

	public ItemDefinition getDefinition(int definitionId)
	{
		return this.definitions.get(definitionId);
	}
	
	public List<ItemDefinition> getDefinitionsByClass(Class cls)
	{
		List<ItemDefinition> definitions = new ArrayList<ItemDefinition>();
		for(ItemDefinition definition : this.definitions.values())
		{
			
		}
		
		return definitions;
	}

	public Collection<ItemDefinition> getAllDefinitions()
	{
		return this.definitions.values();
	}

	public void storeItem(ItemBase item, int ownerId)
	{
		if(item.id <= 0)
		{
			item.id = ServerInstance.get().getDb().executeReturnKey("INSERT INTO items(definition_id,owner_id,state,data) VALUES (" + item.definition.id + "," + ownerId + ",'" + SQLHelper.escape(item.state) + "','" + SQLHelper.escape(item.data) + "')");
		}
	}
	
	public List<RoomItem> getItemsForRoom2(int roomId)
	{
		List<RoomItem> items = new ArrayList<RoomItem>();

		DatabaseResult result = ServerInstance.get().getDb().query("SELECT * FROM items WHERE room_id=" + roomId);
		while(result.row())
		{
			ItemDefinition definition = this.getDefinition(result.getInt("definition_id"));
			if(definition != null)
			{
				RoomItem item = definition.createInstance();
				if(item != null)
				{
					item.id = result.getInt("id");
					item.state = result.getString("state");
					item.data = result.getString("data");
					if(item instanceof RoomWallItem)
					{
						RoomWallItem wallItem = (RoomWallItem)item;
						wallItem.position = result.getString("wallposition");
					}
					else
					{
						RoomFloorItem floorItem = (RoomFloorItem)item;
						floorItem.x = result.getInt("x");
						floorItem.y = result.getInt("y");
						floorItem.z = result.getFloat("z");
						floorItem.rotation = result.getByte("rotation");
					}
					
					items.add(item);
				}
			}
		}
		result.release();

		return items;
	}
	
	public int getTeleporterRoomId(int itemId)
	{
		return ServerInstance.get().getDb().queryInt("SELECT room_id FROM items WHERE id=" + itemId);
	}

	/* Begin singleton */
	private final static ItemManager instance = new ItemManager();

	public final static ItemManager getInstance()
	{
		return instance;
	}
	/* End singleton */
}
