package habbod.game.rooms.items.wired.conditions;

import habbod.game.rooms.instances.RoomTile;
import habbod.game.rooms.instances.RoomUser;
import habbod.game.rooms.items.RoomFloorItem;
import habbod.game.rooms.items.wired.WiredCondition;

import java.util.List;



public class WiredItemsHaveItemsOnTopCondition extends WiredCondition
{
	private final static int PARAM_REQUIREALL = 0;
	
	@Override
	protected int getInterface()
	{
		return 7;
	}
	
	@Override
	protected int getAmountOfParams()
	{
		return 1;
	}
	
	@Override
	public boolean evaluate(RoomUser user)
	{
		// Check items
		List<RoomFloorItem> selected = this.getSelectedItems();
		int count = 0;
		for(RoomFloorItem item : selected)
		{
			// Check all tiles of the selected item
			for(RoomTile tile : item.getTiles())
			{
				// Top item of the tile is higher than the selected item?
				if(tile.topItem.z > item.z)
				{
					count++;
					break; // This item is okay, check the next item!
				}
			}
		}
		
		// Selected item hasn't got items on top
		if(this.getParam(PARAM_REQUIREALL) == 0)
		{
			// Atleast one is needed
			return count > 0;
		}
		else
		{
			// All items need to have items on 'em!
			return (count == selected.size());
		}
	}
}
