package habbod.game.rooms.items.wired.triggers;

import habbod.events.EventListener;
import habbod.game.rooms.items.RoomFloorItem;
import habbod.game.rooms.items.wired.WiredTrigger;
import habbod.sessions.Session;


public class WiredUserSaysPhraseTrigger extends WiredTrigger
{
	private final static int PARAM_OWNERONLY = 0;
	
	@Override
	protected int getInterface()
	{
		return 0;
	}
	
	@Override
	protected int getAmountOfParams()
	{
		return 1;
	}
	
	@Override
	protected boolean suppliesUser()
	{
		return true;
	}
	
	public static class ChatHook implements EventListener
	{
		@Override
		public boolean handle(Object subject, Object args)
		{
			Session session = (Session)subject;
			String text = (String)args;
			
			// Matches the secret word?
			for(RoomFloorItem item : session.getRoomInstance().getFloorItems())
			{
				// Phrase trigger?
				if(item instanceof WiredUserSaysPhraseTrigger)
				{
					// Phrase matches?
					WiredUserSaysPhraseTrigger wired = (WiredUserSaysPhraseTrigger)item;
					if(text.equals(wired.getText()))
					{
						// Owner-only check (if enabled)
						if(wired.getParam(PARAM_OWNERONLY) == 0 || session.getRoomUser().hasRoomOwner)
						{
							// Let the user know he got the phrase correct
							session.sendWhisperNotif(text);
							
							// Fire the trigger
							wired.execute(session.getRoomUser());
							
							// Don't chat the secret to room ;)
							return false;
						}
					}
				}
			}
			
			// Nothing happened!
			return true;
		}
	}
}
