package net.nillus.habbo.game.items.pets;

import net.nillus.habbo.habbod;
import net.nillus.habbo.communication.SerializableObject;
import net.nillus.habbo.communication.ServerMessage;
import net.nillus.habbo.game.util.GameUtil;
import net.nillus.habbo.storage.DatabaseResult;

public final class Pet implements SerializableObject
{
	public final static int[] LEVELS = new int[] { 100, 200, 400, 600, 1000, 1300, 1800, 2400, 3200, 4300, 7200, 8500, 10100, 13300, 17500, 23000, 51900 };
	public final static int MAX_LEVEL = 20;
	public final static int MAX_ENERGY = 100;
	public final static int MAX_NUTRITION = 150;

	public int id;
	public String name;

	public int type;
	public int breed;
	public String color;

	public int ownerId;
	public String ownerName;
	public long createdTime;

	public int level;
	public int experience;
	public int energy;
	public int nutrition;
	public int respect;

	public boolean requiresUpdate;
	
	public int roomId;
	public int x;
	public int y;
	public float z;

	public String getFigure()
	{
		return this.type + " " + String.format("%03d", this.breed) + " " + this.color;
	}

	public int getAgeInDays()
	{
		return GameUtil.daysSince(this.createdTime);
	}

	public void addExperience(int experience)
	{
		//if(this.experience >= Pet.LEVELS[Pet.MAX_LEVEL - 1]) return;
		
		this.experience += experience;
	}
	
	public int getExperienceGoal()
	{
		return Pet.LEVELS[this.level + 1];
	}
	
	public void recalcLevel()
	{
		this.level = Pet.LEVELS.length + 1;
		for(int tmpLevel = 0; tmpLevel < Pet.LEVELS.length; ++tmpLevel)
		{
			if(this.experience < Pet.LEVELS[tmpLevel])
			{
				this.level = tmpLevel + 1;
				break;
			}
		}
	}
	
	public void save()
	{
		habbod.getInstance().getDb().execute("UPDATE pets SET experience=" + this.experience + ",energy=" + this.energy + ",nutrition=" + this.nutrition + ",respect=" + this.respect + ",room_id=" + this.roomId + ",x=" + this.x + ",y=" + this.y + ",z=" + this.z + " WHERE id=" + this.id);
	}
	
	@Override
	public void serialize(ServerMessage msg)
	{
		msg.appendInt(this.id);
		msg.appendStr(this.name);
		msg.appendInt(this.type);
		msg.appendInt(this.breed);
		msg.appendStr(this.color);
	}

	
	public static Pet parse(DatabaseResult row)
	{
		Pet pet = new Pet();
		pet.id = row.getInt("id");
		pet.name = row.getString("name");
		pet.type = row.getByte("type");
		pet.breed = row.getByte("breed");
		pet.color = row.getString("color");
		pet.experience = row.getInt("experience");
		pet.energy = row.getInt("energy");
		pet.nutrition = row.getInt("nutrition");
		pet.respect = row.getInt("respect");
		pet.ownerId = row.getInt("owner_id");
		pet.ownerName = row.getString("owner");
		pet.createdTime = row.getTime("created_at");
		pet.roomId = row.getInt("room_id");
		pet.x = row.getInt("x");
		pet.y = row.getInt("y");
		pet.z = row.getFloat("z");
		
		pet.recalcLevel();
		
		return pet;
	}
}
