package net.nillus.habbo.game.items.pets;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import net.nillus.habbo.habbod;
import net.nillus.habbo.communication.ServerMessage;
import net.nillus.habbo.sessions.Session;
import net.nillus.habbo.storage.DatabaseResult;

public final class PetInventory
{
	private final Session session;
	private final Map<Integer, Pet> pets;

	public PetInventory(Session session)
	{
		this.session = session;
		this.pets = new HashMap<Integer, Pet>();
	}

	public void reload()
	{
		// Clear old items
		this.pets.clear();

		// Reload items
		DatabaseResult result = habbod.getInstance().getDb().query("SELECT p.*,u.username AS owner FROM pets p JOIN users u ON u.id=p.owner_id WHERE owner_id=" + this.session.getUser().id + " AND room_id=0");
		while(result.row())
		{
			Pet pet = Pet.parse(result);
			if(pet != null)
			{
				this.pets.put(pet.id, pet);
			}
		}
		result.release();
	}

	public boolean containsPet(int petId)
	{
		return this.pets.containsKey(petId);
	}

	public Pet getPet(int petId)
	{
		return this.pets.get(petId);
	}

	public void addPet(Pet pet)
	{
		// Set the pet info
		this.pets.put(pet.id, pet);

		// Notify user
		ServerMessage msg = new ServerMessage(603);
		msg.appendObject(pet);
		this.session.send(msg);
	}

	public Pet removePet(int petId)
	{
		Pet pet = this.pets.remove(petId);
		if(pet != null)
		{
			ServerMessage msg = new ServerMessage(604);
			msg.appendInt(petId);
			this.session.send(msg);
		}

		return pet;
	}

	public final Collection<Pet> getPets()
	{
		return this.pets.values();
	}
}
