package net.nillus.habbo.game.messenger;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import net.nillus.habbo.habbod;
import net.nillus.habbo.core.Logger;
import net.nillus.habbo.storage.DatabaseResult;
import net.nillus.habbo.storage.SQLHelper;

public final class InstantMessengerService
{
	private final Map<Integer, MessengerBuddy> registry;

	private InstantMessengerService()
	{
		this.registry = new ConcurrentHashMap<Integer, MessengerBuddy>();
	}
	
	public MessengerBuddy getBuddy(int userId, boolean attemptDbLoad, boolean incrementRef)
	{
		// Lookup the registry
		MessengerBuddy buddy = this.registry.get(userId);

		// Not found? Attempt load from database, then add to registry
		if(buddy == null && attemptDbLoad)
		{
			DatabaseResult result = habbod.getInstance().getDb().query("SELECT id,username,figure,motto,lastonline,im_blockrequests FROM users WHERE id=" + userId);
			if(result.row())
			{
				buddy = MessengerBuddy.parse(result);
				this.registry.put(buddy.userId, buddy);
			}
			result.release();
		}
		
		// Increment refcount?
		if(incrementRef && buddy != null)
		{
			buddy.refCount++;
		}
		
		return buddy;
	}

	public Map<Integer, MessengerBuddy> getBuddyListForUser(int userId)
	{
		Map<Integer, MessengerBuddy> buddies = new HashMap<Integer, MessengerBuddy>();

		DatabaseResult result = habbod.getInstance().getDb().query("SELECT u.id,u.username,u.figure,u.motto,u.lastonline,u.im_blockrequests FROM users u INNER JOIN user_imbuddies rel ON u.id=rel.user_id WHERE rel.user2_id=" + userId + " AND rel.accepted=1 UNION ALL SELECT u.id,u.username,u.figure,u.motto,u.lastonline,u.im_blockrequests FROM users u INNER JOIN user_imbuddies rel ON u.id=rel.user2_id WHERE rel.user_id=" + userId + " AND rel.accepted=1");
		while(result.row())
		{
			// First lookup the registry...
			int buddyUserId = result.getInt("id");
			MessengerBuddy buddy = this.getBuddy(buddyUserId, false, true);

			// Not in registry?
			if(buddy == null)
			{
				// Accept the database data and add to the registry
				buddy = MessengerBuddy.parse(result);
				buddy.refCount++;
				this.registry.put(buddy.userId, buddy);
			}

			// Add to the buddylist
			buddies.put(buddy.userId, buddy);
		}
		result.release();

		return buddies;
	}

	public Map<Integer, String> getBuddyRequestsForUser(int userId)
	{
		Map<Integer, String> requests = new HashMap<Integer, String>();

		DatabaseResult result = habbod.getInstance().getDb().query("SELECT u.id,u.username FROM users u JOIN user_imbuddies rel ON u.id=rel.user_id WHERE rel.user2_id=" + userId + " AND rel.accepted=0");
		while(result.row())
		{
			requests.put(result.getInt("id"), result.getString("username"));
		}
		result.release();

		return requests;
	}

	public List<MessengerBuddy> getUserSearchResults(String criteria)
	{
		List<MessengerBuddy> buddies = new ArrayList<MessengerBuddy>();

		DatabaseResult result = habbod.getInstance().getDb().query("SELECT id,username,figure,motto,lastonline,im_blockrequests FROM users WHERE username LIKE '" + SQLHelper.escape(criteria) + "%' LIMIT 30;");
		while(result.row())
		{
			// Lookup registry first, optionally accept database data, but don't add to registry
			MessengerBuddy buddy = this.getBuddy(result.getInt("id"), false, false);
			if(buddy == null)
			{
				buddy = MessengerBuddy.parse(result);
			}
			buddies.add(buddy);
		}
		result.release();

		return buddies;
	}

	public MessengerBuddy getBuddyInfoByUsername(String username)
	{
		// Fetch user info by username, or null if user does not exist
		DatabaseResult result = habbod.getInstance().getDb().query("SELECT id,username,figure,motto,lastonline,im_blockrequests FROM users WHERE username='" + SQLHelper.escape(username) + "'");
		MessengerBuddy buddy = result.row() ? MessengerBuddy.parse(result) : null;
		result.release();

		return buddy;
	}

	public void createBuddyRequest(int fromUserId, String fromUsername, int toUserId)
	{
		// Delete old requests and create new unaccepted relation in database
		habbod.getInstance().getDb().execute("DELETE FROM user_imbuddies WHERE ((user_id=" + fromUserId + " AND user2_id=" + toUserId + ") OR (user_id=" + toUserId + " AND user2_id=" + fromUserId + ")) AND accepted=0 LIMIT 1");
		habbod.getInstance().getDb().execute("INSERT INTO user_imbuddies(user_id,user2_id) VALUES (" + fromUserId + "," + toUserId + ")");

		// Notify buddy if online
		MessengerBuddy receiver = this.getBuddy(toUserId, true, false);
		if(receiver != null && receiver.online())
		{
			receiver.messenger.addRequest(fromUserId, fromUsername);
		}
	}

	public void acceptBuddyRequest(int toUserId, int fromUserId)
	{
		// Update the relation in the database
		habbod.getInstance().getDb().execute("UPDATE user_imbuddies SET accepted=1 WHERE user_id=" + fromUserId + " AND user2_id=" + toUserId);

		MessengerBuddy buddy;

		// to-user online?
		if((buddy = this.getBuddy(toUserId, false, false)) != null)
		{
			// Add the from-user to it's buddy list
			if(buddy.online())
			{
				buddy.messenger.addBuddy(this.getBuddy(fromUserId, true, false));
			}
		}

		// from-user online?
		if((buddy = this.getBuddy(fromUserId, false, false)) != null)
		{
			// Add the to-user to it's buddylist
			if(buddy.online())
			{
				buddy.messenger.addBuddy(this.getBuddy(toUserId, true, false));
			}
		}
	}

	public void deleteRelation(int userId1, int userId2)
	{
		habbod.getInstance().getDb().execute("DELETE FROM user_imbuddies WHERE (user_id=" + userId1 + " AND user2_id=" + userId2 + ") OR (user_id=" + userId2 + " AND user2_id=" + userId1 + ") LIMIT 1");
	}
	
	public void cleanup()
	{
		// Remove buddies that are no longer online / in a buddy list
		int removed = 0;
		Iterator<MessengerBuddy> it = this.registry.values().iterator();
		while(it.hasNext())
		{
			MessengerBuddy buddy = it.next();
			if(buddy.refCount <= 0)
			{
				it.remove();
				removed++;
			}
		}

		// Anything removed?
		if(removed > 0)
		{
			Logger.getInstance().debug("InstantMessengerService", String.format("Cleaned %d buddies.", removed));
		}
	}
	
	public void clearRegistry()
	{
		this.registry.clear();
	}
	
	public int registrySize()
	{
		return this.registry.size();
	}

	/* Begin singleton */
	private final static InstantMessengerService instance = new InstantMessengerService();

	public final static InstantMessengerService getInstance()
	{
		return instance;
	}
	/* End singleton */
}
