package net.nillus.habbo.game.users.badges;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import net.nillus.habbo.habbod;
import net.nillus.habbo.core.Logger;
import net.nillus.habbo.storage.DatabaseConnection;
import net.nillus.habbo.storage.DatabaseResult;

public class BadgeInventory
{
	private int userId;
	private List<Badge> badges;
	private List<Badge> equipped;
	private boolean requiresSave;

	public BadgeInventory(int userId)
	{
		this.userId = userId;
		this.badges = new ArrayList<Badge>(3);
		this.equipped = new ArrayList<Badge>(3);
		this.requiresSave = false;
	}

	public void clear()
	{
		this.badges.clear();
		this.equipped.clear();
	}

	public void reload()
	{
		// Clear badges + equipped maps
		this.clear();

		// Reload available badges from database
		DatabaseResult result = habbod.getInstance().getDb().query("SELECT badge,slot FROM user_badges WHERE user_id=" + this.userId);
		while(result.row())
		{
			Badge badge = new Badge();
			badge.code = result.getString("badge");
			badge.slot = result.getByte("slot");
		}
		result.release();
	}

	public boolean equip(byte slot, String badge)
	{
		for(Badge b : this.badges)
		{
			if(b.code.equals(badge))
			{
				b.slot = slot;
				return true;
			}
		}
		
		return false;
	}

	public void requireSave()
	{
		this.requiresSave = true;
	}

	public boolean requiresSave()
	{
		return this.requiresSave;
	}

	public void save()
	{
		// Get dedicated database connection
		DatabaseConnection conn = habbod.getInstance().getDb().getConnMgr().getConnection();
		if(conn == null)
		{
			return;
		}

		// Create PreparedStatement, batch everything and execute
		try
		{
			PreparedStatement query = conn.getInnerConnection().prepareStatement("UPDATE user_badges SET slot=? WHERE user_id=" + this.userId + " AND badge=?");
			for(Badge badge : this.badges)
			{
				query.setByte(1, badge.slot);
				query.setString(2, badge.code);
				query.addBatch();
			}
			query.executeBatch();
			query.close();
		}
		catch(SQLException ex)
		{
			Logger.getInstance().error("User", "Error saving badges.", ex);
		}

		// Release the dedicated database connection
		habbod.getInstance().getDb().getConnMgr().releaseConnection(conn);

		// Saved!
		this.requiresSave = false;
	}

	public List<Badge> getBadges()
	{
		return this.badges;
	}

	public List<Badge> getEquippedBadges()
	{
		List<Badge> equipped = new ArrayList<Badge>(6);
		for(Badge badge : this.badges)
		{
			if(badge.slot > 0)
			{
				equipped.add(badge);
			}
		}
		
		return equipped;
	}
}
