package habbod.communication.messages.incoming.inventory.items;

import habbod.communication.ClientMessage;
import habbod.communication.EServerMessage;
import habbod.communication.RequestHandler;
import habbod.communication.ServerMessage;
import habbod.game.items.InventoryItem;
import habbod.game.rooms.instances.RoomInstance;
import habbod.sessions.Session;

public class ApplyRoomDecorationHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Allowed to apply decoration?
		if(session.getRoomUser() == null || !session.getRoomUser().hasRoomOwner)
		{
			return;
		}

		// Verify decoration item
		InventoryItem item = session.getItemInventory().getItem(request.readInt());
		if(item == null)
		{
			return;
		}

		// Remove from inventory and delete
		session.getItemInventory().removeItem(item.id);
		item.delete();
		
		// Apply new props
		RoomInstance room = session.getRoomInstance();
		if(item.definition.data.equals("wallpaper"))
		{
			room.getInfo().wallpaper = item.state;
		}
		else if(item.definition.data.equals("floor"))
		{
			room.getInfo().floor = item.state;
		}
		else if(item.definition.data.equals("landscape"))
		{
			room.getInfo().landscape = item.state;
		}
		room.getInfo().saveDecoration();

		// Notify room
		response.init(EServerMessage.RoomProperty);
		response.appendStr(item.definition.sprite);
		response.appendStr(item.state);
		room.broadcast(response);
	}
}
