package habbod.communication.messages.incoming.navigator;

import habbod.communication.ClientMessage;
import habbod.communication.EServerMessage;
import habbod.communication.RequestHandler;
import habbod.communication.ServerMessage;
import habbod.game.rooms.navigator.NavigatorFrontpageItem;
import habbod.game.rooms.navigator.RoomNavigator;
import habbod.sessions.Session;

import java.util.List;


public class GetNavigatorFrontpageHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Get the configured frontpage items of the navigator
		List<NavigatorFrontpageItem> fpis = RoomNavigator.getInstance().getFrontpageItems();

		// Serialize the frontpage items
		response.init(EServerMessage.NavigatorFrontPageResult);
		response.appendInt(fpis.size());
		for(NavigatorFrontpageItem fpi : fpis)
		{
			this.serializeFrontpageItem(fpi, response);
			
			// TODO: children
		}
		session.send(response);
	}
	
	private void serializeFrontpageItem(NavigatorFrontpageItem fpi, ServerMessage msg)
	{
		msg.appendInt(fpi.id);
		msg.appendStr(fpi.name);
		msg.appendStr(fpi.description);
		msg.appendBool(fpi.showDetails);
		msg.appendStr(fpi.name);
		msg.appendStr(fpi.imageExternal);
		msg.appendInt(fpi.parentId);
		msg.appendInt(fpi.room != null ? fpi.room.usersNow : 0);
		msg.appendInt(fpi.category != null ? 4 : (fpi.room.isGuestRoom() ? 2 : 3));
		if(fpi.category != null)
		{
			msg.appendBool(false); // Auto expand?
		}
		else if(fpi.room.isGuestRoom())
		{
			msg.appendObject(fpi.room);
		}
		else if(fpi.room.isPublicRoom())
		{
			msg.appendStr(fpi.imageExternal); 
			msg.appendInt(0); // unk
			msg.appendInt(0); // Room part
			msg.appendStr(fpi.room.files);
			msg.appendInt(fpi.room.usersMax);
			msg.appendInt(fpi.room.id);
		}
	}
}
