package net.nillus.habbo.communication.messages.incoming.inventory.badges;

import java.util.List;

import net.nillus.habbo.communication.ClientMessage;
import net.nillus.habbo.communication.EServerMessage;
import net.nillus.habbo.communication.RequestHandler;
import net.nillus.habbo.communication.ServerMessage;
import net.nillus.habbo.game.users.badges.Badge;
import net.nillus.habbo.game.users.badges.BadgeInventory;
import net.nillus.habbo.sessions.Session;

public class SetActivatedBadgesHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Get badge inventory
		BadgeInventory inventory = session.getBadgeInventory();
		if(inventory == null)
		{
			return;
		}
		
		// Parse new slot content from the request
		while(request.remaining() > 0)
		{
			// Parse this slot
			byte slot = (byte) request.readInt();
			String badge = request.readStr();

			// Attempt to equip this badge
			inventory.equip(slot, badge);
		}

		// Force update on logout
		inventory.requireSave();

		// Get equipped badges
		List<Badge> equipped = inventory.getEquippedBadges();
		
		// Build update msg
		response.init(EServerMessage.UserBadges);
		response.appendInt(session.getUser().id);
		response.appendInt(equipped.size());
		for(Badge badge : equipped)
		{
			response.appendInt(badge.slot);
			response.appendStr(badge.code);
		}

		// Send update msg to client(s)
		if(session.inRoom())
		{
			session.getRoomInstance().broadcast(response);
		}
		else
		{
			session.send(response);
		}
	}
}
