package net.nillus.habbo.communication.messages.incoming.moderation;

import net.nillus.habbo.communication.ClientMessage;
import net.nillus.habbo.communication.RequestHandler;
import net.nillus.habbo.communication.ServerMessage;
import net.nillus.habbo.game.moderation.ModerationCenter;
import net.nillus.habbo.game.moderation.ModerationTicket;
import net.nillus.habbo.sessions.Session;
import net.nillus.habbo.sessions.SessionManager;


public class ModCloseTicketHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request, ServerMessage response)
	{
		// Has mod?
		if(!session.getUser().rights.contains("fuse_mod"))
		{
			return;
		}
		
		// Get ticket
		int result = request.readInt();
		int unk = request.readInt();
		int ticketId = request.readInt();
		ModerationTicket ticket = ModerationCenter.getInstance().getTicket(ticketId);
		
		// Ticket exists and picked by this user?
		if(ticket != null && ticket.status == ModerationTicket.STATUS_PICKED && ticket.pickerId == session.getUser().id)
		{
			// Closing as what?
			if(result == 1)
			{
				ticket.status = ModerationTicket.STATUS_INVALID;
			}
			else if(result == 2)
			{
				ticket.status = ModerationTicket.STATUS_ABUSIVE;
			}
			else
			{
				ticket.status = ModerationTicket.STATUS_RESOLVED;
				result = 0;
			}
			
			// Save status
			ticket.saveStatus();
			
			// Broadcast update
			ModerationCenter.getInstance().broadcastTicket(ticket);
			
			// Feedback to caller
			Session caller = SessionManager.getInstance().getSessionByUserId(ticket.userId);
			if(caller != null)
			{
				response.init(540);
				response.appendInt(result);
				caller.send(response);
			}
		}
	}
}
