package mod_effects;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import net.nillus.habbo.habbod;
import net.nillus.habbo.communication.messages.outgoing.ServerMessage;
import net.nillus.habbo.sessions.Session;
import net.nillus.habbo.storage.DatabaseResult;

public final class EffectInventory
{
	private final Session session;
	private int currentEffect;
	private final List<UserEffect> effects;

	public EffectInventory(Session session)
	{
		this.session = session;
		this.currentEffect = 0;//-1;
		this.effects = new ArrayList<UserEffect>();
	}

	public void clear()
	{
		this.effects.clear();
	}

	public void reload()
	{
		// Clear old effects
		this.clear();

		// Load new ones from database
		DatabaseResult result = habbod.getInstance().getDb().query("SELECT * FROM user_effects WHERE user_id=" + this.session.getUser().id);
		while(result.row())
		{
			UserEffect effect = new UserEffect();
			effect.userId = result.getInt("user_id");
			effect.effectId = result.getInt("effect_id");
			effect.totalDuration = result.getInt("total_duration");
			effect.enabled = result.getBoolean("enabled");
			if(effect.enabled)
			{
				effect.enabledAt = result.getTime("enabled_at");
			}
			if(effect.hasExpired())
			{
				effect.delete();
			}
			else
			{
				this.effects.add(effect);
			}
		}
		result.release();
	}

	public void addEffect(int effectId, int totalDuration)
	{
		// Create effect
		UserEffect effect = new UserEffect();
		effect.userId = this.session.getUser().id;
		effect.effectId = effectId;
		effect.totalDuration = totalDuration;
		effect.insert();

		// Add to list
		this.effects.add(effect);
		
		// Notify user
		ServerMessage msg = new ServerMessage(EEffectMessage.AvatarEffectAdded);
		msg.appendInt(effect.effectId);
		msg.appendInt(effect.totalDuration);
		this.session.send(msg);
	}

	public int getCurrentEffect()
	{
		return this.currentEffect;
	}

	public void setCurrentEffect(int effectId)
	{
		this.currentEffect = effectId;
		if(session.inRoom())
		{
			session.getRoomUser().setEffect(effectId, true);
		}
	}

	public void checkAndStopExpiredEffects()
	{
		// Has effects?
		if(!this.effects.isEmpty())
		{
			// Process all effects
			Iterator<UserEffect> effects = this.effects.iterator();
			while(effects.hasNext())
			{
				// Effect expired?
				UserEffect effect = effects.next();
				if(effect.hasExpired())
				{
					// Delete data
					effect.delete();
					
					// Remove from iterator
					effects.remove();

					// Notify the user that the effect expired
					ServerMessage msg = new ServerMessage(EEffectMessage.AvatarEffectExpired);
					msg.appendInt(effect.effectId);
					this.session.send(msg);

					// Current effect? Disable it!
					if(effect.effectId == this.currentEffect)
					{
						this.setCurrentEffect(-1);
					}
				}
			}
		}
	}

	public UserEffect getEffect(int effectId)
	{
		for(UserEffect effect : this.effects)
		{
			if(effect.effectId == effectId && !effect.hasExpired())
			{
				return effect;
			}
		}
		return null;
	}

	public List<UserEffect> getEffects()
	{
		return this.effects;
	}
}
