package net.nillus.habbo.game.access;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import net.nillus.habbo.habbod;
import net.nillus.habbo.core.Logger;
import net.nillus.habbo.storage.DatabaseResult;
import net.nillus.habbo.storage.SQLHelper;

public final class AccessControl
{
	private Map<Integer, UserRole> userRoles;

	private AccessControl()
	{
		this.userRoles = Collections.emptyMap();
	}

	public void reloadUserRoles()
	{
		// Create new collection
		Map<Integer, UserRole> tmpRoles = new HashMap<Integer, UserRole>();

		// Reload all the roles
		DatabaseResult result = habbod.getInstance().getDb().query("SELECT id,name,level FROM roles ORDER BY id ASC");
		while(result.row())
		{
			// Load the role info
			UserRole role = new UserRole();
			role.id = result.getInt("id");
			role.name = result.getString("name");
			role.level = result.getInt("level");
			tmpRoles.put(role.id, role);
		}

		// Load all the custom rights for every role
		for(UserRole role : tmpRoles.values())
		{
			// Load custom rights
			result.query("SELECT `right` FROM role_rights WHERE role_id=" + role.id);
			while(result.row())
			{
				role.rights.add(result.getString("right"));
			}
		}
		result.release();

		// Set instance field
		this.userRoles = Collections.unmodifiableMap(tmpRoles);

		// Done!
		Logger.getInstance().info("AccessControl", String.format("Reloaded %d user roles.", this.userRoles.size()));
	}

	public int logUserLogin(int userId, String ip)
	{
		return habbod.getInstance().getDb().executeReturnKey("INSERT INTO user_accesslog(user_id,ip,login) VALUES (" + userId + ",'" + SQLHelper.escape(ip) + "','" + SQLHelper.getDateTimeStr() + "')");
	}

	public void logUserLogout(int accessId)
	{
		habbod.getInstance().getDb().execute("UPDATE user_accesslog SET logout='" + SQLHelper.getDateTimeStr() + "' WHERE id=" + accessId);
	}

	public void logUserAction(int issuerId, int targetId, String action, String message)
	{
		habbod.getInstance().getDb().execute("INSERT INTO system_actionlog(timestamp,issuer_id,action,target_id,message) VALUES('" + SQLHelper.getDateTimeStr() + "'," + issuerId + ",'" + SQLHelper.escape(action) + "'," + targetId + ",'" + SQLHelper.escape(message) + "')");
	}

	public UserRole getUserRole(int roleId)
	{
		return this.userRoles.get(roleId);
	}

	public Collection<UserRole> getAllUserRoles()
	{
		return this.userRoles.values();
	}

	/* Begin singleton */
	private final static AccessControl instance = new AccessControl();

	public final static AccessControl getInstance()
	{
		return instance;
	}
	/* End singleton */
}
