package net.nillus.habbo.game.items;

import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.Map;

import net.nillus.habbo.habbod;
import net.nillus.habbo.communication.EServerMessage;
import net.nillus.habbo.communication.ServerMessage;
import net.nillus.habbo.sessions.Session;
import net.nillus.habbo.storage.DatabaseResult;

public final class ItemInventory
{
	private final Session session;
	private final Map<Integer, Item> items;

	public ItemInventory(Session session)
	{
		this.session = session;
		this.items = new LinkedHashMap<Integer, Item>(); // Use LinkedHashMap to preserve order (and don't use natural sorting on item IDs)
	}

	public void clear()
	{
		this.items.clear();
	}

	public void reload()
	{
		// Clear old items
		this.clear();

		// Reload items
		DatabaseResult result = habbod.getInstance().getDb().query("SELECT * FROM items WHERE owner_id=" + this.session.getUser().id + " AND room_id=0");
		while(result.row())
		{
			Item item = Item.parse(result);
			if(item != null)
			{
				this.items.put(item.id, item);
			}
		}
		result.release();
	}

	public boolean containsItem(int itemId)
	{
		return this.items.containsKey(itemId);
	}

	public Item getItem(int itemId)
	{
		return this.items.get(itemId);
	}

	public void addItem(Item item, boolean save)
	{
		// Set the item info
		item.ownerId = this.session.getUser().id;
		item.roomId = 0;
		this.items.put(item.id, item);
		
		// Save item?
		if(save)
		{
			item.update();
		}
	}

	public void removeItem(int itemId)
	{
		this.items.remove(itemId);
		
		// Notify user
		ServerMessage msg = new ServerMessage(EServerMessage.InventoryRemove);
		msg.appendInt(itemId);
		this.session.send(msg);
	}

	public void refresh()
	{
		this.session.send(new ServerMessage(EServerMessage.InventoryUpdate));
	}

	public final Collection<Item> getItems()
	{
		return this.items.values();
	}
}
