package mod_messenger;

import net.nillus.habbo.communication.messages.incoming.friendlist.AcceptBuddyHandler;
import net.nillus.habbo.communication.messages.incoming.friendlist.DeclineBuddyHandler;
import net.nillus.habbo.communication.messages.incoming.friendlist.FollowBuddyHandler;
import net.nillus.habbo.communication.messages.incoming.friendlist.FriendsListUpdateHandler;
import net.nillus.habbo.communication.messages.incoming.friendlist.GetFriendsRoomsHandler;
import net.nillus.habbo.communication.messages.incoming.friendlist.GetRoomsWithFriendsHandler;
import net.nillus.habbo.communication.messages.incoming.friendlist.InitMessengerHandler;
import net.nillus.habbo.communication.messages.incoming.friendlist.RemoveBuddyHandler;
import net.nillus.habbo.communication.messages.incoming.friendlist.RequestBuddyHandler;
import net.nillus.habbo.communication.messages.incoming.friendlist.SearchUserHandler;
import net.nillus.habbo.communication.messages.incoming.friendlist.SendInstantMessageHandler;
import net.nillus.habbo.communication.messages.incoming.friendlist.SendRoomInviteHandler;
import net.nillus.habbo.communication.messages.incoming.friendlist.VisitFriendsMeetingRoomHandler;
import net.nillus.habbo.game.messenger.InstantMessengerService;
import net.nillus.habbo.modules.Module;
import net.nillus.habbo.modules.ModuleInfo;
import net.nillus.habbo.sessions.Session;


@ModuleInfo
(
		name = "mod_messenger",
		description = "Enables support for the in-game instant messenger.", 
		author = "Nils 'Nillus'",
		url = "http://modules.openhabbo.org/mod_messenger/",
		version = "1.0.0",
		dependencies = { }
)
public final class mod_messenger extends Module
{
	public mod_messenger()
	{
		// Messenger
		this.getAutoLoader().msgHandler(12, new InitMessengerHandler());
		this.getAutoLoader().msgHandler(15, new FriendsListUpdateHandler());
		this.getAutoLoader().msgHandler(34, new SendRoomInviteHandler());
		this.getAutoLoader().msgHandler(40, new RemoveBuddyHandler());
		this.getAutoLoader().msgHandler(41, new SearchUserHandler());
		this.getAutoLoader().msgHandler(33, new SendInstantMessageHandler());
		this.getAutoLoader().msgHandler(37, new AcceptBuddyHandler());
		this.getAutoLoader().msgHandler(38, new DeclineBuddyHandler());
		this.getAutoLoader().msgHandler(39, new RequestBuddyHandler());
		this.getAutoLoader().msgHandler(262, new FollowBuddyHandler());
		
		// Navigator
		this.getAutoLoader().msgHandler(432, new GetFriendsRoomsHandler());
		this.getAutoLoader().msgHandler(433, new GetRoomsWithFriendsHandler());
		this.getAutoLoader().msgHandler(490, new VisitFriendsMeetingRoomHandler());
		
		// Event hooks
		this.getAutoLoader().eventListener(Session.onLogout, new LogoutHook());
		this.getAutoLoader().eventListener(Session.onRoomEnter, new ChangeRoomHook());
		this.getAutoLoader().eventListener(Session.onRoomExit, new ChangeRoomHook());
		
		// Crawl idle buddy refs every 30 seconds to clean mem
		this.enablePulseTimer(30);
	}
	
	@Override
	public void onUnload()
	{
		// Clear registry
		//InstantMessengerService.getInstance().clearRegistry();
	}

	@Override
	protected void onPulse()
	{
		// Clean the cache/registry of buddy info that hasn't been accessed recently
		InstantMessengerService.getInstance().cleanup();
	}

	@Override
	public String getStatusString()
	{
		return "regsize=" + InstantMessengerService.getInstance().registrySize();
	}
}
