package mod_moderation;

import mod_moderation.chatcmds.*;
import mod_moderation.remotecmds.*;
import net.nillus.habbo.communication.messages.incoming.moderation.*;
import net.nillus.habbo.game.moderation.ModerationCenter;
import net.nillus.habbo.modules.Module;
import net.nillus.habbo.modules.ModuleInfo;
import net.nillus.habbo.sessions.Session;


@ModuleInfo
(
		name = "mod_moderation",
		description = "Enables a range of in-game moderation tools, chatcommands and remotecommands",
		author = "Nils 'Nillus'",
		url = "http://modules.openhabbo.org/mod_moderation/",
		version = "1.0.0",
		dependencies = { "mod_chatlog", "mod_roomvisits", "mod_roomevent" }
)
public final class mod_moderation extends Module
{
	public static String inappropriateRoomName;
	public static String inappropriateRoomDescription;
	
	public mod_moderation()
	{
		// Auth hook for ban detection
		this.getAutoLoader().eventListener(Session.onAuthenticate, new AuthBanCheckHook());
		
		// Login hook for sending pending tickets for moderators
		this.getAutoLoader().eventListener(Session.onLogin, new LoginHook());
		
		// Message handlers for modtools
		this.getAutoLoader().msgHandler(453, new SubmitHelpTicketHandler());
        this.getAutoLoader().msgHandler(238, new DeletePendingTicketHandler());
        this.getAutoLoader().msgHandler(200, new ModSendRoomAlertHandler());
        this.getAutoLoader().msgHandler(450, new ModPickTicketHandler());
        this.getAutoLoader().msgHandler(451, new ModReleaseTicketHandler());
        this.getAutoLoader().msgHandler(452, new ModCloseTicketHandler());
        this.getAutoLoader().msgHandler(454, new ModGetUserInfoHandler());
        this.getAutoLoader().msgHandler(455, new ModGetUserChatlogHandler());
        this.getAutoLoader().msgHandler(456, new ModGetRoomChatlogHandler());
        this.getAutoLoader().msgHandler(457, new ModGetTicketChatlogHandler());
        this.getAutoLoader().msgHandler(458, new ModGetRoomVisitsHandler());
        this.getAutoLoader().msgHandler(459, new ModGetRoomToolHandler());
        this.getAutoLoader().msgHandler(460, new ModPerformRoomActionHandler());
        this.getAutoLoader().msgHandler(461, new ModSendUserCautionHandler());
        this.getAutoLoader().msgHandler(462, new ModSendUserMessageHandler());
        this.getAutoLoader().msgHandler(463, new ModKickUserHandler());
        this.getAutoLoader().msgHandler(464, new ModBanUserHandler());
        
		// Chatcommands
		this.getAutoLoader().chatCmd("alert", new UserAlertCommandHandler());
		this.getAutoLoader().chatCmd("kick", new UserKickCommandHandler());
		this.getAutoLoader().chatCmd("roomalert", new RoomAlertCommandHandler());
		this.getAutoLoader().chatCmd("roomkick", new RoomKickCommandHandler());
		
		// Remote commands for site etc
		this.getAutoLoader().remoteCmd("ALERT", new UserAlertHandler());
		this.getAutoLoader().remoteCmd("KICK", new UserKickHandler());
		this.getAutoLoader().remoteCmd("ROOMALERT", new RoomAlertHandler());
		this.getAutoLoader().remoteCmd("ROOMKICK", new RoomKickHandler());
	}
	
	@Override
	public boolean onLoad()
	{
		ModerationCenter.getInstance().loadPendingTickets();
		ModerationCenter.getInstance().getPresetManager().reload();
		
		mod_moderation.inappropriateRoomName = this.getConfigValue("room_inappropriate_name");
		mod_moderation.inappropriateRoomDescription = this.getConfigValue("room_inappropriate_description");
		
		return true;
	}
}
