package habbod2.game.chatcommands.admin;

import habbod2.System;
import habbod2.util.PropertiesBox;
import habbod2.core.Locale;
import habbod2.game.GameManager;
import habbod2.game.access.AccessControl;
import habbod2.game.catalog.Catalog;
import habbod2.game.chatcommands.ChatCommandHandler;
import habbod2.game.chatcommands.ChatCommandManager;
import habbod2.game.help.HelpTool;
import habbod2.game.items.ItemInventory;
import habbod2.game.items.ItemManager;
import habbod2.game.misc.WordFilterManager;
import habbod2.game.moderation.ModerationPresetManager;
import habbod2.game.music.MusicManager;
import habbod2.game.pets.PetManager;
import habbod2.game.rooms.RoomManager;
import habbod2.game.util.GameError;
import habbod2.game.sessions.Session;

public class ReloadCommand extends ChatCommandHandler
{
	@Override
	public void handle(Session session, String... args) throws GameError
	{
		// Get game manager
		GameManager gameMgr = System.get().getGameMgr();
		
		// Execute action
		String component = args[0];
		if(component.equals("config"))
		{
			PropertiesBox config = gameMgr.getConfig();
			config.reload();
			session.sendWhisperNotif(String.format("Reloaded %d system configuration properties.", config.size()));
		}
		else if(component.equals("locale"))
		{
			Locale locale = gameMgr.getSystem().getLocale();
			locale.reload();
			session.sendWhisperNotif(String.format("Reloaded %d entries for %s.", locale.size(), locale.getLocale()));
		}
		else if(component.equals("roles"))
		{
			AccessControl accessCtrl = gameMgr.getAccessCtrl();
			accessCtrl.reload();
			session.sendWhisperNotif(String.format("Reloaded %d user roles.", accessCtrl.getAllUserRoles().size()));
		}
		else if(component.equals("models"))
		{
			RoomManager roomMgr = gameMgr.getRoomMgr();
			roomMgr.reloadModels();
			session.sendWhisperNotif(String.format("Reloaded %d room models.", roomMgr.getAllRoomModels().size()));
		}
		else if(component.equals("navigator"))
		{
			gameMgr.getNavigator().reload();
			session.sendWhisperNotif("Reloaded navigator.");
		}
		else if(component.equals("items"))
		{
			ItemManager itemMgr = gameMgr.getItemMgr();
			itemMgr.reloadDefinitions();
			session.sendWhisperNotif(String.format("Reloaded %d item definitions.", itemMgr.getAllDefinitions().size()));
			this.handle(session, "catalog");
		}
		else if(component.equals("pets"))
		{
			PetManager petMgr = gameMgr.getPetMgr();
			petMgr.reload();
			session.sendWhisperNotif(String.format("Reloaded %d pet types.", petMgr.types()));
		}
		else if(component.equals("songs"))
		{
			MusicManager musicMgr = gameMgr.getMusicMgr();
			musicMgr.reload();
			session.sendWhisperNotif(String.format("Reloaded %d songs.", musicMgr.songs()));
		}
		else if(component.equals("catalog"))
		{
			Catalog catalog = gameMgr.getCatalog();
			catalog.reload();
			session.sendWhisperNotif(String.format("Reloaded %d pages and %d products.", catalog.getAllPages().size(), catalog.getAllProducts().size()));
		}
		else if(component.equals("inventory"))
		{
			ItemInventory inventory = session.getItemInventory();
			inventory.reload();
			inventory.refresh();
			session.sendWhisperNotif(String.format("Reloaded %d items in your inventory.", inventory.getItems().size()));
		}
		else if(component.equals("helptool"))
		{
			HelpTool help = gameMgr.getHelpTool();
			help.reload();
			session.sendWhisperNotif(String.format("Reloaded %d categories, %d topics (of which %d important) and %d known issues.", help.getAllCategories().size(), help.getAllTopics().size(), help.getImportantTopics().size(), help.getKnownIssues().size()));
		}
		else if(component.equals("modpresets"))
		{
			ModerationPresetManager presets = gameMgr.getModerationCenter().getPresetManager();
			presets.reload();
			session.sendWhisperNotif("Reloaded all presets.");
		}
		else if(component.equals("filter"))
		{
			WordFilterManager filter = gameMgr.getWordFilterMgr();
			filter.reload();
			session.sendWhisperNotif(String.format("Reloaded %d words.", filter.words()));
		}
		else if(component.equals("commands"))
		{
			ChatCommandManager cmdMgr = gameMgr.getChatCmdMgr();
			cmdMgr.reload();
			session.sendWhisperNotif(String.format("Reloaded %d chatcommands.", cmdMgr.count()));
		}
		else
		{
			session.sendNotif("The following system components are reloadable:\r\r- config\r- locale\r- roles\r- models\r- navigator\r- items\r- pets\r- songs\r- catalog\r- inventory\r- helptool\r- modpresets\r- filter\r- commands\r\rUsage is :reload <component>.");
		}
	}
}
