package habbod2.game.chatcommands.core;

import java.lang.management.ManagementFactory;
import java.lang.management.OperatingSystemMXBean;
import java.lang.management.ThreadMXBean;
import java.util.Properties;

import habbod2.System;
import habbod2.game.chatcommands.ChatCommandHandler;
import habbod2.game.util.GameError;
import habbod2.game.sessions.Session;

public class StatsCommand extends ChatCommandHandler
{
	@Override
	public void handle(Session session, String... args) throws GameError
	{
		// Gather the required information
		System system = System.get();
		Properties props = java.lang.System.getProperties();
		Runtime vm = system.getVM();
		OperatingSystemMXBean os = ManagementFactory.getOperatingSystemMXBean();
		ThreadMXBean threads = ManagementFactory.getThreadMXBean();

		// Calculate uptime
		long uptime = (System.currentTimeMillis() - system.getStartTime());
		long days = (uptime / (1000 * 60 * 60 * 24));
		long hours = (uptime - days * (1000 * 60 * 60 * 24)) / (1000 * 60 * 60);
		long minutes = (uptime - days * (1000 * 60 * 60 * 24) - hours * (1000 * 60 * 60)) / (1000 * 60);
		long seconds = (uptime - days * (1000 * 60 * 60 * 24) - hours * (1000 * 60 * 60) - minutes * (1000 * 60)) / (1000);

		// Build notification
		StringBuilder notif = new StringBuilder();

		// Uptime
		notif.append(system.getVersion());
		notif.append("\r");
		notif.append(String.format("Server uptime is %d day(s), %d hour(s), %d minute(s) and %d second(s).\r", days, hours, minutes, seconds));
		notif.append(String.format("There are %d user(s) online, and your session ID is #%d.\r", system.getGameMgr().getSessionMgr().authedSessions(), session.id));
		notif.append(String.format("There are %d room instance(s) loaded.\r\r", system.getGameMgr().getRoomMgr().getAllRoomInstances().size()));

		// Environment
		notif.append(String.format("OS: %s %s (%s)\r", os.getName(), os.getArch(), os.getVersion()));
		notif.append(String.format("VM: %s %s\r", props.get("java.vm.name"), props.get("java.vm.version")));
		notif.append(String.format("CPU: %d cores (active server threads: %d)\r", vm.availableProcessors(), (threads.getThreadCount() - threads.getDaemonThreadCount())));
		notif.append(String.format("MEM: %d MiB\r", system.getMemoryUsage() / 1024));
		notif.append(String.format("DB: %d connections", system.getDb().getPool().size()));

		// Send it
		session.sendSystemNotif(notif.toString());
	}
}
