package habbod2.communication.messages.incoming.catalog;

import java.util.List;

import habbod2.System;
import habbod2.net.protocol.ClientMessage;
import habbod2.communication.RequestHandler;
import habbod2.communication.messages.outgoing.catalog.PurchaseOKMessageComposer;
import habbod2.communication.messages.outgoing.notifications.UnseenItemsMessageComposer;
import habbod2.game.catalog.Catalog;
import habbod2.game.catalog.CatalogProduct;
import habbod2.game.items.InventoryItem;
import habbod2.game.sessions.Session;

public class PurchaseFromCatalogHandler implements RequestHandler
{
	@Override
	@SuppressWarnings("unused")
	public void handle(Session session, ClientMessage request)
	{
		// Parse purchase info
		int pageId = request.readInt();
		int productId = request.readInt();
		String purchaseData = request.readStr(true);

		// Valid product?
		Catalog catalog = System.get().getGameMgr().getCatalog();
		CatalogProduct product = catalog.getProduct(productId);
		if(product == null || !product.isPurchasableBy(session.getUser()))
		{
			//System.get().getLogger().warning(this, String.format("User #%d tried to purchase product #%d from the Catalog, which is not existent/available/purchasable to this user.", session.getUser().id, productId));
			return;
		}

		// Attempt item purchase
		List<InventoryItem> shipping = catalog.purchaseProduct(product, session.getUser(), purchaseData, true);
		if(shipping == null)
		{
			session.sendNotif(String.format("Purchase of catalog product #%d failed, you will not be billed. Please try again in a few minutes. If this keeps happening, please contact Support.", product.id));
			return;
		}

		// Any items?
		if(!shipping.isEmpty())
		{
			// Add them to inventory
			for(InventoryItem item : shipping)
			{
				session.getItemInventory().addItem(item, false);
			}
			session.getItemInventory().refresh();
			
			// Notify of changes
			session.send(UnseenItemsMessageComposer.compose(shipping));
		}
		
		// Bill user for product
		catalog.billUser(session, product);

		// Purchase okay!
		session.send(PurchaseOKMessageComposer.compose(product));
	}
}
