package habbod2.communication.messages.incoming.handshake;

import habbod2.System;
import habbod2.net.protocol.ClientMessage;
import habbod2.communication.RequestHandler;
import habbod2.communication.messages.outgoing.handshake.UserObjectMessageComposer;
import habbod2.communication.messages.outgoing.handshake.UserRightsMessageComposer;
import habbod2.communication.messages.outgoing.help.TutorialStatusMessageComposer;
import habbod2.communication.messages.outgoing.inventory.avatareffect.AvatarEffectsMessageComposer;
import habbod2.communication.messages.outgoing.moderation.ModerationTicketMessageComposer;
import habbod2.communication.messages.outgoing.moderation.ModeratorInitMessageComposer;
import habbod2.communication.messages.outgoing.navigator.FavouriteRoomsMessageComposer;
import habbod2.communication.messages.outgoing.navigator.NavigatorSettingsMessageComposer;
import habbod2.communication.messages.outgoing.notifications.InfoFeedEnableMessageComposer;
import habbod2.game.moderation.ModerationCenter;
import habbod2.game.moderation.ModerationPresetManager;
import habbod2.game.moderation.ModerationTicket;
import habbod2.game.sessions.Session;

public class InfoRetrieveHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request)
	{
		// Send the rights list
		session.send(UserRightsMessageComposer.compose(session.getUser()));
		
		// Push user data
		session.send(UserObjectMessageComposer.compose(session.getUser()));
		
		// Enable info feed
		session.send(InfoFeedEnableMessageComposer.compose(true,  false));
		
		// Push navigator settings
		session.send(NavigatorSettingsMessageComposer.compose(session.getUser().homeRoomId));
		
		// Push favorite rooms
		session.send(FavouriteRoomsMessageComposer.compose(session.getFavoriteRooms().getFavorites()));
		
		// Push effects
		session.send(AvatarEffectsMessageComposer.compose(session.getEffectInventory().getEffects()));
		
		// User is moderator?
		if(session.getUser().rights.contains("fuse_mod"))
		{
			// Get moderation center
			ModerationCenter modCenter = System.get().getGameMgr().getModerationCenter();
			
			// Send preset moderation responses
			ModerationPresetManager presets = modCenter.getPresetManager();
			session.send(ModeratorInitMessageComposer.compose(presets.getUserMessagePresets(), presets.getUserActionPresets(), presets.getRoomMessagePresets()));
			
			// Send open tickets
			for(ModerationTicket ticket : modCenter.getAllTickets())
			{
				if(ticket.status == ModerationTicket.STATUS_OPEN || ticket.status == ModerationTicket.STATUS_PICKED)
				{
					session.send(ModerationTicketMessageComposer.compose(ticket));
				}
			}
		}
		
		// Related to name changing etc
		session.send(TutorialStatusMessageComposer.compose(1, false, 1));
	}
}
