package habbod2.communication.messages.incoming.inventory.furni;

import habbod2.net.protocol.ClientMessage;
import habbod2.communication.RequestHandler;
import habbod2.communication.messages.outgoing.room.engine.RoomPropertyMessageComposer;
import habbod2.game.items.InventoryItem;
import habbod2.game.rooms.instance.RoomInstance;
import habbod2.game.rooms.items.misc.DecorationItem;
import habbod2.game.sessions.Session;

public class RequestRoomPropertySetHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request)
	{
		// Allowed to apply decoration?
		if(!session.inRoom() || !session.getRoomUser().hasRoomOwner)
		{
			return;
		}

		// Verify decoration item
		InventoryItem item = session.getItemInventory().getItem(request.readInt());
		if(item == null || !item.is(DecorationItem.class))
		{
			return;
		}

		// Remove from inventory and delete
		session.getItemInventory().removeItem(item.id);
		item.delete();
		
		// Apply new props
		RoomInstance room = session.getRoom();
		if(item.definition.data.equals("wallpaper"))
		{
			room.getInfo().wallpaper = item.state;
		}
		else if(item.definition.data.equals("floor"))
		{
			room.getInfo().floor = item.state;
		}
		else if(item.definition.data.equals("landscape"))
		{
			room.getInfo().landscape = item.state;
		}
		room.getInfo().saveDecoration();

		// Notify room
		room.broadcast(RoomPropertyMessageComposer.compose(item.definition.data, item.state));
	}
}
