package habbod2.communication.messages.incoming.moderator;

import habbod2.System;
import habbod2.net.protocol.ClientMessage;
import habbod2.communication.RequestHandler;
import habbod2.communication.messages.outgoing.help.IssueCloseNotificationMessageComposer;
import habbod2.game.moderation.ModerationCenter;
import habbod2.game.moderation.ModerationTicket;
import habbod2.game.sessions.Session;

public class CloseIssuesHandler implements RequestHandler
{
	@Override
	@SuppressWarnings("unused")
	public void handle(Session session, ClientMessage request)
	{
		// Has mod?
		if(!session.getUser().rights.contains("fuse_mod"))
		{
			return;
		}
		
		// Get center
		ModerationCenter ctr = System.get().getGameMgr().getModerationCenter();
		
		// Get ticket
		int result = request.readInt();
		int unk = request.readInt();
		int ticketId = request.readInt();
		ModerationTicket ticket = ctr.getTicket(ticketId);
		
		// Ticket exists and picked by this user?
		if(ticket != null && ticket.status == ModerationTicket.STATUS_PICKED && ticket.pickerId == session.getUser().id)
		{
			// Closing as what?
			if(result == 1)
			{
				ticket.status = ModerationTicket.STATUS_INVALID;
			}
			else if(result == 2)
			{
				ticket.status = ModerationTicket.STATUS_ABUSIVE;
			}
			else
			{
				ticket.status = ModerationTicket.STATUS_RESOLVED;
				result = 0;
			}
			
			// Save status
			ticket.saveStatus();
			
			// Broadcast update
			ctr.broadcastTicket(ticket);
			
			// Feedback to caller
			Session caller = System.get().getGameMgr().getSessionMgr().getSessionByUserId(ticket.userId);
			if(caller != null)
			{
				caller.send(IssueCloseNotificationMessageComposer.compose(result));
			}
		}
	}
}
