package habbod2.communication.messages.incoming.room.chat;

import habbod2.System;
import habbod2.net.protocol.ClientMessage;
import habbod2.communication.RequestHandler;
import habbod2.communication.messages.outgoing.room.chat.WhisperMessageComposer;
import habbod2.net.protocol.ServerMessage;
import habbod2.game.rooms.instances.RoomUser;
import habbod2.game.util.SecurityUtil;
import habbod2.game.sessions.Session;

public class WhisperHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request)
	{
		// Parse receiver + text
		final String[] data = request.readStr().split(" ", 2);
		if(data.length != 2)
		{
			return;
		}
		final String receiverName = data[0];
		final String message = SecurityUtil.filterStr(data[1]);

		// Actually in room?
		if(session.inRoom())
		{
			// Receiver in room too?
			RoomUser user = session.getRoomUser();
			RoomUser receiver = session.getRoomInstance().getUserByUsername(receiverName);
			if(receiver != null)
			{
				// Send to this user & receiver
				ServerMessage msg = WhisperMessageComposer.compose(user, message);
				user.getSession().send(msg);
				receiver.getSession().send(msg);
				
				// Log it
				System.get().getGameMgr().getChatlogSvc().log(session.getUser().id, session.getRoomInstance().getInfo().id, message, receiver.getInfo().id);
				
				// Eavesdropping!
				for(RoomUser tmp : session.getRoomInstance().getAllUsers())
				{
					// Exclude the sender/receiver
					if(tmp != user && tmp != receiver)
					{
						// Can see whispers?
						if(tmp.getInfo().rights.contains("see_whispers"))
						{
							// Can only see whispers of lower role
							if(tmp.getInfo().role.level > user.getInfo().role.level)
							{
								tmp.getSession().send(WhisperMessageComposer.compose(user, String.format("[>> %s] %s", receiver.getInfo().username, message)));
							}
						}
					}
				}
			}
		}
	}
}
