package habbod2.communication.messages.incoming.room.engine;

import habbod2.net.protocol.ClientMessage;
import habbod2.communication.RequestHandler;
import habbod2.game.rooms.items.RoomItem;
import habbod2.game.rooms.items.wall.PostItItem;
import habbod2.game.util.SecurityUtil;
import habbod2.game.sessions.Session;

public class SetItemDataHandler implements RequestHandler
{
	private static final String[] ALLOWED_COLORS = new String[] { "FFFF33", "FF9CFF", "9CCEFF", "9CFF9C" };
	
	public boolean colorIsValid(String color)
	{
		for(int x = 0; x < ALLOWED_COLORS.length; x++)
		{
			if(ALLOWED_COLORS[x].equals(color))
			{
				return true;
			}
		}
		return false;
	}
	
	public String getDefaultColor()
	{
		return ALLOWED_COLORS[0];
	}
	
	@Override
	public void handle(Session session, ClientMessage request)
	{
		// Get item
		RoomItem item = session.getRoomInstance().getItem(request.readInt());
		if(item != null && item instanceof PostItItem)
		{
			// Parse new data
			String[] info = request.readStr().split(" ", 2);
			String color = info[0];
			String text = SecurityUtil.filterStr(info[1], false);
			
			// Validate new color
			if(!this.colorIsValid(color))
			{
				return;
			}
			
			// If user hasn't got room rights...
			if(!session.getRoomUser().hasRoomRights)
			{
				// Then only allow extending of the text (previously this check was clientside)
				if(!item.data.startsWith(text))
				{
					return;
				}
			}
			
			// Save item
			item.data = text;
			item.setState(color);
		}
	}
}
