package habbod2.communication.messages.incoming.room.furniture;

import java.util.List;

import habbod2.System;
import static habbod2.core.Locale._I;
import habbod2.net.protocol.ClientMessage;
import habbod2.communication.RequestHandler;
import habbod2.game.catalog.Catalog;
import habbod2.game.catalog.CatalogProduct;
import habbod2.game.items.InventoryItem;
import habbod2.game.rooms.items.RoomFloorItem;
import habbod2.game.util.GameUtil;
import habbod2.game.sessions.Session;
import habbod2.game.rooms.items.floor.PresentBoxItem;
import habbod2.communication.messages.outgoing.room.furniture.PresentOpenedMessageComposer;

public class PresentOpenHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request)
	{
		// Allowed to open presentboxes in this room?
		if(!session.inRoom() || !session.getRoomUser().hasRoomOwner)
		{
			return;
		}

		// Verify the item
		RoomFloorItem item = session.getRoomInstance().getFloorItem(request.readInt());
		if(item == null || !(item instanceof PresentBoxItem))
		{
			return;
		}

		// Pickup + delete the item
		InventoryItem invItem = session.getRoomInstance().pickupFloorItem(item.id, null);
		invItem.delete();
		
		// Get giftinfo
		String[] giftInfo = item.data.split(GameUtil.DATA_DELIMITER, 2);
		if(giftInfo.length != 2)
		{
			System.get().getLogger().warning(session, String.format("Opening presentbox #%d failed: gift info is corrupted!", item.id));
			return;
		}

		// Redeem contents
		Catalog catalog = System.get().getGameMgr().getCatalog();
		CatalogProduct product = catalog.getProduct(GameUtil.parseInt(giftInfo[0]));
		List<InventoryItem> content = catalog.purchaseProduct(product, session.getUser(), giftInfo[1], false);
		if(product == null || content == null)
		{
			session.sendNotif(_I("Invalid presentbox content!"));
			return;
		}
		
		// Present contents
		session.send(PresentOpenedMessageComposer.compose(product));
		
		// Add to inventory
		for(InventoryItem tmpItem : content)
		{
			session.getItemInventory().addItem(tmpItem, false);
		}
		session.getItemInventory().refresh();
	}
}
