package habbod2.communication.messages.outgoing.room.engine;

import java.util.Collection;

import habbod2.net.protocol.ServerMessage;
import habbod2.communication.messages.EServerMessage;
import habbod2.game.pets.Pet;
import habbod2.game.pets.RoomPet;
import habbod2.game.rooms.instance.RoomAvatar;
import habbod2.game.rooms.instance.RoomUser;
import habbod2.game.users.User;

public class UsersMessageComposer
{
	public static ServerMessage compose(RoomAvatar avatar)
	{
		ServerMessage msg = ServerMessage.create(EServerMessage.Users);
		msg.appendInt(1);
		serializeAvatar(avatar, msg);
		
		return msg;
	}
	
	public static ServerMessage compose(Collection<RoomAvatar> avatars)
	{
		ServerMessage msg = ServerMessage.create(EServerMessage.Users);
		msg.appendInt(avatars.size());
		for(RoomAvatar avatar : avatars)
		{
			serializeAvatar(avatar, msg);
		}
		
		return msg;
	}

	public static void serializeAvatar(RoomAvatar avatar, ServerMessage msg)
	{
		if(avatar instanceof RoomUser)
		{
			serializeAvatar((RoomUser)avatar, msg);
		}
		else if(avatar instanceof RoomPet)
		{
			serializeAvatar((RoomPet)avatar, msg);
		}
		else
		{
			// TODO: bots
		}
	}
	
	private static void serializeAvatar(RoomUser avatar, ServerMessage msg)
	{
		User user = avatar.getInfo();
		msg.appendInt(user.id);
		msg.appendStr(user.username);
		msg.appendStr(user.motto);
		msg.appendStr(user.figure);
		msg.appendInt(avatar.avatarId);
		msg.appendInt(avatar.tile.x);
		msg.appendInt(avatar.tile.y);
		msg.appendStr(avatar.tile.height);
		msg.appendInt(2);
		msg.appendInt(1);
		msg.appendStr(user.gender);
		msg.appendInt(-1);
		msg.appendInt(-1);
		msg.appendInt(-1);
		msg.appendStr("");
		msg.appendInt(user.respect); // Achievement score
	}
	
	private static void serializeAvatar(RoomPet avatar, ServerMessage msg)
	{
		Pet pet = avatar.getInfo();
		msg.appendInt(pet.id);
		msg.appendStr(pet.name);
		msg.appendStr("");
		msg.appendStr(pet.getFigure());
		msg.appendInt(avatar.avatarId);
		msg.appendInt(avatar.tile.x);
		msg.appendInt(avatar.tile.y);
		msg.appendStr(avatar.tile.height);
		msg.appendInt(4);
		msg.appendInt(2);
		msg.appendInt(0);
	}
}
