package habbod2.communication.messages.outgoing.roomsettings;

import java.util.Set;
import java.util.Map.Entry;

import habbod2.communication.EServerMessage;
import habbod2.game.rooms.Room;
import habbod2.game.rooms.instances.RoomInstance;
import habbod2.net.protocol.ServerMessage;

public class RoomSettingsDataMessageComposer
{
	public static ServerMessage compose(RoomInstance instance)
	{
		ServerMessage msg = ServerMessage.create(EServerMessage.RoomSettingsData);
		
		// Basic
		Room room = instance.getInfo();
		msg.appendInt(room.id);
		msg.appendStr(room.name);
		msg.appendStr(room.description);
		msg.appendInt(room.state);
		msg.appendInt(room.categoryId);
		msg.appendInt(room.usersMax);
		msg.appendInt(25); // default max

		// Tags
		msg.appendStrList(room.tags);
		
		// Rights
		Set<Entry<Integer, String>> rights = instance.getUsersWithRights();
		msg.appendInt(rights.size());
		for(Entry<Integer, String> user : rights)
		{
			msg.appendInt(user.getKey());
			msg.appendStr(user.getValue());
		}
		msg.appendInt(rights.size());

		// Other flags
		msg.appendBool(room.allowPets);
		msg.appendBool(room.allowPetsEat);
		msg.appendBool(room.allowWalkthrough);
		msg.appendBool(room.hideWalls);
		msg.appendInt(room.wallThickness);
		msg.appendInt(room.floorThickness);
		
		return msg;
	}
}
