package habbod2.core;

import habbod2.System;
import habbod2.game.GameManager;

public class StatsMonitor extends Thread
{
	public static final int UPDATE_INTERVAL_SECONDS = 30;
	
	private System instance;
	private long updateIntervalSeconds;
	private int peakOnlineUsers;
	
	public StatsMonitor(System instance)
	{
		this.setName("StatsMonitor");
		this.setPriority(Thread.MIN_PRIORITY);
		this.instance = instance;
		this.updateIntervalSeconds = UPDATE_INTERVAL_SECONDS;
	}
	
	public void setUpdateInterval(int seconds)
	{
		this.updateIntervalSeconds = seconds;
	}
	
	public void reloadPeakOnlineUsers()
	{
		this.peakOnlineUsers = this.instance.getDb().queryInt("SELECT onlineusers_peak FROM system_monitor");
	}
	
	public void resetStatistics()
	{
		this.instance.getDb().execute("UPDATE users SET online=0");
		this.instance.getDb().execute("UPDATE rooms SET users_now=0");
	}
	
	public void updateFromSnapshot()
	{
		// Gather system stats
		long memUsage = this.instance.getMemoryUsage();
		int dbConns = this.instance.getDb().getPool().size();

		// Gather game stats
		GameManager gameMgr = this.instance.getGameMgr();
		int onlineUsers = gameMgr.getSessionMgr().authedSessions();
		int roomInstances = gameMgr.getRoomMgr().getAllRoomInstances().size();
		
		// Update peak?
		if(onlineUsers > this.peakOnlineUsers)
		{
			this.peakOnlineUsers = onlineUsers;
		}
		
		// Update stats
		this.instance.getDb().execute("UPDATE system_monitor SET onlineusers=" + onlineUsers + ",onlineusers_peak=" + this.peakOnlineUsers + ",roominstances=" + roomInstances + ",memusage=" + memUsage + ",dbconnections=" + dbConns);
	}
	
	@Override
	public void run()
	{
		while(!Thread.interrupted())
		{
			// Sleep...
			try
			{
				Thread.sleep(this.updateIntervalSeconds * 1000);
			}
			catch (InterruptedException e)
			{
				
			}
			
			// Update!
			this.updateFromSnapshot();
		}
	}
}
