package habbod2.net.protocol;

import habbod2.net.codec.Base64Codec;
import habbod2.net.codec.WireCodec;

public class ClientMessage
{
	public static final int HEADER_LENGTH = 3;
	public static final int TYPE_LENGTH = 2;
	
	public int type;
	private byte[] payload;
	private int pointer;
	
	public int getType()
	{
		return this.type;
	}
	
	public void setType(int type)
	{
		this.type = type;
	}
	
	public byte[] getPayload()
	{
		return this.payload;
	}
	
	public void setPayload(byte[] payload)
	{
		this.payload = payload;
	}
	
	public String getBody()
	{
		return new String(this.payload);
	}

	public int length()
	{
		return this.payload.length;
	}

	public int remaining()
	{
		return this.payload.length - this.pointer;
	}
	
	public void reset()
	{
		this.pointer = 0;
	}

	public void advance(int n)
	{
		this.pointer += n;
	}

	public boolean contains(byte b)
	{
		for(int x = 0; x < this.payload.length; x++)
		{
			if(this.payload[x] == b)
			{
				return true;
			}
		}
		return false;
	}

	private byte[] readBytes(int n)
	{
		byte[] data = new byte[n];
		for(int x = 0; x < n; x++, this.pointer++)
		{
			data[x] = this.payload[this.pointer];
		}
		return data;
	}

	public boolean readFixedBool()
	{
		return(this.payload[this.pointer++] == 'A');
	}

	public int readFixedInt()
	{
		return Base64Codec.decodeInt(new byte[] { this.payload[this.pointer++], this.payload[this.pointer++] });
	}

	public String readStr()
	{
		return new String(this.readBytes(this.readFixedInt()));
	}
	
	public String[] readStrArray()
	{
		String[] arr = new String[this.readInt()];
		for(int x = 0; x < arr.length; x++)
		{
			arr[x] = this.readStr();
		}
		return arr;
	}

	public int readParseFixedInt()
	{
		try
		{
			return Integer.parseInt(this.readStr());
		}
		catch(NumberFormatException ex)
		{
			return 0;
		}
	}

	public Boolean readBool()
	{
		return(this.remaining() > 0 && this.payload[this.pointer++] == WireCodec.POSITIVE);
	}

	public int readInt()
	{
		byte[] bytes = new byte[this.remaining() < WireCodec.MAX_INTEGER_BYTE_AMOUNT ? this.remaining() : WireCodec.MAX_INTEGER_BYTE_AMOUNT];
		for(int x = 0, y = this.pointer; x < bytes.length; x++, y++)
		{
			bytes[x] = this.payload[y];
		}
		int[] ret = WireCodec.decodeInt(bytes);
		this.pointer += ret[1];
		return ret[0];
	}

	public String header()
	{
		return new String(Base64Codec.encodeInt(this.type, TYPE_LENGTH));
	}

	@Override
	public String toString()
	{
		return this.header() + this.getBody();
	}
}
