package habbod2.net.protocol;

import java.util.List;
import java.io.ByteArrayOutputStream;
import java.io.IOException;

import habbod2.net.codec.Base64Codec;
import habbod2.net.codec.WireCodec;

public class ServerMessage
{
	public static final byte MSG_EOF = (byte) 1;

	public int type;
	private ByteArrayOutputStream buff;

	private ServerMessage()
	{
		this.buff = new ByteArrayOutputStream();
	}
	
	private ServerMessage(int type)
	{
		this();
		this.type = type;
	}

	public void reset(int type)
	{
		this.type = type;
		this.buff.reset();
	}

	public void append(byte b)
	{
		this.buff.write(b);
	}

	public void append(char c)
	{
		this.append((byte) c);
	}

	public void append(byte[] bytes)
	{
		try
		{
			this.buff.write(bytes);
		}
		catch(IOException e)
		{
			e.printStackTrace();
		}
	}

	public void append(String s)
	{
		if(s != null && s.length() > 0)
		{
			this.append(s.getBytes());
		}
	}

	public void appendBool(boolean b)
	{
		if(b)
		{
			this.append(WireCodec.POSITIVE);
		}
		else
		{
			this.append(WireCodec.NEGATIVE);
		}
	}
	
	public void appendBool(Boolean b)
	{
		this.appendBool((b != null && b));
	}

	public void appendInt(int i)
	{
		if(i == 0)
		{
			this.appendBool(false);
		}
		else if(i == 1)
		{
			this.appendBool(true);
		}
		else
		{
			this.append(WireCodec.encodeInt(i));
		}
	}
	
	public void appendStr(char c)
	{
		this.appendStr(c, (byte)2);
	}
	
	public void appendStr(char c, byte tail)
	{
		this.append(c);
		this.append(tail);
	}
	
	public void appendStr(int i)
	{
		this.appendStr(i, (byte)2);
	}
	
	public void appendStr(int i, byte tail)
	{
		this.append(Integer.toString(i));
		this.append(tail);
	}

	public void appendStr(float f)
	{
		this.appendStr(f, (byte)2);
	}
	
	public void appendStr(float f, byte tail)
	{
		this.append(Float.toString(f));
		this.append(tail);
	}
	
	public void appendStr(String s)
	{
		this.appendStr(s, (byte) 2);
	}

	public void appendStr(String s, byte tail)
	{
		this.append(s != null ? s : "");
		this.append(tail);
	}

	public void appendStrLine(String s)
	{
		this.appendStr(s, (byte) 13);
	}

	public void appendStrList(String[] arr)
	{
		this.appendInt(arr.length);
		for(int x = 0; x < arr.length; x++)
		{
			this.appendStr(arr[x]);
		}
	}
	
	public void appendStrList(List<String> list)
	{
		this.appendInt(list.size());
		for(int x = 0; x < list.size(); x++)
		{
			this.appendStr(list.get(x));
		}
	}

	public byte[] getPayload()
	{
		return this.buff.toByteArray();
	}
	
	public byte[] getResult()
	{
		this.buff.write(ServerMessage.MSG_EOF);
		return this.buff.toByteArray();
	}

	public String header()
	{
		return new String(Base64Codec.encodeInt(this.type, 2));
	}

	public int length()
	{
		if(this.buff.size() > 0)
		{
			return this.buff.size() - 2;
		}
		else
		{
			return 0;
		}
	}

	@Override
	public String toString()
	{
		return new String(this.header() + new String(this.buff.toByteArray()));
	}
	
	public static ServerMessage create()
	{
		return ServerMessage.create(0);
	}
	
	public static ServerMessage create(int type)
	{
		// TODO: return a previously finalized ServerMessage?
		return new ServerMessage(type);
	}
}
