package habbod2.storage;

import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.SQLException;

import habbod2.System;
import habbod2.SystemComponent;

public class Database extends SystemComponent
{
	private DatabaseEndpoint endpoint;
	private DatabaseConnectionPool pool;
	private DatabaseUpdateQueue updateQueue;
	
	public Database(System system, DatabaseEndpoint endpoint)
	{
		super(system);
		this.endpoint = endpoint;
		this.pool = new DatabaseConnectionPool(this);
		this.updateQueue = new DatabaseUpdateQueue(this);
	}
	
	public boolean test()
	{
		// Register connection driver package
		String driverClassName = this.getEndpoint().driver;
		try
		{
			Driver driver = (Driver) Class.forName(driverClassName).newInstance();
			DriverManager.registerDriver(driver);
		}
		catch(Exception ex)
		{
			this.getLogger().error(this, String.format("Can't find driver '%s', please copy the driver library to the local Java extensions directory (lib/ext).", driverClassName));
			return false;
		}

		// Create test connection
		DatabaseConnection test = this.pool.getConnection();
		if(test != null)
		{
			this.pool.releaseConnection(test);
			this.getLogger().info(this, String.format("Connected to %s.", this.endpoint));
			return true;
		}
		else
		{
			return false;
		}
	}
	
	public void execute(String sql)
	{
		if(this.updateQueue.isOpen())
		{
			this.updateQueue.enqueue(sql);
		}
		else
		{
			DatabaseConnection connection = this.pool.getConnection();
			if(connection != null)
			{
				Statement st = null;
				try
				{
					st = connection.getInnerConnection().createStatement();
					st.executeUpdate(sql);
				}
				catch(SQLException ex)
				{
					
				}
				finally
				{
					if(st != null)
					{
						try
						{
							st.close();
						}
						catch (SQLException ex)
						{
							
						}
					}
					this.pool.releaseConnection(connection);
				}
			}
		}
	}

	public int executeReturnKey(String sql)
	{
		DatabaseConnection connection = this.pool.getConnection();
		if(connection != null)
		{
			try
			{
				Statement statement = connection.getInnerConnection().createStatement();
				statement.execute(sql, Statement.RETURN_GENERATED_KEYS);
				ResultSet keys = statement.getGeneratedKeys();
				if(keys.next())
				{
					return keys.getInt(1);
				}
				else
				{
					return 0;
				}
			}
			catch(SQLException ex)
			{
				this.getLogger().error(this, "Error during INSERT.", ex);
			}
			finally
			{
				this.pool.releaseConnection(connection);
			}
		}
		return -1;
	}

	public DatabaseResult query(String sql)
	{
		DatabaseConnection connection = this.pool.getConnection();
		if(connection != null)
		{
			try
			{
				ResultSet result = connection.getInnerConnection().createStatement().executeQuery(sql);
				return new DatabaseResult(connection, result);
			}
			catch(SQLException ex)
			{
				this.getLogger().error(this, "Error during SELECT.", ex);
				this.pool.releaseConnection(connection);
			}
		}
		return DatabaseResult.INVALID_RESULT;
	}
	
	public int queryInt(String sql)
	{
		DatabaseConnection connection = this.pool.getConnection();
		if(connection != null)
		{
			try
			{
				ResultSet result = connection.getInnerConnection().createStatement().executeQuery(sql);
				if(result.next())
				{
					return result.getInt(1);
				}
			}
			catch(SQLException ex)
			{
				this.getLogger().error(this, "Error during SELECT.", ex);
			}
			finally
			{
				this.pool.releaseConnection(connection);
			}
		}

		// No result
		return 0;
	}
	
	public String queryStr(String sql)
	{
		DatabaseConnection connection = this.pool.getConnection();
		if(connection != null)
		{
			try
			{
				ResultSet result = connection.getInnerConnection().createStatement().executeQuery(sql);
				if(result.next())
				{
					return result.getString(1);
				}
			}
			catch(SQLException ex)
			{
				this.getLogger().error(this, "Error during SELECT.", ex);
			}
			finally
			{
				this.pool.releaseConnection(connection);
			}
		}

		// No result
		return null;
	}
	
	public DatabaseEndpoint getEndpoint()
	{
		return this.endpoint;
	}
	
	public DatabaseConnectionPool getPool()
	{
		return this.pool;
	}
	
	public DatabaseUpdateQueue getUpdateQueue()
	{
		return this.updateQueue;
	}
}
