package habbod2.storage;

import habbod2.core.SystemStats;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;

public class DatabaseResult implements AutoCloseable
{
	public static final DatabaseResult INVALID_RESULT = new DatabaseResult();

	private boolean valid;
	private DatabaseConnection connection;
	private ResultSet result;

	public DatabaseResult()
	{
		this.valid = false;
	}

	public DatabaseResult(DatabaseConnection connection, ResultSet result)
	{
		this.connection = connection;
		this.result = result;
		this.valid = true;
	}

	private void release()
	{
		if(this.valid)
		{
			try
			{
				this.result.getStatement().close();
				this.connection.getPool().releaseConnection(this.connection);
			}
			catch(SQLException ex)
			{
				this.onError("closing statement", ex);
			}
		}
	}

	public boolean isValid()
	{
		return this.valid;
	}

	public boolean row()
	{
		if(this.valid)
		{
			try
			{
				return this.result.next();
			}
			catch(SQLException ex)
			{
				this.onError("going to next row", ex);
			}
		}
		return false;
	}

	public boolean getBoolean(String c)
	{
		if(this.valid)
		{
			try
			{
				return this.result.getBoolean(c);
			}
			catch(SQLException ex)
			{
				this.onError("getting boolean value", ex);
			}
		}
		return false;
	}

	public byte getByte(String c)
	{
		if(this.valid)
		{
			try
			{
				return this.result.getByte(c);
			}
			catch(SQLException ex)
			{
				this.onError("getting byte value", ex);
			}
		}
		return 0;
	}

	public int getInt(String c)
	{
		if(this.valid)
		{
			try
			{
				return this.result.getInt(c);
			}
			catch(SQLException ex)
			{
				this.onError("getting int value", ex);
			}
		}
		return 0;
	}

	public float getFloat(String c)
	{
		if(this.valid)
		{
			try
			{
				return this.result.getFloat(c);
			}
			catch(SQLException ex)
			{
				this.onError("getting float value", ex);
			}
		}
		return 0;
	}

	public long getLong(String c)
	{
		if(this.valid)
		{
			try
			{
				return this.result.getLong(c);
			}
			catch(SQLException ex)
			{
				this.onError("getting long value", ex);
			}
		}
		return 0;
	}

	public String getString(String c)
	{
		if(this.valid)
		{
			try
			{
				return this.result.getString(c);
			}
			catch(SQLException ex)
			{
				this.onError("getting string value", ex);
			}
		}
		return null;
	}

	public Timestamp getTimestamp(String c)
	{
		if(this.valid)
		{
			try
			{
				return this.result.getTimestamp(c);
			}
			catch(SQLException ex)
			{
				this.onError("getting timestamp value", ex);
			}
		}
		return null;
	}
	
	public long getTime(String c)
	{
		Timestamp t = this.getTimestamp(c);
		return t != null ? t.getTime() : 0;
	}

	public void query(String sql)
	{
		if(this.valid)
		{
			SystemStats.executedQueries++;
			
			try
			{
				Statement st = this.result.getStatement();
				this.result.close();
				this.result = st.executeQuery(sql);
			}
			catch(SQLException ex)
			{
				this.onError("executing additional query", ex);
			}
		}
	}

	private void onError(String operation, Exception ex)
	{
		this.connection.getPool().getLogger().error("DatabaseResult", String.format("Error in operation '%s'.", operation), ex);
	}

	@Override
	public void close()
	{
		this.release();
	}
}
