package habbod2.game;

import habbod2.System;
import habbod2.game.access.AccessControl;
import habbod2.game.availability.AvailabilityManager;
import habbod2.game.bots.BotManager;
import habbod2.game.catalog.Catalog;
import habbod2.game.catalog.VoucherManager;
import habbod2.game.chatcommands.ChatCommandManager;
import habbod2.game.chatlog.ChatlogService;
import habbod2.game.help.HelpTool;
import habbod2.game.items.ItemManager;
import habbod2.game.messenger.MessengerService;
import habbod2.game.misc.WordFilterManager;
import habbod2.game.moderation.ModerationCenter;
import habbod2.game.music.MusicManager;
import habbod2.game.pets.PetManager;
import habbod2.game.rooms.RoomManager;
import habbod2.game.rooms.navigator.RoomEventManager;
import habbod2.game.rooms.navigator.RoomNavigator;
import habbod2.game.sessions.SessionManager;
import habbod2.game.users.UserManager;
import habbod2.game.users.achievements.AchievementManager;
import habbod2.game.users.effects.EffectExpirationChecker;
import habbod2.game.users.subscriptions.SubscriptionManager;
import habbod2.game.users.updaters.AutoCreditsUpdater;
import habbod2.game.users.updaters.AutoPixelsUpdater;
import habbod2.game.users.updaters.AutoRespectUpdater;
import habbod2.storage.SQLHelper;
import habbod2.util.ScheduledTask;

public class GameManager extends System.Component
{
	private AccessControl accessCtrl;
	private AvailabilityManager availabilityMgr;
	private UserManager userMgr;
	private AchievementManager achievementMgr;
	private SubscriptionManager subscriptionMgr;
	private AutoRespectUpdater autoRespectUpdater;
	private HelpTool helpTool;
	private SessionManager sessionMgr;
	private MessengerService messengerSvc;
	private ScheduledTask effectsChecker;
	private VoucherManager voucherMgr;
	private ItemManager itemMgr;
	private PetManager petMgr;
	private BotManager botMgr;
	private MusicManager musicMgr;
	private Catalog catalog;
	private RoomManager roomMgr;
	private RoomEventManager roomEventMgr;
	private RoomNavigator navigator;
	private ModerationCenter moderationCenter;
	private ChatlogService chatlogSvc;
	private ChatCommandManager chatCmdMgr;
	private WordFilterManager wordFilterMgr;
	
	private AutoCreditsUpdater autoCreditsUpdater;
	private AutoPixelsUpdater autoPixelsUpdater;
	
	public GameManager(System system)
	{
		super(system);
		this.accessCtrl = new AccessControl(this);
		this.availabilityMgr = new AvailabilityManager(this);
		this.userMgr = new UserManager(this);
		this.achievementMgr = new AchievementManager(this);
		this.subscriptionMgr = new SubscriptionManager(this);
		this.autoRespectUpdater = new AutoRespectUpdater(this);
		this.helpTool = new HelpTool(this);
		this.sessionMgr = new SessionManager(this);
		this.messengerSvc = new MessengerService(this);
		this.voucherMgr = new VoucherManager(this);
		this.itemMgr = new ItemManager(this);
		this.petMgr = new PetManager(this);
		this.botMgr = new BotManager(this);
		this.musicMgr = new MusicManager(this);
		this.catalog = new Catalog(this);
		this.roomMgr = new RoomManager(this);
		this.roomEventMgr = new RoomEventManager(this);
		this.navigator = new RoomNavigator(this);
		this.moderationCenter = new ModerationCenter(this);
		this.chatlogSvc = new ChatlogService(this);
		this.chatCmdMgr = new ChatCommandManager(this);
		this.wordFilterMgr = new WordFilterManager(this);
		
		// Custom
		this.autoCreditsUpdater = new AutoCreditsUpdater(this);
		this.autoPixelsUpdater = new AutoPixelsUpdater(this);
	}
	
	public void start()
	{
		this.userMgr.start();
		this.chatlogSvc.start();
		this.messengerSvc.start();
		this.roomMgr.start();
		this.sessionMgr.start();
		this.effectsChecker = this.getSystem().scheduleTask(new EffectExpirationChecker(this.sessionMgr), 1000);
		this.autoRespectUpdater.scheduleDailyAtMidnight();
		this.autoCreditsUpdater.reschedule();
		this.autoPixelsUpdater.reschedule();
	}
	
	public void reload()
	{
		this.accessCtrl.reload();
		this.achievementMgr.reload();
		this.subscriptionMgr.reload();
		this.helpTool.reload();
		this.itemMgr.reloadDefinitions();
		this.petMgr.reload();
		this.musicMgr.reload();
		this.catalog.reload();
		this.roomMgr.reloadModels();
		this.navigator.reload();
		this.moderationCenter.reload();
		this.chatCmdMgr.reload();
		this.wordFilterMgr.reload();
	}
	
	public void shutdown()
	{
		this.effectsChecker.stop();
		this.messengerSvc.shutdown();
		this.sessionMgr.shutdown();
		this.roomMgr.shutdown();
		this.chatlogSvc.shutdown();
		this.getLogger().info(this, "All subsystems shutdown.");
	}
	
	public void updateVersion()
	{
		this.getDb().execute("UPDATE external_texts SET `value`='" + SQLHelper.escape(this.getSystem().getVersion()) + "' WHERE `key` LIKE '%.server.version'");
	}
	
	public AccessControl getAccessCtrl()
	{
		return this.accessCtrl;
	}
	
	public AvailabilityManager getAvailabilityMgr()
	{
		return this.availabilityMgr;
	}
	
	public UserManager getUserMgr()
	{
		return this.userMgr;
	}
	
	public AchievementManager getAchievementMgr()
	{
		return this.achievementMgr;
	}
	
	public SubscriptionManager getSubscriptionMgr()
	{
		return this.subscriptionMgr;
	}

	public HelpTool getHelpTool()
	{
		return this.helpTool;
	}
	
	public SessionManager getSessionMgr()
	{
		return this.sessionMgr;
	}
	
	public MessengerService getMessengerSvc()
	{
		return this.messengerSvc;
	}
	
	public VoucherManager getVoucherMgr()
	{
		return this.voucherMgr;
	}
	
	public ItemManager getItemMgr()
	{
		return this.itemMgr;
	}
	
	public PetManager getPetMgr()
	{
		return this.petMgr;
	}
	
	public BotManager getBotMgr()
	{
		return this.botMgr;
	}
	
	public MusicManager getMusicMgr()
	{
		return this.musicMgr;
	}
	
	public Catalog getCatalog()
	{
		return this.catalog;
	}
	
	
	public RoomManager getRoomMgr()
	{
		return this.roomMgr;
	}
	
	public RoomEventManager getRoomEventMgr()
	{
		return this.roomEventMgr;
	}
	
	public RoomNavigator getNavigator()
	{
		return this.navigator;
	}
	
	public ModerationCenter getModerationCenter()
	{
		return this.moderationCenter;
	}
	
	public ChatlogService getChatlogSvc()
	{
		return this.chatlogSvc;
	}
	
	public ChatCommandManager getChatCmdMgr()
	{
		return this.chatCmdMgr;
	}
	
	public WordFilterManager getWordFilterMgr()
	{
		return this.wordFilterMgr;
	}
	
	public static abstract class Component extends System.Component
	{
		private GameManager gameMgr;
		
		public Component(GameManager gameMgr)
		{
			super(gameMgr.getSystem());
			this.gameMgr = gameMgr;
		}
		
		public GameManager getGameMgr()
		{
			return this.gameMgr;
		}
	}
}
