package habbod2.game.catalog;

import habbod2.game.items.EItemFlags;
import habbod2.game.items.ItemDefinition;
import habbod2.game.rooms.items.RoomItem;
import habbod2.game.users.User;
//import habbod2.game.catalog.filters.PrePurchaseFilter;

public class CatalogProduct
{
	public int id;
	public int pageId;
	public CatalogPage page;

	public String code;
	public int itemDefinitionId;
	public ItemDefinition itemDefinition;
	public int itemAmount;

	public int costCredits;
	public int costPixels;
	public int costQuestPoints;

	//private PrePurchaseFilter prepurchaseFilter;
	
	public boolean isPhysicalItem()
	{
		return (this.itemDefinition != null && this.itemDefinition.is(EItemFlags.FLOORITEM) || this.itemDefinition.is(EItemFlags.WALLITEM));
	}
	
	public char getType()
	{
		return this.itemDefinition.getType();
	}
	
	public boolean is(Class<? extends RoomItem> roomClass)
	{
		return (this.itemDefinition != null && this.itemDefinition.is(roomClass));
	}
	
	public boolean isPurchasableBy(User user)
	{
		// Can user access page?
		if(this.page == null || !this.page.isAccessibleTo(user))
		{
			return false;
		}
		
		// Can user actually afford this?
		if(user.credits < this.costCredits || user.pixels < this.costPixels)
		{
			return false;
		}

		// Purchasable!
		return true;
	}
	
	public boolean isGiftable(User user)
	{
		// Check flags
		if(!this.isPhysicalItem() || this.itemDefinition.is(EItemFlags.NOT_GIFTABLE))
		{
			return false;
		}
		
		// Check access
		if(!this.isPurchasableBy(user))
		{
			return false;
		}
		
		// Giftable!
		return true;
	}
}
