package habbod2.game.items;

import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.Map;

import habbod2.System;
import habbod2.storage.DatabaseResult;
import habbod2.communication.messages.outgoing.inventory.furni.FurniListRemoveMessageComposer;
import habbod2.communication.messages.outgoing.inventory.furni.FurniListUpdateMessageComposer;
import habbod2.game.sessions.Session;

public class ItemInventory
{
	private Session session;
	private Map<Integer, InventoryItem> items;

	public ItemInventory(Session session)
	{
		this.session = session;
		this.items = new LinkedHashMap<Integer, InventoryItem>(); // Use LinkedHashMap to preserve order (and don't use natural sorting on item IDs)
	}

	public void clear()
	{
		this.items.clear();
	}

	public void reload()
	{
		// Clear old items
		this.clear();

		// Get item mgr
		ItemManager itemMgr = System.get().getGameMgr().getItemMgr();
		
		// Reload items
		DatabaseResult result = System.get().getDb().query("SELECT * FROM items WHERE owner_id=" + this.session.getUser().id + " AND room_id=0");
		while(result.row())
		{
			InventoryItem item = new InventoryItem();
			item.id = result.getInt("id");
			item.definition = itemMgr.getDefinition(result.getInt("definition_id"));
			item.state = result.getString("state");
			item.data = result.getString("data");
			if(item != null)
			{
				this.items.put(item.id, item);
			}
		}
		result.release();
	}

	public boolean containsItem(int itemId)
	{
		return this.items.containsKey(itemId);
	}

	public InventoryItem getItem(int itemId)
	{
		return this.items.get(itemId);
	}

	public void addItem(InventoryItem item, boolean save)
	{
		item.update(this.session.getUser().id);
		this.items.put(item.id, item);
	}

	public InventoryItem removeItem(int itemId)
	{
		InventoryItem item = this.items.remove(itemId);
		if(item != null)
		{
			this.session.send(FurniListRemoveMessageComposer.compose(item));
		}
		return item;
	}

	public void refresh()
	{
		this.session.send(FurniListUpdateMessageComposer.compose());
	}

	public Collection<InventoryItem> getItems()
	{
		return this.items.values();
	}
}
