package habbod2.game.items;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import habbod2.game.GameManager;
import habbod2.storage.DatabaseResult;
import habbod2.storage.SQLHelper;
import habbod2.game.rooms.items.RoomItem;

public class ItemManager extends GameManager.Component
{
	private Map<Integer, ItemDefinition> definitions;

	public ItemManager(GameManager gameMgr)
	{
		super(gameMgr);
		this.definitions = Collections.emptyMap();
	}

	public void reloadDefinitions()
	{
		// Create new collection
		Map<Integer, ItemDefinition> tmpDefinitions = new HashMap<Integer, ItemDefinition>();

		// Load definitions from database
		try(DatabaseResult result = this.getDb().query("SELECT * FROM itemdefinitions"))
		{
			while(result.row())
			{
				ItemDefinition definition = ItemDefinition.parse(result);
				tmpDefinitions.put(definition.id, definition);
			}
		}

		// Set the instance field
		this.definitions = Collections.unmodifiableMap(tmpDefinitions);

		// Done!
		this.getLogger().info(this, String.format("Loaded %d item definitions.", this.definitions.size()));
	}

	public ItemDefinition getDefinition(int definitionId)
	{
		return this.definitions.get(definitionId);
	}
	
	public List<ItemDefinition> getDefinitionsByRoomClass(Class<? extends RoomItem> cls)
	{
		List<ItemDefinition> definitions = new ArrayList<ItemDefinition>();
		for(ItemDefinition definition : this.definitions.values())
		{
			if(definition.getRoomClass() == cls)
			{
				definitions.add(definition);
			}
		}
		
		return definitions;
	}

	public Collection<ItemDefinition> getAllDefinitions()
	{
		return this.definitions.values();
	}

	public void storeItem(Item item, int ownerId)
	{
		if(item.id <= 0)
		{
			item.id = this.getDb().executeReturnKey("INSERT INTO items(definition_id,owner_id,state,data) VALUES (" + item.definition.id + "," + ownerId + ",'" + SQLHelper.escape(item.state) + "','" + SQLHelper.escape(item.data) + "')");
		}
	}
	
	public int getItemRoomId(int itemId)
	{
		return this.getDb().queryInt("SELECT room_id FROM items WHERE id=" + itemId);
	}
	
	public InventoryItem getItem(int itemId)
	{
		try(DatabaseResult result = this.getDb().query("SELECT id,definition_id,state,data FROM items WHERE id=" + itemId))
		{
			if(result.row())
			{
				ItemDefinition definition = this.getDefinition(result.getInt("definition_id"));
				if(definition != null)
				{
					InventoryItem item = new InventoryItem();
					item.id = result.getInt("id");
					item.definition = definition;
					item.state = result.getString("state");
					item.data = result.getString("data");
					
					return item;
				}
			}
		}
		
		return null;
	}
}
