package habbod2.game.items.pets;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import habbod2.System;
import habbod2.storage.DatabaseResult;
import habbod2.game.sessions.Session;
import habbod2.communication.messages.outgoing.inventory.pets.*;

public class PetInventory
{
	private Session session;
	private Map<Integer, Pet> pets;

	public PetInventory(Session session)
	{
		this.session = session;
		this.pets = new HashMap<Integer, Pet>();
	}

	public void reload()
	{
		// Clear old items
		this.pets.clear();

		// Reload items
		DatabaseResult result = System.get().getDb().query("SELECT p.*,u.username AS owner FROM pets p JOIN users u ON u.id=p.owner_id WHERE owner_id=" + this.session.getUser().id + " AND room_id=0");
		while(result.row())
		{
			Pet pet = Pet.parse(result);
			if(pet != null)
			{
				this.pets.put(pet.id, pet);
			}
		}
		result.release();
	}

	public boolean containsPet(int petId)
	{
		return this.pets.containsKey(petId);
	}

	public Pet getPet(int petId)
	{
		return this.pets.get(petId);
	}

	public void addPet(Pet pet)
	{
		// Set the pet info
		this.pets.put(pet.id, pet);

		// Notify user
		this.session.send(PetAddedToInventoryMessageComposer.compose(pet));
	}

	public Pet removePet(int petId)
	{
		Pet pet = this.pets.remove(petId);
		if(pet != null)
		{
			this.session.send(PetRemovedFromInventoryMessageComposer.compose(pet));
		}

		return pet;
	}

	public final Collection<Pet> getPets()
	{
		return this.pets.values();
	}
}
