package habbod2.game.items.trading;

import java.util.List;
import java.util.ArrayList;

import habbod2.game.sessions.Session;
import habbod2.game.items.EItemFlags;
import habbod2.game.items.InventoryItem;
import habbod2.communication.messages.outgoing.inventory.trading.*;

public class TradePartner
{
	private Trade trade;
	private Session session;
	private boolean accepted;
	private boolean confirmed;
	private List<InventoryItem> offer;

	public TradePartner(Trade trade, Session session)
	{
		this.trade = trade;
		this.session = session;
		this.accepted = false;
		this.confirmed = false;
		this.offer = new ArrayList<InventoryItem>();

		this.session.setTrade(trade);
		this.session.getRoomUser().addStatus("trd");
	}

	public void clear()
	{
		this.offer.clear();

		this.session.setTrade(null);
		this.session.getRoomUser().removeStatus("trd");

		this.session.getItemInventory().refresh();
	}

	public void accept(boolean accept)
	{
		// Update accept/confirm status
		this.accepted = accept;
		if(!accept)
		{
			this.confirmed = false;
		}
		
		// Notify users
		this.trade.send(TradingAcceptMessageComposer.compose(this));
		
		// Move into confirm state?
		if(this.trade.bothAccepted())
		{
			this.trade.send(TradingConfirmationMessageComposer.compose());
		}
	}

	public void confirm()
	{
		// this.accepted = false;
		this.confirmed = true;
		this.trade.complete();
	}

	public void addItem(InventoryItem item)
	{
		if(this.offer.contains(item))
		{
			return;
		}

		if(!item.is(EItemFlags.ALLOW_TRADE))
		{
			return;
		}

		// Item added to trade!
		this.offer.add(item);
		this.trade.update();
	}

	public void removeItem(InventoryItem item)
	{
		// Item removed from trade!
		this.offer.remove(item);
		this.trade.update();
	}

	public boolean verifyItems()
	{
		// Verify the items are still here
		for(InventoryItem item : this.offer)
		{
			if(!this.session.getItemInventory().containsItem(item.id))
			{
				return false;
			}
		}
		return true;
	}

	public void transferItems(TradePartner to)
	{
		for(InventoryItem item : this.offer)
		{
			this.session.getItemInventory().removeItem(item.id);
			to.getSession().getItemInventory().addItem(item);;
		}
	}

	public boolean accepted()
	{
		return this.accepted;
	}
	
	public boolean confirmed()
	{
		return this.confirmed;
	}

	public Trade getTrade()
	{
		return this.trade;
	}

	public Session getSession()
	{
		return this.session;
	}

	public List<InventoryItem> getOffer()
	{
		return this.offer;
	}
}