package habbod2.game.marketplace;

import java.util.ArrayList;
import java.util.List;

import habbod2.game.GameManager;
import habbod2.game.items.ItemManager;
import habbod2.storage.DatabaseResult;
import habbod2.storage.SQLHelper;
import habbod2.util.StringUtil;

public class Marketplace extends GameManager.Component
{
	public enum SearchType
	{
		MostExpensive, LeastExpensive, TodayMostSold, TodayLeastSold, MostOffered, LeastOffered
	}

	public Marketplace(GameManager gameMgr)
	{
		super(gameMgr);
	}

	public MarketplaceItemStats getItemStats(int displayId)
	{
		return null;
	}

	public List<MarketplaceOffer> getOffersOfUser(int userId)
	{
		List<MarketplaceOffer> offers = new ArrayList<MarketplaceOffer>();

		try(DatabaseResult result = this.getDb().query(""))
		{
			while(result.row())
			{

			}
		}
		return offers;
	}

	public List<MarketplaceOffer> searchOffers(SearchType type, int minPrice, int maxPrice, String queryStr)
	{
		// Build query string
		String query = "SELECT o.id,d.id AS definition_id,o.price,COUNT(o.id) AS total,AVG(o.price) AS avg_price FROM marketplace_offers o JOIN items i ON i.id=o.item_id JOIN itemdefinitions d ON d.id=i.definition_id";

		// Criteria
		List<String> conditions = new ArrayList<String>(3);
		if(minPrice > 0)
			conditions.add("o.price >= " + minPrice);
		if(maxPrice > 0)
			conditions.add("o.price <= " + maxPrice);
		if(queryStr.length() > 0)
			conditions.add("d.name LIKE '%" + SQLHelper.escape(queryStr) + "%'");
		if(type == SearchType.TodayMostSold || type == SearchType.TodayLeastSold)
		{
			conditions.add("o.is_sold=1");
			conditions.add("DATE(timestamp) = DATE(NOW())");
		}
		if(conditions.size() > 0)
		{
			query += " WHERE " + StringUtil.join(" AND ", conditions);
		}

		// Grouping
		query += " GROUP BY d.id";

		// Ordering
		if(type != null)
		{
			switch(type)
			{
				case MostExpensive:
					query += " ORDER BY o.price DESC";
					break;

				case LeastExpensive:
					query += " ORDER BY o.price ASC";
					break;

				case TodayMostSold:
				case MostOffered:
					query += " ORDER BY total DESC";
					break;

				case TodayLeastSold:
				case LeastOffered:
					query += " ORDER BY total ASC";
			}
		}
		System.out.println(query);

		// Process result
		ItemManager itemMgr = this.getGameMgr().getItemMgr();
		List<MarketplaceOffer> offers = new ArrayList<MarketplaceOffer>();
		try(DatabaseResult result = this.getDb().query(query))
		{
			while(result.row())
			{
				MarketplaceOffer offer = new MarketplaceOffer();
				offer.id = result.getInt("id");
				offer.itemDefinition = itemMgr.getDefinition(result.getInt("definition_id"));
				offer.price = result.getInt("price");
				offer.totalAmount = result.getInt("total");
				offer.avgPrice = result.getInt("avg_price");
				if(offer.itemDefinition != null)
				{
					offers.add(offer);
				}
			}
		}

		return offers;
	}

	public int getCommissionPrice(float askingPrice)
	{
		return (int) Math.ceil((double) (askingPrice / 100.0f));
	}
}
