package habbod2.game.moderation;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import habbod2.storage.DatabaseResult;

public class ModerationPresetManager
{
	private ModerationCenter moderationCenter;
	private List<String> userMessagePresets;
	private List<String> roomMessagePresets;
	private Map<Integer, String> userActionPresetCategories;
	private Map<Integer, Map<String, String>> userActionPresetMessages;
	
	public ModerationPresetManager(ModerationCenter moderationCenter)
	{
		this.moderationCenter = moderationCenter;
		this.userMessagePresets = new ArrayList<String>();
		this.roomMessagePresets = new ArrayList<String>();
		this.userActionPresetCategories = new HashMap<Integer, String>();
		this.userActionPresetMessages = new HashMap<Integer, Map<String, String>>();
	}
	
	public void reload()
	{
		// Clear current presets
		this.userMessagePresets.clear();
		this.userActionPresetCategories.clear();
		this.userActionPresetMessages.clear();
		
		// Load user message presets
		DatabaseResult result = this.moderationCenter.getDb().query("SELECT * FROM moderation_presets");
		while(result.row())
		{
			String type = result.getString("type");
			String message = result.getString("message");
			if(type.equals("user"))
			{
				this.userMessagePresets.add(message);
			}
			else if(type.equals("room"))
			{
				this.roomMessagePresets.add(message);
			}
		}
		
		// Load user action presets
		result.query("SELECT * FROM moderation_preset_action_categories");
		while(result.row())
		{
			this.userActionPresetCategories.put(result.getInt("id"), result.getString("name"));
		}
		result.query("SELECT * FROM moderation_preset_action_messages");
		while(result.row())
		{
			int parentId = result.getInt("parent_id");
			if(!this.userActionPresetMessages.containsKey(parentId))
			{
				this.userActionPresetMessages.put(parentId, new HashMap<String, String>());
			}
			this.userActionPresetMessages.get(parentId).put(result.getString("name"), result.getString("message"));
		}
		result.release();
		
		// Done!
		this.moderationCenter.getLogger().info(this.moderationCenter, "Loaded all presets.");
	}
	
	public List<String> getUserMessagePresets()
	{
		return this.userMessagePresets;
	}
	
	public List<String> getRoomMessagePresets()
	{
		return this.roomMessagePresets;
	}
	
	public Map<String, Map<String, String>> getUserActionPresets()
	{
		Map<String, Map<String, String>> result = new HashMap<String, Map<String, String>>();
		for(Entry<Integer, String> category : this.userActionPresetCategories.entrySet())
		{
			result.put(category.getValue(), new HashMap<String, String>());
			if(this.userActionPresetMessages.containsKey(category.getKey()))
			{
				for(Entry<String, String> preset : this.userActionPresetMessages.get(category.getKey()).entrySet())
				{
					result.get(category.getValue()).put(preset.getKey(), preset.getValue());
				}
			}
		}
		
		return result;
	}
}
