package habbod2.game.pets;

import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;

import habbod2.storage.DatabaseResult;
import habbod2.storage.SQLHelper;
import habbod2.game.GameManager;
import habbod2.game.sessions.Session;

public class PetManager extends GameManager.Component
{
	private Map<Integer, PetType> types;
	
	public PetManager(GameManager gameMgr)
	{
		super(gameMgr);
		this.types = new HashMap<Integer, PetType>();
	}
	
	public void reload()
	{
		this.types.clear();
	
		try(DatabaseResult result = this.getDb().query("SELECT * FROM pettypes WHERE enabled=1"))
		{
			while(result.row())
			{
				PetType type = new PetType();
				type.id = result.getInt("id");
				type.sellableBreeds = result.getInt("sellable_breeds");
				type.canSwim = result.getBoolean("can_swim");
				type.mountHeight = result.getFloat("mount_height");
				this.types.put(type.id, type);
			}
		}
		
		this.getLogger().info(this, String.format("Loaded %d pet types.", this.types.size()));
	}
	
	public PetType getPetType(int typeId)
	{
		return this.types.get(typeId);
	}

	public Pet getPetById(int petId)
	{
		try(DatabaseResult result = this.getDb().query("SELECT p.*,u.username AS owner FROM pets p JOIN users u ON u.id=p.owner_id WHERE p.id=" + petId))
		{
			if(result.row())
			{
				return this.processPetFromRow(result);
			}
			else
			{
				return null;
			}
		}
	}
	
	public void createPet(int userId, String name, int type, int breed, String color)
	{
		// Additional default info
		int experience = 0;
		int energy = 100;
		int nutrition = 0;
		int respect = 0;
		
		// Store pet, get ID
		int petId = this.getDb().executeReturnKey("INSERT INTO pets(name,type,breed,color,experience,energy,nutrition,respect,owner_id,created_at) VALUES ('" + SQLHelper.escape(name) + "'," + type + "," + breed + ",'" + SQLHelper.escape(color) + "'," + experience + "," + energy + "," + nutrition + "," + respect + "," + userId + ",'" + SQLHelper.getDateTime() + "')");
		
		// Owner online?
		Session session = this.getSystem().getGameMgr().getSessionMgr().getSessionByUserId(userId);
		if(session != null)
		{
			Pet pet = this.getPetById(petId);
			if(pet != null)
			{
				session.getPetInventory().addPet(pet);
			}
		}
	}
	
	public List<Pet> getInventoryPetsForUser(int userId)
	{
		List<Pet> pets = new ArrayList<Pet>(0);
		
		try(DatabaseResult result = this.getDb().query("SELECT p.*,u.username AS owner FROM pets p JOIN users u ON u.id=p.owner_id WHERE p.owner_id=" + userId + " AND p.room_id=0"))
		{
			while(result.row())
			{
				Pet pet = this.processPetFromRow(result);
				if(pet != null)
				{
					pets.add(pet);
				}
			}
		}
		
		return pets;
	}

	public List<Pet> getPetsForRoom(int roomId)
	{
		List<Pet> pets = new ArrayList<Pet>(0);
		
		try(DatabaseResult result = this.getDb().query("SELECT p.*,u.username AS owner FROM pets p JOIN users u ON u.id=p.owner_id WHERE p.room_id=" + roomId))
		{
			while(result.row())
			{
				Pet pet = this.processPetFromRow(result);
				if(pet != null)
				{
					pets.add(pet);
				}
			}
		}
		
		return pets;
	}
	
	private Pet processPetFromRow(DatabaseResult row)
	{
		// Valid pet type?
		Pet pet = new Pet();
		pet.type = this.types.get(row.getInt("type"));
		if(pet.type == null)
		{
			return null;
		}
		
		// Process the remaining data
		pet.id = row.getInt("id");
		pet.name = row.getString("name");
		pet.breed = row.getByte("breed");
		pet.color = row.getString("color");
		pet.experience = row.getInt("experience");
		pet.energy = row.getInt("energy");
		pet.nutrition = row.getInt("nutrition");
		pet.respect = row.getInt("respect");
		pet.ownerId = row.getInt("owner_id");
		pet.ownerName = row.getString("owner");
		pet.createdTime = row.getTime("created_at");
		pet.roomId = row.getInt("room_id");
		pet.x = row.getInt("x");
		pet.y = row.getInt("y");
		pet.z = row.getFloat("z");
		
		// Calculate level (based on exp)
		pet.recalcLevel();
		
		return pet;
	}
	
	public int types()
	{
		return this.types.size();
	}
}
