package habbod2.game.rooms.games;

import habbod2.game.rooms.instances.RoomUser;
import habbod2.game.rooms.items.RoomFloorItem;
import habbod2.game.util.GameUtil;

public abstract class GameTimerItem extends RoomFloorItem
{
	// Displays the remaining time for a game and serves as a game 'controller'
	
	@Override
	protected boolean onToggle(RoomUser user, int instruction)
	{
		// Needs rights
		if(user == null || user.hasRoomRights)
		{
			return false;
		}
		
		// Start/pause game
		if(instruction == 1)
		{
			if(this.hasTimeout())
			{
				this.clearTimeout();
			}
			else
			{
				this.gameStart();
				this.setTimeout(GameUtil.secondsToRoomPulses(1));
			}
		}
		
		// Adjust time
		else if(instruction == 2)
		{
			int timeLeft = GameUtil.parseInt(this.state);
			if(timeLeft == 0)
			{
				timeLeft = 30;
			}
			else if(timeLeft == 30)
			{
				timeLeft = 60;
			}
			else if(timeLeft == 60)
			{
				timeLeft = 120;
			}
			else if(timeLeft == 120)
			{
				timeLeft = 180;
			}
			else if(timeLeft == 180)
			{
				timeLeft = 240;
			}
			else if(timeLeft == 240)
			{
				timeLeft = 300;
			}
			else if(timeLeft == 300)
			{
				timeLeft = 600;
			}
			else
			{
				timeLeft = 0;
			}
			this.setState(Integer.toString(timeLeft));
			this.clearTimeout();
		}
		
		return true;
	}
	
	@Override
	public void onTimeout()
	{
		int timeLeft = GameUtil.parseInt(this.state);
		if(timeLeft > 0)
		{
			this.gameTick();
			this.setState(Integer.toString(timeLeft - 1));
			this.setTimeout(GameUtil.secondsToRoomPulses(1));	
		}
		else
		{
			this.gameEnd();
		}
	}
	
	protected abstract void gameStart();
	
	protected abstract void gameTick();
	
	protected abstract void gameEnd();
}
