package habbod2.game.rooms.geometry;

import habbod2.game.util.GameUtil;

public enum Direction
{
	North,
	NorthEast,
	East,
	SouthEast,
	South,
	SouthWest,
	West,
	NorthWest;
	
	public final int num;
	public final int modX;
	public final int modY;
	
	Direction()
	{
		this.num = this.ordinal();
		switch(this.num)
		{
			// North
			case 0: this.modX = 0; this.modY = -1; break;
			
			// NorthEast
			case 1: this.modX = -1; this.modY = +1; break;
			
			// East
			case 2: this.modX = +1; this.modY = 0; break;
			
			// SouthEast
			case 3: this.modX = +1; this.modY = +1; break;
			
			// South
			case 4: this.modX = 0; this.modY = +1; break;
			
			// SouthWest
			case 5: this.modX = -1; this.modY = +1; break;
			
			// West
			case 6: this.modX = -1; this.modY = 0; break;
			
			// NorthWest
			case 7: this.modX = -1; this.modY = -1; break;
			
			// Uh...
			default: this.modX = 0; this.modY = 0;
		}
	}
	
	public final Direction invert()
	{
		switch(this)
		{
			case North: return South;
			case NorthEast: return SouthWest;
			case East: return West;
			case SouthEast: return NorthWest;
			case South: return North;
			case SouthWest: return NorthEast;
			case West: return East;
			case NorthWest: return SouthEast;
			default: return this;
		}
	}
	
	public final int num()
	{
		return this.ordinal();
	}
	
	public static final Direction get(int num)
	{
		return Direction.values()[num];
	}
	
	public static final Direction random()
	{
		return Direction.get(GameUtil.getRandomInt(0, 7));
	}
	
	public static final Direction calculateDirection(int x, int y, int tileX, int tileY)
	{
		if(x > tileX && y > tileY)
		{
			return NorthWest;
		}
		else if(x < tileX && y < tileY)
		{
			return SouthEast;
		}
		else if(x > tileX && y < tileY)
		{
			return SouthWest;
		}
		else if(x < tileX && y > tileY)
		{
			return NorthEast;
		}
		else if(x > tileX)
		{
			return West;
		}
		else if(x < tileX)
		{
			return East;
		}
		else if(y < tileY)
		{
			return South;
		}
		else
		{
			return North;
		}
	}

	public static final Direction ofMove(int x, int y, int x2, int y2)
	{
		if(x == x2)
		{
			if(y < y2) return South;
			else return North;
		}
		else if(x > x2)
		{
			if(y == y2) return West;
			else if(y < y2) return SouthWest;
			else return NorthWest;
		}
		else if(x < x2)
		{
			if(y == y2) return East;
			else if(y < y2) return SouthEast;
			else return NorthEast;
		}
		else
		{
			return North;
		}
	}
}