package habbod2.game.rooms.items;

import java.util.List;
import java.util.ArrayList;

import habbod2.System;
import habbod2.storage.SQLHelper;
import habbod2.game.items.EItemFlags;
import habbod2.game.rooms.Room;
import habbod2.game.rooms.instances.RoomAvatar;
import habbod2.game.rooms.instances.RoomTile;
import habbod2.game.rooms.instances.RoomUser;
import habbod2.game.rooms.items.wired.WiredTrigger;
import habbod2.game.rooms.items.wired.triggers.*;
import habbod2.communication.messages.outgoing.room.engine.ObjectDataUpdateMessageComposer;

public abstract class RoomFloorItem extends RoomItem
{
	public int x;
	public int y;
	public float z;
	public byte rotation;
	
	private List<WiredTrigger> triggers;
	
	@Override
	public void update()
	{
		Room room = this.getRoom().getInfo();
		System.get().getDb().execute("UPDATE items SET owner_id=" + room.ownerId + ",room_id=" + room.id + ",x=" + this.x + ",y=" + this.y + ",z=" + this.z + ",rotation=" + this.rotation + ",state='" + SQLHelper.escape(this.state) + "',data='" + SQLHelper.escape(this.data) + "' WHERE id=" + this.id);
 	}
	
	@Override
	public void refreshState()
	{
		this.getRoom().broadcast(ObjectDataUpdateMessageComposer.compose(this));
	}
	
	public float getTopHeight()
	{
		return (this.z + this.definition.height);
	}
	
	public RoomTile getTile()
	{
		return this.getRoom().getMap().getTile(this.x, this.y);
	}
	
	public List<RoomTile> getTiles()
	{
		return this.getRoom().getMap().getAffectedTiles(this.x, this.y, this.definition.length, this.definition.width, this.rotation);
	}
	
	public RoomTile getTileInFront()
	{
		int x = this.x;
		int y = this.y;
		if(this.rotation == 0)
		{
			y--;
		}
		else if(this.rotation == 2)
		{
			x++;
		}
		else if(this.rotation == 4)
		{
			y++;
		}
		else if(this.rotation == 6)
		{
			x--;
		}
		return this.getRoom().getMap().getTile(x, y);
	}

	public RoomTile getTileBehind()
	{
		int x = this.x;
		int y = this.y;
		if(this.rotation == 0)
		{
			y++;
		}
		else if(this.rotation == 2)
		{
			x--;
		}
		else if(this.rotation == 4)
		{
			y--;
		}
		else if(this.rotation == 6)
		{
			x++;
		}
		return this.getRoom().getMap().getTile(x, y);
	}
	
	public void onGenerateMapTile(RoomTile tile)
	{
		// Can stand on top?
		if(this.is(EItemFlags.CAN_STAND_ON_TOP))
		{
			// Tile not closed yet?
			if(!tile.is(RoomTile.CLOSED))
			{
				tile.state = RoomTile.OPEN;
				tile.topItem = this;
				tile.height = this.getTopHeight();
			}
		}
		
		// Can sit on top?
		else if(this.is(EItemFlags.CAN_SIT_ON_TOP))
		{
			tile.state = (RoomTile.CLOSED | RoomTile.LASTSTEP);
			tile.topItem = this;
		}
		
		// Can lay on top?
		else if(this.is(EItemFlags.CAN_LAY_ON_TOP))
		{
			tile.state = (RoomTile.CLOSED | RoomTile.LASTSTEP);
			tile.topItem = this;
			if(this.rotation == 0 || this.rotation == 4)
			{
				tile.redirect = this.getRoom().getMap().getTile(tile.x, this.y);
			}
			else
			{
				tile.redirect = this.getRoom().getMap().getTile(this.x, tile.y);
			}
		}
		
		// Solid item!
		else
		{
			// No last step item here?
			if(!tile.is(RoomTile.LASTSTEP))
			{
				// Close the tile
				tile.height = this.getTopHeight();
				tile.state = RoomTile.CLOSED;
			}
		}
	}
	
	public void onAvatarStepOn(RoomAvatar avatar)
	{
		// Default item interaction types
		if(this.is(EItemFlags.CAN_SIT_ON_TOP))
		{
			avatar.headRotation = avatar.bodyRotation = this.rotation;
			avatar.addStatus("sit", Float.toString(avatar instanceof RoomUser ? this.definition.height : this.definition.height / 2));
		}
		else if(this.is(EItemFlags.CAN_LAY_ON_TOP))
		{
			avatar.headRotation = avatar.bodyRotation = this.rotation;
			avatar.addStatus("lay", Float.toString(this.definition.height) + " null");
		}
		
		// Raise triggers
		if(avatar instanceof RoomUser)
		{
			this.executeTriggers((RoomUser)avatar, UserStepsOnItemTrigger.class);
		}
	}

	public void onAvatarStepOff(RoomAvatar avatar)
	{
		// Raise triggers
		if(avatar instanceof RoomUser)
		{
			this.executeTriggers((RoomUser)avatar, UserStepsOffItemTrigger.class);
		}
	}
	
	@Override
	public void onStateChange(RoomUser user)
	{
		if(user != null)
		{
			this.executeTriggers(user, UserTogglesItemTrigger.class);
		}
	}
	
	public void addTrigger(WiredTrigger trigger)
	{
		if(this.triggers == null)
		{
			this.triggers = new ArrayList<WiredTrigger>(1);
		}
		this.triggers.add(trigger);
	}
	
	public void removeTrigger(WiredTrigger trigger)
	{
		if(this.triggers != null)
		{
			this.triggers.remove(trigger);
		}
	}
	
	public void executeTriggers(RoomUser user, Class<? extends WiredTrigger> type)
	{
		if(this.triggers != null)
		{
			for(WiredTrigger trigger : this.triggers)
			{
				if(trigger.getClass() == type)
				{
					trigger.execute(user);
				}
			}
		}
	}
}
