package habbod2.game.rooms.items;

import habbod2.game.rooms.instances.RoomInstance;
import habbod2.game.rooms.instances.RoomUser;
import habbod2.game.sessions.Session;
import habbod2.game.items.InventoryItem;
import habbod2.game.items.ItemBase;

public abstract class RoomItem extends ItemBase
{
	private RoomInstance room;
	private int pulseTimer;
	
	public final RoomInstance getRoom()
	{
		return this.room;
	}
	
	public final void setRoom(RoomInstance newRoom)
	{
		if(this.room != null || newRoom != null)
		{
			if(newRoom != null)
			{
				this.room = newRoom;
				this.onRoomAdd(newRoom);
			}
			else
			{
				this.onRoomRemove(this.room);
				this.room = null;
			}
		}
	}
	
	public final void clearRoom()
	{
		this.setRoom(null);
	}
	
	protected void onRoomAdd(RoomInstance room)
	{
		
	}
	
	protected void onRoomRemove(RoomInstance room)
	{
		
	}
	
	public boolean onPlace(Session session, RoomInstance room)
	{
		// Optionally overridden by impl class
		return true;
	}

	public boolean onPickup(Session session)
	{
		// Optionally overridden by impl class
		return true;
	}
	
	public final void setState(int state)
	{
		this.setState(Integer.toString(state));
	}
	
	public final void setState(String state)
	{
		this.state = state;
		this.refreshState();
		this.requiresDbUpdate();
	}
	
	public final void setTempState(int state)
	{
		this.setTempState(Integer.toString(state));
	}

	public final void setTempState(String state)
	{
		String backup = this.state;
		this.state = state;
		this.refreshState();
		this.state = backup;
	}

	public abstract void refreshState();
	
	public void toggle(RoomUser user, int instruction)
	{
		// Toggle
		String previousState = this.state;
		this.onToggle(user, instruction);
		
		// State changed?
		if(this.state != null && !this.state.equals(previousState))
		{
			this.onStateChange(user);
		}
	}
	
	protected boolean onToggle(RoomUser user, int instruction)
	{
		return false;
	}
	
	public void onStateChange(RoomUser user)
	{
		
	}

	public boolean requiresPulse()
	{
		return this.hasTimeout();
	}

	protected final boolean hasTimeout()
	{
		return(this.pulseTimer > 0);
	}

	protected final void setTimeout(int pulses)
	{
		this.pulseTimer = pulses;
	}

	protected final void clearTimeout()
	{
		this.pulseTimer = 0;
	}

	public final void pulse()
	{		
		// Pulse!
		this.onPulse();
		if(this.pulseTimer > 0)
		{
			this.pulseTimer--;
		}
		
		// End of timeout?
		if(this.pulseTimer == 0)
		{
			this.clearTimeout();
			this.onTimeout();
		}
	}

	protected void onPulse()
	{
		// Optionally overridden by impl class
	}

	protected void onTimeout()
	{
		// Optionally overridden by impl class
	}
	
	public abstract void update();

	public InventoryItem toInventoryItem()
	{
		InventoryItem item = new InventoryItem();
		item.id = this.id;
		item.definition = this.definition;
		item.state = this.state;
		item.data = this.data;
		
		return item;
	}
}
