package habbod2.game.rooms.minigames;

import habbod2.System;
import static habbod2.core.Locale._I;
import habbod2.storage.Database;
import habbod2.game.rooms.instance.RoomAvatar;
import habbod2.game.rooms.instance.RoomInstance;
import habbod2.game.rooms.instance.RoomUser;
import habbod2.game.rooms.items.RoomFloorItem;
import habbod2.game.rooms.items.minigame.ball.BallItem;
import habbod2.game.rooms.items.minigame.ball.GoalItem;
import habbod2.game.rooms.minigames.GameTeam.ETeam;
import habbod2.game.rooms.minigames.GameUserPlayer.GameValues;

public class BallGame extends Game
{
	public BallGame(RoomInstance room)
	{
		super(room);
	}
	
	@Override
	public int getRequiredTeamsWithPlayers()
	{
		return 0;
	}

	@Override
	protected void onStart()
	{
		
	}
	
	@Override
	protected void onStop(GameTeam winner)
	{
		Database db = System.get().getDb();
		
		// Update stats
		for(GameTeam team : this.getTeams())
		{
			for(GameUserPlayer player : team.getPlayers())
			{
				if(player.getScore() > 0)
				{
					db.execute("UPDATE users SET bg_goals=bg_goals + " + player.getScore() + " WHERE id=" + player.getUser().getInfo().id);
				}
			}
		}
	}
	
	@Override
	protected void onPulse()
	{
		// Check all the balls
		for(RoomFloorItem item : this.getRoom().getFloorItems())
		{
			// This a ball?
			if(item instanceof BallItem)
			{
				// Recently shot by avatar?
				BallItem ball = (BallItem)item;
				RoomAvatar avatar = ball.getLastAvatar();
				if(avatar != null)
				{
					// Ball in goal?
					for(RoomFloorItem tmp : ball.getTile().getItems())
					{
						if(tmp instanceof GoalItem)
						{
							GoalItem goal = (GoalItem)tmp;
							
							// Handle avatar/user scoring
							GameTeam team = this.getTeam(goal.getTeamId());
							if(avatar instanceof RoomUser)
							{
								RoomUser user = (RoomUser)avatar;
								
								// Update team (user is in team last scored for)
								team.addPlayer(user);
								
								// Score!
								user.getPlayer().addScore(1);
								user.wave();
							}
							else
							{
								avatar.chat(_I("GOAL!"), true);
								this.getTeam(goal.getTeamId()).addScore(1);
							}
							
							// Processed!
							ball.clearLastAvatar();
						}
					}
				}
			}
		}
	}

	@Override
	public GameValues createValuesForPlayer()
	{
		return null;
	}
	
	public static final ETeam getOpponent(ETeam team)
	{
		if(team == ETeam.Red)
		{
			return ETeam.Blue;
		}
		else if(team == ETeam.Blue)
		{
			return ETeam.Red;
		}
		else if(team == ETeam.Green)
		{
			return ETeam.Yellow;
		}
		else if(team == ETeam.Yellow)
		{
			return ETeam.Green;
		}
		else
		{
			return team;
		}
	}
}
