package habbod2.game.rooms.minigames;

import habbod2.System;
import habbod2.game.rooms.instance.RoomInstance;
import habbod2.game.rooms.items.RoomFloorItem;
import habbod2.game.rooms.items.banzai.ApparatusItem;
import habbod2.game.rooms.items.banzai.TileItem;
import habbod2.storage.Database;

public class BanzaiGame extends Game
{
	private int sealableTileCount;
	
	public BanzaiGame(RoomInstance room)
	{
		super(room);
	}
	
	protected void onStart()
	{
		this.setStartDelay(3);
		this.sealableTileCount = 0;
	}
	
	protected void onStartProcessItem(RoomFloorItem item)
	{
		if(item instanceof TileItem)
		{
			this.sealableTileCount++;
		}
		else if(item instanceof ApparatusItem)
		{
			((ApparatusItem)item).enable();
		}
	}
	
	protected void onStartDelayExpired()
	{
		// Enable tiles (lights on)
		for(RoomFloorItem item : this.getRoom().getFloorItems())
		{
			if(item instanceof TileItem)
			{
				((TileItem)item).setState(true, null, TileItem.LEVEL_EMPTY);
			}
		}
	}

	@Override
	protected void onStop(GameTeam winner)
	{
		Database db = System.get().getDb();
		
		// Update game wins?
		if(winner != null)
		{
			for(GameUserPlayer player : winner.getPlayers())
			{
				db.execute("UPDATE users SET bb_wins=bb_wins + 1 WHERE id=" + player.getUser().getInfo().id);
			}
		}
		
		// Update stats
		for(GameTeam team : this.getTeams())
		{
			for(GameUserPlayer player : team.getPlayers())
			{
				db.execute("UPDATE users SET bb_tiles=bb_tiles + " + player.getStats() + " WHERE id=" + player.getUser().getInfo().id);
			}
		}
	}
	
	@Override
	protected void onStopProcessItem(RoomFloorItem item, GameTeam winner)
	{
		// When game stops because all tiles have been taken, the winning tiles flash
		if(item instanceof TileItem)
		{
			TileItem tile = (TileItem)item;
			if(this.sealableTileCount > 0)
			{
				tile.setState(0);
			}
			else
			{
				if(winner != null && tile.getTeamId() == winner.getId())
				{
					tile.flashFor(6);
				}
				else
				{
					tile.disableAfter(6);
				}
			}
		}
		else if(item instanceof ApparatusItem)
		{
			ApparatusItem apparatus = (ApparatusItem)item;
			if(winner != null)
			{
				apparatus.showWinner(winner.getId());
			}
			else
			{
				apparatus.disable();
			}
		}
	}
	
	public void sealedTile()
	{
		this.sealableTileCount--;
		if(this.sealableTileCount == 0)
		{
			this.stop();
		}
	}
}
