package habbod2.game.rooms.minigames;

import habbod2.System;
import habbod2.storage.Database;
import habbod2.game.rooms.instance.RoomInstance;
import habbod2.game.rooms.items.RoomFloorItem;
import habbod2.game.rooms.items.minigame.banzai.ApparatusItem;
import habbod2.game.rooms.items.minigame.banzai.GameTileItem;
import habbod2.game.rooms.minigames.GameUserPlayer.GameValues;

public class BanzaiGame extends Game
{
	private int sealableTileCount;
	
	public BanzaiGame(RoomInstance room)
	{
		super(room);
	}
	
	protected void onStart()
	{
		this.setStartDelay(3);
		this.sealableTileCount = 0;
	}
	
	protected void onStartProcessItem(RoomFloorItem item)
	{
		if(item instanceof GameTileItem)
		{
			this.sealableTileCount++;
		}
		else if(item instanceof ApparatusItem)
		{
			((ApparatusItem)item).enable();
		}
	}
	
	protected void onStartDelayExpired()
	{
		// Enable tiles (lights on)
		for(RoomFloorItem item : this.getRoom().getFloorItems())
		{
			if(item instanceof GameTileItem)
			{
				((GameTileItem)item).setState(true, null, GameTileItem.LEVEL_EMPTY);
			}
		}
	}

	@Override
	protected void onStop(GameTeam winner)
	{
		Database db = System.get().getDb();
		
		// Update game wins?
		if(winner != null)
		{
			for(GameUserPlayer player : winner.getPlayers())
			{
				db.execute("UPDATE users SET bb_wins=bb_wins + 1 WHERE id=" + player.getUser().getInfo().id);
			}
		}
		
		// Update stats
		for(GameTeam team : this.getTeams())
		{
			for(GameUserPlayer player : team.getPlayers())
			{
				BanzaiValues values = (BanzaiValues)player.getValues();
				if(values.sealedTileCount > 0)
				{
					db.execute("UPDATE users SET bb_tiles=bb_tiles + " + values.sealedTileCount + " WHERE id=" + player.getUser().getInfo().id);
				}
			}
		}
	}
	
	@Override
	protected void onStopProcessItem(RoomFloorItem item, GameTeam winner)
	{
		// TODO: when game stops because all tiles have been taken, the winning tiles flash
		if(item instanceof GameTileItem)
		{
			GameTileItem tile = (GameTileItem)item;
			tile.setState(0);
		}
		
		// Show the winner
		else if(item instanceof ApparatusItem)
		{
			ApparatusItem apparatus = (ApparatusItem)item;
			apparatus.showWinner((winner != null) ? winner.getId() : null);
		}
	}
	
	@Override
	protected void onPulse()
	{
		// Process tile coloring/filling/whatever for all players in all teams
		for(GameTeam team : this.getTeams())
		{
			for(GameUserPlayer player : team.getPlayers())
			{
				// Get this player's Banzai-specific values
				BanzaiValues values = (BanzaiValues) player.getValues();
				
				// Process pending tile?
				if(values.pendingTile != null)
				{
					// Take pending tile
					GameTileItem tile = values.pendingTile;
					values.pendingTile = null;
					
					// Tile is not sealed yet?
					if(tile.getLevel() < GameTileItem.LEVEL_SEALED)
					{
						// Update the tile
						if(tile.getTeamId() != team.getId())
						{
							tile.setState(true, team.getId(), GameTileItem.LEVEL_EMPTY);
						}
						else
						{
							tile.setState(true, team.getId(), tile.getLevel() + 1);
						}
						
						// Scored a point!
						player.addScore(1);
						
						// Tile is now sealed?
						if(tile.getLevel() == GameTileItem.LEVEL_SEALED)
						{
							this.sealableTileCount--;
							values.sealedTileCount++;
						}
					}
					
					// Set as last
					values.lastTile = tile;
				}
				
				// On a different tile item?
				RoomFloorItem item = player.getUser().getTile().topItem;
				if(item instanceof GameTileItem && item != values.lastTile)
				{
					// Process this one in the next pulse
					values.pendingTile = (GameTileItem)item;
				}
			}
		}
		
		// Show leading team
		/*
		GameTeam leader = this.getTeamWithBestScore();
		for(RoomFloorItem item : this.getRoom().getFloorItems())
		{
			if(item instanceof ApparatusItem)
			{
				ApparatusItem apparatus = (ApparatusItem)item;
				apparatus.showLeader((leader != null) ? leader.getId() : null);
			}
		}*/
		
		// Game ended?
		if(this.sealableTileCount <= 0)
		{
			this.stop();
		}
	}

	@Override
	public GameValues createValuesForPlayer()
	{
		return new BanzaiValues();
	}
	
	private class BanzaiValues implements GameUserPlayer.GameValues
	{
		public GameTileItem pendingTile;
		public GameTileItem lastTile;
		public int sealedTileCount;
	}
}
