package habbod2.game.rooms.minigames;

import habbod2.game.rooms.instance.RoomUser;
import habbod2.game.rooms.items.RoomFloorItem;
import habbod2.game.rooms.items.minigame.GateItem;
import habbod2.game.rooms.items.minigame.ScoreboardItem;
import habbod2.game.rooms.items.wired.triggers.GameScoreAchievedTrigger;

import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

public class GameTeam
{
	public enum ETeam
	{
		Red,
		Green,
		Blue,
		Yellow
	}
	
	public static final int MAX_PLAYERS = 5;
	
	private ETeam id;
	private Game game;
	private List<GameUserPlayer> players;
	private int score;
	
	public GameTeam(ETeam teamId, Game game)
	{
		this.id = teamId;
		this.game = game;
		this.players = new CopyOnWriteArrayList<GameUserPlayer>();
	}
	
	public boolean addPlayer(RoomUser user)
	{
		// Team is full?
		if(this.players.size() >= MAX_PLAYERS)
		{
			return false;
		}
		
		// Add player
		GameUserPlayer session = new GameUserPlayer(user, this);
		user.setPlayer(session);
		this.players.add(session);
		
		// Update gate
		this.updateGate();
		
		return true;
	}

	public void removePlayer(GameUserPlayer session)
	{
		if(this.players.remove(session))
		{
			this.updateGate();
		}
	}
	
	private void updateGate()
	{
		GateItem gate = this.getGate();
		if(gate != null)
		{
			gate.setPlayerCount(this.players.size());
		}
	}
	
	public ETeam getId()
	{
		return this.id;
	}
	
	public Game getGame()
	{
		return this.game;
	}
	
	public GateItem getGate()
	{
		for(RoomFloorItem item : this.game.getRoom().getFloorItems())
		{
			if(item instanceof GateItem)
			{
				GateItem gate = (GateItem)item;
				if(gate.getTeamId() == this.id)
				{
					return gate;
				}
			}
		}
		
		return null;
	}
	
	public ScoreboardItem getScoreboard()
	{
		for(RoomFloorItem item : this.game.getRoom().getFloorItems())
		{
			if(item instanceof ScoreboardItem)
			{
				ScoreboardItem scoreboard = (ScoreboardItem)item;
				if(scoreboard.getTeamId() == this.id)
				{
					return scoreboard;
				}
			}
		}
		
		return null;
	}
	
	public int getScore()
	{
		return this.score;
	}
	
	public void addScore(int points)
	{
		// Add the score
		this.score += points;
		
		// Scoreboard placed?
		ScoreboardItem scoreboard = this.getScoreboard();
		if(scoreboard != null)
		{
			scoreboard.setScore(this.score);
		}
		
		// Triggers!
		for(RoomFloorItem item : this.game.getRoom().getFloorItems())
		{
			if(item instanceof GameScoreAchievedTrigger)
			{
				GameScoreAchievedTrigger trigger = (GameScoreAchievedTrigger)item;
				if(this.score == trigger.getParam(GameScoreAchievedTrigger.PARAM_SCORE))
				{
					trigger.execute(null);
				}
			}
		}
	}
	
	public void reset()
	{
		this.score = 0;
		for(GameUserPlayer player : this.players)
		{
			player.reset();
		}
	}
	
	public List<GameUserPlayer> getPlayers()
	{
		return this.players;
	}
}
