package habbod2.game.rooms.minigames.game;

import habbod2.game.rooms.geometry.Direction;
import habbod2.game.rooms.instance.RoomMap;
import habbod2.game.rooms.instance.RoomTile;
import habbod2.game.rooms.items.minigame.banzai.GameTileItem;
import habbod2.game.rooms.minigames.GameTeam.ETeam;

import java.awt.Point;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.List;

public class BanzaiSolver
{
	public static void solveShape(RoomMap map, GameTileItem tile, ETeam teamId)
	{
		// Create a shape of all the locked tiles in this team's color
		Polygon shape = new Polygon();
		List<Point> done = new ArrayList<Point>();
		BanzaiSolver.discover(map, tile, teamId, shape, done);
		
		// Check all the inner tiles
		Rectangle box = shape.getBounds();
		if(!box.isEmpty())
		{
			// Box closed?
			//for(int x = (int)box.getMinX(); x <= (int)box.getMaxX
			for(int x = (int)box.getMinX(); x <= box.getMaxX(); x++)
			{
				for(int y = (int)box.getMinY(); y <= box.getMaxY(); y++)
				{
					if(!box.contains(x, y))
					{
						continue;
					}
					
					RoomTile tmp = map.getTile(x, y);
					if(tmp != null && tmp.topItem instanceof GameTileItem)
					{
						GameTileItem t = (GameTileItem)tmp.topItem;
						if(t.getLevel() < GameTileItem.LEVEL_LOCKED)
						{
							System.out.println(t.getTile());
						}
					}
				}
			}
		}
	}
	
	private static void discoverLockedNeighbours(RoomMap map, GameTileItem src, ETeam teamId, Polygon shape)
	{		
		System.out.println("Finding neighbours of " + src.x + "," + src.y);
		
		// Add this tile to the shape
		shape.addPoint(src.x, src.y);
		
		// Search recursively in all directions
		for(Direction dir : Direction.values())
		{
			// Game tile here?
			RoomTile tmp = map.getTile(src.x + dir.modX, src.y + dir.modY);
			if(tmp != null && tmp.topItem instanceof GameTileItem)
			{
				// Tile locked in same color?
				GameTileItem tile = (GameTileItem)tmp.topItem;
				if(tile.getTeamId() == teamId && tile.getLevel() == GameTileItem.LEVEL_LOCKED)
				{
					// Already done?
					if(!shape.contains(tile.x, tile.y))
					{
						BanzaiSolver.discoverLockedNeighbours(map, tile, teamId, shape);
					}
				}
			}
		}
	}
	
	private static void discover(RoomMap map, GameTileItem src, ETeam teamId, Polygon shape, List<Point> done)
	{		
		System.out.println("Finding neighbours of " + src.x + "," + src.y);
		
		// Add this tile to the shape
		shape.addPoint(src.x, src.y);
		done.add(new Point(src.x, src.y));
		
		// Search recursively in all directions
		for(Direction dir : Direction.values())
		{
			// Game tile here?
			RoomTile tmp = map.getTile(src.x + dir.modX, src.y + dir.modY);
			if(tmp != null && tmp.topItem instanceof GameTileItem)
			{
				// Tile locked in same color?
				GameTileItem tile = (GameTileItem)tmp.topItem;
				if(tile.getTeamId() == teamId && tile.getLevel() == GameTileItem.LEVEL_LOCKED)
				{
					// Already done?
					if(!done.contains(new Point(tile.x, tile.y)))
					{
						BanzaiSolver.discover(map, tile, teamId, shape, done);
					}
				}
			}
		}
	}
	
	public static List<GameTileItem> solveShapesGetExtraTiles(GameTileItem tile, RoomMap map, ETeam teamId)
	{
		// Get all linked locked & same colored tiles from here (edges)
		List<GameTileItem> locked = new ArrayList<GameTileItem>();
		discoverLockedTiles(tile, map, teamId, locked);
		
		// Now bake a polygon
		Polygon pg = new Polygon();
		for(GameTileItem t : locked)
		{
			pg.addPoint(t.x, t.y);
		}
		
		// Walk over the edges of the polygon
		/*
		PathIterator it = pg.getPathIterator(null);
		while(!it.isDone())
		{
			float[] coords = new float[2];
			it.currentSegment(coords);
			it.next();
			System.out.println(Arrays.toString(coords));
		}
		System.out.println();*/
		
		List<GameTileItem> extra = new ArrayList<GameTileItem>();
		for(int x = 0; x <= map.getModel().getMaxX(); x++)
		{
			for(int y = 0; y <= map.getModel().getMaxY(); y++)
			{
				if(pg.contains(x, y))
				{
					RoomTile tmp = map.getTile(x, y);
					if(tmp.topItem instanceof GameTileItem)
					{
						GameTileItem t = (GameTileItem)tmp.topItem;
						if(!t.isLocked())
						{
							boolean enclosedAllDirs = true;
							for(Direction dir : Direction.values())
							{
								boolean enclosed = false;
								RoomTile tmp2 = tmp;
								while(!enclosed)
								{
									tmp2 = map.getTile(tmp2.x + dir.modX, tmp2.y + dir.modY);
									if(tmp2 == null)
									{
										break;
									}
									else
									{
										if(tmp2.topItem instanceof GameTileItem)
										{
											GameTileItem t2 = (GameTileItem) tmp2.topItem;
											if(t2.isLocked() && t2.getTeamId() == teamId)
											{
												enclosed = true;
												break;
											}
										}
									}
								}
								if(!enclosed)
								{
									enclosedAllDirs = false;
									break;
								}
							}
							
							if(enclosedAllDirs)
							{
								extra.add(t);
								System.out.println(tmp.toString());
							}
						}
					}
				}
			}
		}
		
		
		return extra;
	}
	
	private static void discoverLockedTiles(GameTileItem src, RoomMap map, ETeam teamId, List<GameTileItem> locked)
	{
		locked.add(src);
		for(Direction dir : Direction.values())
		{
			RoomTile tmp = map.getTile(src.x + dir.modX, src.y + dir.modY);
			if(tmp != null && tmp.topItem instanceof GameTileItem)
			{
				GameTileItem tile = (GameTileItem)tmp.topItem;
				if(tile.isLocked() && tile.getTeamId() == teamId && !locked.contains(tile))
				{
					discoverLockedTiles(tile, map, teamId, locked);
				}
			}
		}
	}
}
