package habbod2.game.users.achievements;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import habbod2.game.GameComponent;
import habbod2.game.GameManager;
import habbod2.game.sessions.Session;
import habbod2.game.users.User;
import habbod2.storage.DatabaseResult;

public class AchievementManager extends GameComponent
{
	private Map<Integer, Achievement> achievements;

	public AchievementManager(GameManager gameMgr)
	{
		super(gameMgr);
		this.achievements = Collections.emptyMap();
	}

	public void reload()
	{
		// Create new collection
		Map<Integer, Achievement> tmpAchievements = new HashMap<Integer, Achievement>();

		// Load from database
		DatabaseResult result = this.getDb().query("SELECT * FROM achievements");
		while(result.row())
		{
			Achievement ach = new Achievement();
			ach.id = result.getInt("id");
			ach.maxLevel = result.getInt("max_level");
			ach.badge = result.getString("badge");
			ach.badgeDynamicLevel = result.getBoolean("badge_dynamiclevel");
			ach.pixelsBase = result.getInt("pixels_base");
			ach.pixelsMultiplier = result.getFloat("pixels_multiplier");
			tmpAchievements.put(ach.id, ach);
		}
		result.release();

		// Set instance field
		this.achievements = Collections.unmodifiableMap(tmpAchievements);

		// Done!
		this.getLogger().info(this, String.format("Reloaded %d achievements.", this.achievements.size()));
	}

	public Achievement getAchievement(int achievementId)
	{
		return this.achievements.get(achievementId);
	}

	public Collection<Achievement> getAllAchievements()
	{
		return this.achievements.values();
	}

	public Map<Integer, Integer> getAchievementsForUser(int userId)
	{
		Map<Integer, Integer> achs = new HashMap<Integer, Integer>();

		DatabaseResult result = this.getDb().query("SELECT achievement_id,level FROM user_achievements WHERE user_id=" + userId);
		while(result.row())
		{
			achs.put(result.getInt("achievement_id"), result.getInt("level"));
		}
		result.release();

		return achs;
	}

	public void unlockAchievementForUser(int userId, int achievementId)
	{
		this.unlockAchievementForUser(userId, achievementId, Achievement.START_LEVEL);
	}

	public void unlockAchievementForUser(int userId, int achievementId, int level)
	{
		// User and achievement exist?
		User user = this.getGameMgr().getUserMgr().getUserById(userId);
		Achievement ach = this.getAchievement(achievementId);
		if(user == null || ach == null)
		{
			return;
		}

		// Valid level?
		if(level > ach.maxLevel)
		{
			level = ach.maxLevel;
		}

		// Calculate pixel reward
		int pixels = ach.getPixels(level);

		// Delete old achievement level (if any)
		this.getDb().execute("DELETE FROM user_achievements WHERE user_id=" + userId + " AND achievement_id=" + achievementId);

		// New achievement level
		this.getDb().execute("INSERT INTO user_achievements(user_id,achievement_id,level) VALUES (" + userId + "," + achievementId + "," + level + ")");

		// Give pixels
		user.pixels += pixels;
		user.saveCurrencies();

		// User online?
		Session session = this.getGameMgr().getSessionMgr().getSessionByUserId(userId);
		if(session != null)
		{
			// Update achievement list
			session.getUser().achievements.put(achievementId, level);

			// Notify of achievement gain
			/*
			ServerMessage msg = new ServerMessage(EServerMessage.AchievementNotification);
			msg.appendInt(achievementId);
			msg.appendInt(level);
			msg.appendStr(ach.getBadge(level));
			if(level == Achievement.START_LEVEL)
			{
				// No previous badge
				msg.appendStr("");
			}
			else
			{
				// Previous badge
				msg.appendStr(ach.getBadge(level - 1));
			}
			session.send(msg);*/

			// Notify of pixel gain
			session.sendPixelsAddedNotif(pixels);
		}
	}
}
