package habbod2.game.users.updaters;

import java.util.Calendar;

import habbod2.game.GameManager;
import habbod2.game.sessions.Session;
import habbod2.game.users.User;
import habbod2.util.PropertiesBox;
import habbod2.communication.messages.outgoing.handshake.UserObjectMessageComposer;

public class AutoRespectUpdater extends GameManager.Component implements Runnable
{
	public AutoRespectUpdater(GameManager gameMgr)
	{
		super(gameMgr);
	}

	public void scheduleDailyAtMidnight()
	{
		// First time: today, 0:00 (if past today, it will run immediately)
		Calendar firstTime = Calendar.getInstance();
		firstTime.setTimeInMillis(System.currentTimeMillis());
		//firstTime.set(Calendar.DAY_OF_YEAR, firstTime.get(Calendar.DAY_OF_YEAR) + 1);
		firstTime.set(Calendar.HOUR_OF_DAY, 0);
		firstTime.set(Calendar.MINUTE, 0);
		firstTime.set(Calendar.SECOND, 0);
		firstTime.set(Calendar.MILLISECOND, 0);
		this.getSystem().scheduleTaskWithFirstTime(this, 60 * 60 * 24 * 1000, firstTime.getTimeInMillis());
	}
	
	@Override
	public void run()
	{
		// Get configuration
		PropertiesBox config = this.getConfig();
		int userRespectPoints = config.getInt("game.users.autorespect.users.amount");
		int petRespectPoints = config.getInt("game.users.autorespect.pets.amount");
		//long lastUpdateTime = GameUtil.parseDateTime(config.get("game.users.autorespect.lastupdate"));
		
		// TODO: actually run now?
		
		// Update it in the database
		this.getDb().execute("UPDATE users SET respect_points=" + userRespectPoints + ",petrespect_points=" + petRespectPoints);
		
		// Update + refresh online users
		for(Session session : this.getGameMgr().getSessionMgr().getAllSessions())
		{
			User user = session.getUser();
			user.respectPoints = userRespectPoints;
			user.petRespectPoints = petRespectPoints;
			session.send(UserObjectMessageComposer.compose(user));
		}
		
		// Done!
		this.getLogger().info("AutoRespectUpdater", String.format("Updated respect points for all users. (users=%d, pets: %d)", userRespectPoints, petRespectPoints));
	}
}
