package habbod2.game.users.updaters;

import habbod2.communication.messages.outgoing.handshake.UserObjectMessageComposer;
import habbod2.game.GameManager;
import habbod2.game.sessions.Session;
import habbod2.game.users.User;

import java.util.Calendar;
import java.util.TimerTask;

public class AutoRespectUpdater extends TimerTask
{
	private GameManager gameMgr;
	
	public AutoRespectUpdater(GameManager gameMgr)
	{
		this.gameMgr = gameMgr;
	}

	public void scheduleDailyAtMidnight()
	{
		Calendar moment = Calendar.getInstance();
		moment.set(Calendar.HOUR_OF_DAY, 0);
		moment.set(Calendar.MINUTE, 0);
		moment.set(Calendar.SECOND, 0);
		moment.set(Calendar.MILLISECOND, 0);
		this.gameMgr.getSystem().scheduleTask(this, moment.getTime(), 60 * 60 * 24 * 1000);
	}
	
	@Override
	public void run()
	{
		// Get configuration
		int userRespectPoints = this.gameMgr.getProperties().getInt("game.users.autorespect.users.amount");
		int petRespectPoints = this.gameMgr.getProperties().getInt("game.users.autorespect.pets.amount");
		
		// Update it in the database
		this.gameMgr.getDb().execute("UPDATE users SET respect_points=" + userRespectPoints + ",petrespect_points=" + petRespectPoints);
		
		// Update + refresh online users
		for(Session session : this.gameMgr.getSessionMgr().getAllSessions())
		{
			User user = session.getUser();
			user.respectPoints = userRespectPoints;
			user.petRespectPoints = petRespectPoints;
			session.send(UserObjectMessageComposer.compose(user));
		}
		
		// Done!
		this.gameMgr.getLogger().info("AutoRespectUpdater", String.format("Updated respect points for all users. (users=%d, pets: %d)", userRespectPoints, petRespectPoints));
	}
}
