package habbod2.game.rooms.items.banzai;

import habbod2.game.rooms.instance.RoomAvatar;
import habbod2.game.rooms.instance.RoomUser;
import habbod2.game.rooms.items.RoomFloorItem;
import habbod2.game.rooms.minigames.BanzaiGame;
import habbod2.game.rooms.minigames.GameTeam.ETeam;
import habbod2.game.rooms.minigames.GameUserPlayer;
import habbod2.game.util.GameUtil;

public class TileItem extends RoomFloorItem
{
	public static final int LEVEL_EMPTY = 0;
	public static final int LEVEL_MEDIUM = 1;
	public static final int LEVEL_SEALED = 2;
	
	private ETeam teamId;
	private int level;
	private int pulses;
	private boolean doDisable;
	
	// 3-4-5 = red
	// 6-7-8 = green
	// 9-10-11 = blue
	// 12-13-14 = yellow
	
	// level: 0-1-2 (2 = locked)
	public void onAvatarStepOn(RoomAvatar avatar)
	{
		// Tile sealed? Save further checking
		if(this.level == LEVEL_SEALED)
		{
			return;
		}
		
		// RoomUser?
		if(!(avatar instanceof RoomUser))
		{
			return;
		}
		
		// Playing Banzai?
		GameUserPlayer player = ((RoomUser)avatar).getPlayer();
		if(player == null || !player.getGame().isRunning() || !(player.getGame() instanceof BanzaiGame))
		{
			return;
		}
		
		// Increment or decrement?
		ETeam teamId = player.getTeam().getId();
		int level = (this.teamId == teamId) ? this.level + 1 : LEVEL_EMPTY;
		this.setState(true, teamId, level);
		
		// Scored a point!
		player.addScore(1);
		
		// Tile is now sealed?
		if(this.level == LEVEL_SEALED)
		{
			player.addStats(1);
			((BanzaiGame)player.getGame()).sealedTile();
		}
	}
	
	public void setState(boolean isEnabled, ETeam teamId, int level)
	{
		this.teamId = teamId;
		this.level = level;
		if(this.teamId == null)
		{
			this.setState(isEnabled ? 1 : 0);
		}
		else
		{
			this.setState(((teamId.ordinal() + 1) * 3) + level);
		}
	}
	
	public void flashFor(int seconds)
	{
		this.pulses = GameUtil.secondsToRoomPulses(seconds);
		this.setTimeout(1);
	}
	
	public void disableAfter(int seconds)
	{
		this.doDisable = true;
		this.setTimeout(GameUtil.secondsToRoomPulses(seconds));
	}
	
	public void onTimeout()
	{
		// Flash!
		this.pulses--;
		if(this.doDisable || this.pulses == 0 || this.pulses % 2 == 0)
		{
			if(this.doDisable)
			{
				this.setState(0);
				this.doDisable = false;
			}
			else
			{
				this.setState(1);
			}
		}
		else
		{
			this.setState(false, this.teamId, LEVEL_SEALED);
		}
		
		// Do it again?
		if(this.pulses > 0)
		{
			this.setTimeout(1);
		}
		else
		{
			this.setState(0);
		}
	}
	
	public ETeam getTeamId()
	{
		return this.teamId;
	}
}
