package habbod2.game.rooms.items.floor;

import static habbod2.core.Locale._I;

import habbod2.System;
import habbod2.game.rooms.geometry.Direction;
import habbod2.game.rooms.instance.RoomTile;
import habbod2.game.rooms.instance.RoomUser;
import habbod2.game.rooms.items.RoomItem;
import habbod2.game.util.GameUtil;

public class TeleporterItem extends UsableFloorItem
{
	private boolean isWaiting = false;
	
	@Override
	protected boolean onToggle(RoomUser user, int instruction)
	{
		// Can go in teleporters?
		if(user == null || !user.canMove())
		{
			return false;
		}

		// Infront of teleporter? Attempt to move to the front
		RoomTile frontTile = this.getTileInFront();
		if(!user.at(frontTile))
		{
			user.setGoal(frontTile);
			return false;
		}
		
		// In use?
		if(this.inUse())
		{
			return false;
		}
		
		// Tadaa
		this.setUser(user);

		// Lock own movement, move into the teleporter
		user.lockMovement();
		user.overrideNextTile = true;
		user.setGoal(this.getTile());

		// Wait...
		this.setTimeout(3);
		
		return true;
	}

	@Override
	protected void onTimeout()
	{
		// User inside?
		if(this.inUse())
		{
			// Get user
			RoomUser user = this.getUser();
			
			// Get out?
			if(user.getSession().authenticatedTeleporterId == this.id)
			{
				user.getSession().authenticatedTeleporterId = 0;
				this.kickUserOut();
				return;
			}
			
			// Get tele info
			int otherId = GameUtil.parseInt(this.data);
			int otherRoomId = System.get().getGameMgr().getItemMgr().getItemRoomId(otherId);
			
			// Waiting for teleport start?
			if(this.isWaiting)
			{
				// Same room?
				if(otherRoomId == this.getRoom().getInfo().id)
				{
					RoomItem item = this.getRoom().getItem(otherId);
					if(item != null && item instanceof TeleporterItem)
					{
						// Update user direction + position
						TeleporterItem tele = (TeleporterItem)item;
						user.setDirection(Direction.get(tele.rotation));
						user.setTile(tele.getTile(), false);
						
						// Open the door in the other tele
						tele.openDoorTemporarily();
						
						// Get out!
						user.unlockMovement();
						user.setGoal(tele.getTileInFront());
					}
				}
				else
				{
					// Teleporting!
					user.getSession().authenticatedTeleporterId = otherId;
					user.getSession().tryGetRoomAccess(otherRoomId, "");
				}
				
				// Waiting done! Done with this teleporter
				this.isWaiting = false;
				this.clearUser();
			}
			
			// User just entered teleporter?
			else if(user.at(this.getTile()))
			{
				// Linked teleporter in room?
				if(otherRoomId == 0)
				{
					user.getSession().sendNotif(_I("This teleporter is currently not linked."));
					this.kickUserOut();
				}
				else
				{
					// Flash this teleporter, wait 2 pulses before the actual teleport
					this.isWaiting = true;
					this.setTempState(2);
					this.setTimeout(3);
				}
			}
		}
		else
		{
			this.setState(0);
		}
	}

	private void openDoorTemporarily()
	{
		this.setTempState(1);
		this.setTimeout(2);
	}

	private void kickUserOut()
	{
		RoomUser user = this.getUser();
		if(user != null)
		{
			// Open the door...
			this.clearUser();
			this.openDoorTemporarily();

			// Start move to tile infront
			user.unlockMovement();
			user.setGoal(this.getTileInFront());
		}
	}
	
	public void arriveUser(RoomUser user)
	{
		this.setUser(user);
		this.setTimeout(2);
		user.setDirection(Direction.get(this.rotation));
	}
}
