package habbod2.game.rooms.items.minigame.freeze;

import habbod2.game.rooms.instance.RoomInstance;
import habbod2.game.rooms.instance.RoomTile;
import habbod2.game.rooms.instance.RoomUser;
import habbod2.game.rooms.items.RoomFloorItem;
import habbod2.game.rooms.minigames.GameUserPlayer;
import habbod2.game.rooms.minigames.game.FreezeGame;
import habbod2.game.rooms.minigames.game.FreezeGame.FreezeValues;
import habbod2.game.util.GameUtil;

public class PowerUpBoxItem extends RoomFloorItem
{
	public enum EPowerUp
	{
		None,
		PerpendicularExplosion,
		MultiBalls,
		DiagonalExplosion,
		MegaExplosion,
		Life,
		Shield
	}
	
	private boolean isCrushed;
	private EPowerUp powerUp;
	
	// 1 = nothing
	// 2 = blue move arrows
	// 3 = 2 snowballs
	// 4 = yellow move arrows
	// 5 = red ball?
	// 6 = +1 life
	// 7 = shield (protects against 1 hit)
	
	@Override
	public void onRoomAdd(RoomInstance room)
	{
		this.reset();
	}
	
	@Override
	public void onGenerateMapTile(RoomTile tile)
	{
		if(tile != null)
		{
			tile.state = this.isCrushed ? RoomTile.OPEN : RoomTile.CLOSED;
		}
	}
	
	@Override
	public boolean onToggle(RoomUser user, int operation)
	{
		this.crush(EPowerUp.Life);
		
		// Playing?
		GameUserPlayer player = user.getPlayer();
		if(player == null || !(player.getGame() instanceof FreezeGame) || !player.getGame().isRunning())
		{
			return false;
		}
		
		// Get player values
		FreezeValues values = (FreezeValues)player.getValues();
		
		// TODO: give random powerup
		this.crush(GameUtil.getRandomElement(EPowerUp.values()));
		user.getSession().sendWhisperNotif(this.powerUp.toString());
		
		return true;
	}
	
	public void reset()
	{
		this.update(false, EPowerUp.None);
	}
	
	public void crush(EPowerUp powerUp)
	{
		this.update(true, powerUp);
	}
	
	private void update(boolean isCrushed, EPowerUp powerUp)
	{
		this.isCrushed = isCrushed;
		this.powerUp = powerUp;
		this.setState(isCrushed ? (powerUp.ordinal() + 1) * 1000 : 0);
		this.onGenerateMapTile(this.getTile());
	}
}
