package habbod2.game.rooms.items.wired;

import java.util.ArrayList;
import java.util.List;

import org.json.JSONArray;
import org.json.JSONObject;

import habbod2.net.protocol.ServerMessage;
import habbod2.game.rooms.instances.RoomInstance;
import habbod2.game.rooms.instances.RoomUser;
import habbod2.game.rooms.items.RoomFloorItem;

public abstract class WiredItem extends RoomFloorItem
{
	public static final int SELECT_LIMIT = 5;
	
	private int selectionType;
	private List<Integer> selectedIds;
	private String text;
	private int[] params;
	
	private boolean flashToggle;
	
	@Override
	public final void onRoomAdd(RoomInstance room)
	{
		this.flashToggle = false;
		this.parseData();
	}
	
	@Override
	public void onRoomRemove(RoomInstance room)
	{
		this.data = "{}";
		this.selectionType = 0;
		this.clearSelectedItems();
		this.selectedIds = null;
		this.text = null;
		this.params = null;
	}
	
	private void parseData()
	{
		this.selectedIds = new ArrayList<Integer>(0);
		
		// Any available data?
		if(this.data == null || this.data.isEmpty())
		{
			this.data = "{}";
		}
		
		// Try parsing the data
		try
		{	
			JSONObject data = new JSONObject(this.data);
			this.onUnwrap(data);
			return;
		}
		catch (Exception e)
		{
			// Meh!
			e.printStackTrace();
		}
	}
	
	public void saveSettings()
	{
		try
		{
			JSONObject data = new JSONObject();
			this.onWrap(data);
			this.data = data.toString();
		}
		catch(Exception ex)
		{
			
		}
		this.requireDbUpdate();
	}
	
	protected void onWrap(JSONObject data) throws Exception
	{
		if(this.selectionType != 0) data.put("sel", this.selectionType);
		if(this.selectedIds.size() > 0) data.put("ids", this.selectedIds);
		if(this.text.length() > 0) data.put("txt", this.text);
		if(this.params.length > 0) data.put("prm", this.params);
	}
	
	protected void onUnwrap(JSONObject data) throws Exception
	{
		this.selectionType = data.optInt("sel");
		
		// Parse selected IDs
		JSONArray array = data.optJSONArray("ids");
		if(array != null)
		{
			for(int x = 0; x < array.length(); x++)
			{
				this.selectItem(array.getInt(x));
			}
		}
		this.text = (String) data.optString("txt");
		
		// Parse params
		this.params = new int[this.getAmountOfParams()];
		array = data.optJSONArray("prm");
		if(array != null)
		{
			for(int x = 0; x < array.length(); x++)
			{
				this.setParam(x, array.getInt(x));
			}
		}
	}
	
	public abstract int getInterface();
	
	protected abstract int getAmountOfParams();
	
	public final int[] getParams()
	{
		return this.params;
	}
	
	public final int getParam(int param)
	{
		if(param >= 0 && param < this.params.length)
		{
			return this.params[param];
		}
		else
		{
			return 0;
		}
	}
	
	public final void setParam(int param, int value)
	{
		if(param >= 0 && param < this.params.length)
		{
			this.params[param] = value;
		}
	}
	
	public final String getText()
	{
		return this.text;
	}
	
	public final void setText(String text)
	{
		this.text = text;
	}
	
	public final int getSelectionType()
	{
		return this.selectionType;
	}
	
	public final void setSelectionType(int selectionType)
	{
		this.selectionType = selectionType;
	}
	
	public final void clearSelectedItems()
	{
		if(this instanceof WiredTrigger)
		{
			for(RoomFloorItem item : this.getSelectedItems())
			{
				item.removeTrigger((WiredTrigger)this);
			}
		}
		this.selectedIds.clear();
	}
	
	public final void selectItem(int itemId)
	{
		if(this.selectedIds.size() < WiredItem.SELECT_LIMIT)
		{
			this.selectedIds.add(itemId);
			if(this instanceof WiredTrigger)
			{
				RoomFloorItem item = this.getRoom().getFloorItem(itemId);
				if(item != null)
				{
					item.addTrigger((WiredTrigger)this);
				}
			}
		}
	}
	
	public final List<RoomFloorItem> getSelectedItems()
	{
		List<RoomFloorItem> items = new ArrayList<RoomFloorItem>(1);
		for(int itemId : this.selectedIds)
		{
			RoomFloorItem item = this.getRoom().getFloorItem(itemId);
			if(item != null)
			{
				items.add(item);
			}
		}
		return items;
	}
	
	public final List<RoomFloorItem> getStackedItems()
	{
		return this.getTile().getItems();
	}
	
	@Override
	protected
	final boolean onToggle(RoomUser user, int instruction)
	{
		if(user != null && user.hasRoomOwner)
		{
			user.getSession().send(this.getDialog());
		}
		
		return false;
	}
	
	public final void flash()
	{
		this.setTempState(this.flashToggle ? "0" : "1");
		this.flashToggle = !this.flashToggle;
	}
	
	protected abstract ServerMessage getDialog();
}
