package habbod2.game.rooms.items.wired.actions;

import java.util.List;

import habbod2.game.rooms.geometry.RoomTile;
import habbod2.game.rooms.instance.RoomUser;
import habbod2.game.rooms.items.RoomFloorItem;
import habbod2.game.rooms.items.wired.WiredAction;
import habbod2.communication.messages.outgoing.room.engine.SlideObjectBundleMessageComposer;

public class RestoreItemToSnapshotAction extends WiredAction
{

	@Override
	public int getInterface()
	{
		return 3;
	}
	
	@Override
	protected int getAmountOfParams()
	{
		return 3;
	}
	
	@Override
	public boolean requiresUser()
	{
		return false;
	}
	
	@Override
	public void execute(RoomUser user)
	{
		// Snapshots loaded?
		if(this.snapshots == null)
		{
			return;
		}
		
		// Restore all selected items to their saved snapshots
		List<RoomFloorItem> selected = this.getSelectedItems();
		for(ItemSnapshot snapshot : this.snapshots)
		{
			RoomFloorItem item = (RoomFloorItem)this.getRoom().getItem(snapshot.itemId);
			if(selected.contains(item))
			{
				// Backup old values
				RoomTile from = item.getTile();
				float fromZ = item.z;
				
				// Placeholders for new values
				byte newRotation = item.rotation;
				int newX = from.x;
				int newY = from.y;
				
				// Restore what needs restoring
				if(this.getParam(PARAM_STATE) == 1)
				{
					item.setState(snapshot.state);
				}
				if(this.getParam(PARAM_ROTATION) == 1)
				{
					newRotation = snapshot.rotation;
				}
				if(this.getParam(PARAM_POSITION) == 1)
				{
					newX = snapshot.x;
					newY = snapshot.y;
				}
				
				// Move item
				if(this.getRoom().moveItem(item.id, newX, newY, newRotation, true))
				{
					this.getRoom().broadcast(SlideObjectBundleMessageComposer.compose(item, from, fromZ, null));
				}
			}
		}
	}
}