package habbod2.communication.messages.incoming.catalog;

import habbod2.Instance;
import habbod2.net.protocol.ClientMessage;
import habbod2.communication.RequestHandler;
import habbod2.communication.messages.outgoing.catalog.GiftReceiverNotFoundMessageComposer;
import habbod2.communication.messages.outgoing.catalog.PurchaseOKMessageComposer;
import habbod2.game.GameManager;
import habbod2.game.catalog.CatalogProduct;
import habbod2.game.sessions.Session;
import habbod2.game.util.WordFilter;

public class PurchaseFromCatalogAsGiftHandler implements RequestHandler
{
	@Override
	@SuppressWarnings("unused")
	public void handle(Session session, ClientMessage request)
	{
		// Parse purchase info
		int pageId = request.readInt();
		int productId = request.readInt();
		String purchaseData = request.readStr(true);
		String giftUsername = request.readStr();
		String giftNote = WordFilter.getInstance().filter(request.readStr(true));

		// Get game mgr
		GameManager game = Instance.get().getGameMgr();
				
		// Valid product?
		CatalogProduct product = game.getCatalog().getProduct(productId);
		if(product == null || !product.isGiftable(session.getUser()))
		{
			Instance.get().getLogger().warning(this, String.format("User #%d cannot buy #%d from the Catalog. (bad permissions etc)", session.getUser().id, productId));
			return;
		}
		
		// Receiver exists?
		int receiverId = game.getUserMgr().getUserId(giftUsername);
		if(receiverId == 0)
		{
			session.send(GiftReceiverNotFoundMessageComposer.compose(giftUsername));
			return;
		}

		// Issue purchase
		if(!game.getCatalog().purchaseAsGift(product, session.getUser(), purchaseData, receiverId, giftNote))
		{
			session.sendNotif(String.format("Purchase of catalog product #%d failed, you will not be billed. Please try again in a few minutes. If this keeps happening, please contact Support.", product.id));
			return;
		}
		
		// Bill the purchasing user for gifted product
		game.getCatalog().billUser(session, product);
		
		// Purchase okay!
		session.send(PurchaseOKMessageComposer.compose(product));
	}
}
