package habbod2.communication.messages.incoming.room.chat;

import habbod2.Instance;
import habbod2.net.protocol.ClientMessage;
import habbod2.communication.RequestHandler;
import habbod2.communication.messages.outgoing.room.chat.WhisperMessageComposer;
import habbod2.net.protocol.ServerMessage;
import habbod2.game.rooms.instance.RoomUser;
import habbod2.game.util.WordFilter;
import habbod2.game.sessions.Session;

public class WhisperHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request)
	{
		// Parse receiver + text
		String[] data = request.readStr().split(" ", 2);
		if(data.length != 2)
		{
			return;
		}
		String receiverName = data[0];
		String unfiltered = data[1];

		// Actually in room?
		if(session.inRoom())
		{
			// Receiver in room too?
			RoomUser user = session.getRoomUser();
			RoomUser receiver = session.getRoom().getUserByUsername(receiverName);
			if(receiver != null)
			{
				// Apply wordfilter
				String message = session.getUser().hasRight("override_wordfilter") ? unfiltered : WordFilter.getInstance().filter(unfiltered);
				
				// Send to this user & receiver
				ServerMessage msg = WhisperMessageComposer.compose(user, message);
				user.getSession().send(msg);
				receiver.getSession().send(msg);
				
				// Log unfiltered message
				Instance.get().getGameMgr().getChatlogSvc().log(session.getUser().id, session.getRoom().getInfo().id, unfiltered, receiver.getInfo().id);
				
				// Eavesdropping!
				for(RoomUser tmp : session.getRoom().getAllUsers())
				{
					// Exclude the sender/receiver
					if(tmp != user && tmp != receiver)
					{
						// Can see whispers?
						if(tmp.getInfo().hasRight("see_whispers"))
						{
							// Can only see whispers of lower role
							if(tmp.getInfo().role.level > user.getInfo().role.level)
							{
								tmp.getSession().send(WhisperMessageComposer.compose(user, String.format("[>> %s] %s", receiver.getInfo().username, unfiltered)));
							}
						}
					}
				}
			}
		}
	}
}
