package habbod2.communication.messages.incoming.userdefinedroomevents;

import habbod2.net.protocol.ClientMessage;
import habbod2.communication.RequestHandler;
import habbod2.game.rooms.items.wired.WiredAction;
import habbod2.game.rooms.items.wired.AbstractWiredItem;
import habbod2.game.rooms.items.wired.conditions.TextValueCondition;
import habbod2.game.util.WordFilter;
import habbod2.game.sessions.Session;

public class UpdateActionHandler implements RequestHandler
{
	@Override
	public void handle(Session session, ClientMessage request)
	{
		// User valid?
		if(!(session.inRoom() && session.getRoomUser().hasRoomOwner))
		{
			return;
		}
		
		// Get the item
		AbstractWiredItem wired = (AbstractWiredItem) session.getRoom().getItem(request.readInt());
		if(wired == null || !(wired instanceof WiredAction || wired instanceof TextValueCondition))
		{
			return;
		}
		
		// Update the parameters
		int parameters = request.readInt();
		for(int param = 0; param < parameters; param++)
		{
			wired.setParam(param, request.readInt());
		}
		
		// Update the text
		wired.setText(WordFilter.getInstance().filter(request.readStr()));
		
		// Update the selected items
		wired.clearSelectedItems();
		int selected = request.readInt();
		for(int x = 0; x < selected; x++)
		{
			wired.selectItem(request.readInt());
		}
		
		// Update delay
		if(wired instanceof WiredAction)
		{
			((WiredAction)wired).setDelay(request.readInt());
		}
		else
		{
			request.readInt();
		}
		
		// Update selection type
		wired.setSelectionType(request.readInt());
		 
		// Save changes!
		wired.saveSettings();
	}
}
