package habbod2.communication.messages.outgoing.room.engine;

import java.util.Collection;

import habbod2.net.protocol.ServerMessage;
import habbod2.communication.messages.EServerMessage;
import habbod2.game.bots.Bot;
import habbod2.game.bots.RoomBot;
import habbod2.game.pets.Pet;
import habbod2.game.pets.RoomPet;
import habbod2.game.rooms.instance.RoomActor;
import habbod2.game.rooms.instance.RoomUser;
import habbod2.game.users.User;

public class UsersMessageComposer
{
	public static ServerMessage compose(RoomActor actor)
	{
		ServerMessage msg = ServerMessage.create(EServerMessage.Users);
		msg.appendInt(1);
		serializeAvatar(actor, msg);
		
		return msg;
	}
	
	public static ServerMessage compose(Collection<RoomActor> actors)
	{
		ServerMessage msg = ServerMessage.create(EServerMessage.Users);
		msg.appendInt(actors.size());
		for(RoomActor actor : actors)
		{
			serializeAvatar(actor, msg);
		}
		
		return msg;
	}

	public static void serializeAvatar(RoomActor actor, ServerMessage msg)
	{
		if(actor instanceof RoomUser)
		{
			serializeUser((RoomUser)actor, msg);
		}
		else if(actor instanceof RoomPet)
		{
			serializePet((RoomPet)actor, msg);
		}
		else if(actor instanceof RoomBot)
		{
			serializeBot((RoomBot)actor, msg);
		}
	}
	
	private static void serializeUser(RoomUser actor, ServerMessage msg)
	{
		User user = actor.getInfo();
		msg.appendInt(user.id);
		msg.appendStr(user.username);
		msg.appendStr(user.motto);
		msg.appendStr(actor.getFigure());
		msg.appendInt(actor.actorId);
		msg.appendInt(actor.x);
		msg.appendInt(actor.y);
		msg.appendStr(actor.z + actor.getExtraDisplayHeight());
		msg.appendInt(2);
		msg.appendInt(1);
		msg.appendStr(user.gender);
		msg.appendInt(-1);
		msg.appendInt(-1); // Group ID
		msg.appendInt(-1);
		msg.appendStr("");
		msg.appendInt(user.respect); // Achievement score
	}
	
	private static void serializePet(RoomPet actor, ServerMessage msg)
	{
		Pet pet = actor.getInfo();
		msg.appendInt(pet.id);
		msg.appendStr(pet.name);
		msg.appendStr("");
		msg.appendStr(pet.getFigure());
		msg.appendInt(actor.actorId);
		msg.appendInt(actor.x);
		msg.appendInt(actor.y);
		msg.appendStr(actor.z + actor.getExtraDisplayHeight());
		msg.appendInt(4);
		msg.appendInt(2);
		msg.appendInt(0);
	}
	
	private static void serializeBot(RoomBot actor, ServerMessage msg)
	{
		Bot bot = actor.getInfo();
		msg.appendInt(bot.id);
        msg.appendStr(bot.name);
        msg.appendStr(bot.motto);
        msg.appendStr(bot.figure);
        msg.appendInt(actor.actorId);
        msg.appendInt(actor.x);
        msg.appendInt(actor.y);
        msg.appendStr(actor.z + actor.getExtraDisplayHeight());
        msg.appendInt(4);
        msg.appendInt(3);
	}
}
