package habbod2.util;

import habbod2.Instance;
import org.keplerproject.luajava.LuaState;
import org.keplerproject.luajava.LuaStateFactory;

public class ScriptHandler
{
	public static final boolean hasNativeLibrary = java.lang.System.getProperty("java.library.path").contains("luajava");
	
	private LuaState lua;
	private String name;
	
	private ScriptHandler(String name, String file)
	{
		this.lua = LuaStateFactory.newLuaState();
		this.lua.openLibs();
		this.lua.LdoFile(file);
		this.name = name;
	}
	
	public void destroy()
	{
		this.lua.close();
	}
	
	public synchronized void call(String func, Object... args)
	{
		try
		{
			this.lua.getGlobal(func);
			for(Object arg : args)
			{
				this.lua.pushObjectValue(arg);
			}
			this.lua.call(args.length, 0);
		}
		catch(Exception ex)
		{
			Instance.get().getLogger().error("ScriptHandler", String.format("%s: error calling %s()!", this.name, func), ex);
		}
	}
	
	public static ScriptHandler load(String scriptName)
	{
		if(ScriptHandler.hasNativeLibrary)
		{
			return new ScriptHandler(scriptName, "scripts/" + scriptName.replace('.', '/') + ".lua");
		}
		else
		{
			Instance.get().getLogger().warning("ScriptHandler", "No Lua support available, install luajava + LuaBinaries to use ScriptHandler.");
			return null;
		}
	}
	
	public String getName()
	{
		return this.name;
	}
}
