package habbod2.game.help;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import habbod2.game.GameManager;
import habbod2.storage.DatabaseResult;

public class HelpTool extends GameManager.Component
{
	private Map<Integer, HelpTopic> topics;
	private Map<Integer, HelpCategory> categories;

	private List<HelpTopic> importantTopics;
	private List<HelpTopic> knownIssues;

	public HelpTool(GameManager gameMgr)
	{
		super(gameMgr);
		this.topics = Collections.emptyMap();
		this.categories = Collections.emptyMap();
		this.importantTopics = Collections.emptyList();
		this.knownIssues = Collections.emptyList();
	}

	public void reload()
	{
		// Create new collections
		Map<Integer, HelpTopic> tmpTopics = new HashMap<Integer, HelpTopic>();
		Map<Integer, HelpCategory> tmpCategories = new HashMap<Integer, HelpCategory>();
		List<HelpTopic> tmpImportantTopics = new ArrayList<HelpTopic>();
		List<HelpTopic> tmpKnownIssues = new ArrayList<HelpTopic>();

		// Fetch all categories
		try(DatabaseResult result = this.getDb().query("SELECT * FROM help_categories"))
		{
			// Load categories
			while(result.row())
			{
				HelpCategory category = new HelpCategory();
				category.id = result.getInt("id");
				category.name = result.getString("name");
				category.topics = new ArrayList<HelpTopic>();
				tmpCategories.put(category.id, category);
			}
	
			// Load topics
			result.query("SELECT * FROM help_topics");
			while(result.row())
			{
				HelpTopic topic = new HelpTopic();
				topic.id = result.getInt("id");
				topic.categoryId = result.getInt("category_id");
				topic.title = result.getString("title");
				topic.text = result.getString("text");
	
				// Add to topics
				tmpTopics.put(topic.id, topic);
				
				// Add to category
				HelpCategory parent = tmpCategories.get(topic.categoryId);
				if(parent != null)
				{
					parent.topics.add(topic);
				}
	
				// Add to special lists
				String type = result.getString("type");
				if(type.equals("important"))
				{
					tmpImportantTopics.add(topic);
				}
				else if(type.equals("knownissue"))
				{
					tmpKnownIssues.add(topic);
				}
			}
		}

		// Set instance fields
		this.topics = Collections.unmodifiableMap(tmpTopics);
		this.categories = Collections.unmodifiableMap(tmpCategories);
		this.importantTopics = Collections.unmodifiableList(tmpImportantTopics);
		this.knownIssues = Collections.unmodifiableList(tmpKnownIssues);
	}

	public HelpTopic getTopic(int topicId)
	{
		return this.topics.get(topicId);
	}
	
	public Collection<HelpTopic> getAllTopics()
	{
		return this.topics.values();
	}

	public HelpCategory getCategory(int categoryId)
	{
		return this.categories.get(categoryId);
	}
	
	public Collection<HelpCategory> getAllCategories()
	{
		return this.categories.values();
	}
	
	public List<HelpTopic> getImportantTopics()
	{
		return this.importantTopics;
	}
	
	public List<HelpTopic> getKnownIssues()
	{
		return this.knownIssues;
	}
}
