package habbod2.game.pets.ai;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import habbod2.game.pets.RoomPet;
import habbod2.game.rooms.items.RoomFloorItem;
import habbod2.game.rooms.items.pets.PetAccessoryItem;

public class AccessoryFinder implements Comparator<PetAccessoryItem>
{
	private RoomPet pet;
	private List<PetAccessoryItem> list;
	
	public AccessoryFinder(RoomPet pet)
	{
		this.pet = pet;
		this.list = new ArrayList<PetAccessoryItem>();
	}
	
	@SuppressWarnings("unchecked")
	public <T extends PetAccessoryItem> T findClosest(Class<? extends PetAccessoryItem> type)
	{
		// Discover compatible items
		List<Integer> compatible = this.pet.getInfo().type.accessories;
		for(RoomFloorItem item : this.pet.getRoom().getFloorItems())
		{
			if(item.is(type))
			{
				PetAccessoryItem acc = (PetAccessoryItem)item;
				if(!acc.isInUse() && acc.isValid() && compatible.contains(acc.definition.id))
				{
					this.list.add(acc);
				}
			}
		}
		
		// Sort on distance (closest-first)
		Collections.sort(this.list, this);
		
		// Determine closest, reachable item
		PetAccessoryItem result = null;
		for(PetAccessoryItem acc : this.list)
		{
			if(acc.getTile().isReachableBy(this.pet))
			{
				result = acc;
				break;
			}
		}
		
		// Cleanup & return result
		this.list.clear();
		return (T) result;
	}
	
	
	private static final int square(int i)
	{
		return i * i;
	}
	
	private final int getDistanceTo(PetAccessoryItem item)
	{
		return (int) Math.sqrt(square(item.x - this.pet.x) + square(item.y - this.pet.y));
	}

	@Override
	public int compare(PetAccessoryItem item1, PetAccessoryItem item2)
	{
		int d1 = this.getDistanceTo(item1);
		int d2 = this.getDistanceTo(item2);
		if(d1 > d2)
		{
			return 1;
		}
		else if(d1 < d2)
		{
			return -1;
		}
		else
		{
			return 0;
		}
	}
}
