package habbod2.game.pets.ai;

import habbod2.game.pets.RoomPet;
import habbod2.game.rooms.items.pets.PetAccessoryItem;
import habbod2.game.util.GameUtil;

public abstract class AccessoryHandler<T extends PetAccessoryItem> extends ActionHandler
{
	protected T item;
	protected int interactTimer;
	
	public AccessoryHandler(RoomPet pet)
	{
		super(pet);
	}
	
	protected abstract Class<? extends PetAccessoryItem> getItemType();
	
	protected abstract String getNoItemFoundEvent();
	
	protected String getIntentionEmotion() { return null; }
	
	protected abstract int getHappinessPenalty();
	
	protected abstract int getHappinessBoost();
	
	protected abstract String getInteractionStatus();
	
	protected abstract String getActivityEvent();
	
	protected abstract void submitResult(boolean success);
	
	protected abstract int getActionDuration();
	
	protected void beginInteract() { }
	
	protected void endInteract() { }
	
	protected void interactPulse() { }
	
	public void pulse()
	{
		// Detect invalidation
		if(this.item != null && this.item.invalidated())
		{
			this.item = null;
		}
		
		// Needs item?
		if(this.item == null)
		{
			// Show intention
			String intention = this.getIntentionEmotion();
			if(intention != null)
			{
				this.pet.setGesture(intention);
			}
			
			// Discover item?
			Class<? extends PetAccessoryItem> discoverType = this.getItemType();
			if(discoverType != null)
			{
				this.item = this.pet.getFinder().findClosest(discoverType);
			}
			
			// No item found?
			if(this.item == null)
			{
				this.submitResult(false);
				this.pet.sayEventMessage(this.getNoItemFoundEvent());
				this.pet.getInfo().modifyHappiness(-this.getHappinessPenalty());
				this.pet.release();
			}
			else
			{
				this.pet.setGoal(this.item.getTile());
			}
		}
		else
		{
			// Stopped moving?
			//if(!this.pet.hasGoal())
			{
				// Not at the item?
				if(!this.pet.at(this.item))
				{
					if(!this.pet.hasGoal())
					{
					// Re-start trip or find new item?
					if(this.item.getTile().isReachableBy(this.pet))
					{
						this.pet.setGoal(this.item.getTile());
					}
					else
					{
						this.item = null;
					}
					}
				}
				else
				{
					// Begin/end interaction
					if(this.interactTimer == 0)
					{
						// Initialize timer
						this.interactTimer = GameUtil.secondsToRoomPulses(this.getActionDuration());
						
						// Process happiness boost
						this.pet.getInfo().modifyHappiness(+this.getHappinessBoost());
						
						// Start interaction
						this.pet.setDirection(this.item.getInteractDirection());
						this.pet.addStatus(this.getInteractionStatus());
						
						// Say something
						this.pet.sayEventMessage(this.getActivityEvent());
						
						// Other stuff
						this.beginInteract();
						this.submitResult(true);
					}
					else
					{
						this.interactTimer--;
						if(this.interactTimer == 0)
						{
							this.pet.release();
						}
						else
						{
							this.interactPulse();
						}
					}
				}
			}
		}
	}
	
	@Override
	public void clear()
	{
		String status = this.getInteractionStatus();
		if(this.pet.statuses.containsKey(status))
		{
			this.pet.removeStatus(status);
			if(this.item != null)
			{
				this.endInteract();
			}
		}
	}
}
