package habbod2.game.rooms.minigames.game;

import habbod2.Instance;
import static habbod2.core.Locale._I;
import habbod2.storage.Database;
import habbod2.game.rooms.instance.RoomActor;
import habbod2.game.rooms.instance.RoomInstance;
import habbod2.game.rooms.instance.RoomUser;
import habbod2.game.rooms.items.RoomFloorItem;
import habbod2.game.rooms.items.minigame.TimerItem;
import habbod2.game.rooms.items.minigame.ball.BallItem;
import habbod2.game.rooms.items.minigame.ball.GoalItem;
import habbod2.game.rooms.minigames.Game;
import habbod2.game.rooms.minigames.GameTeam;
import habbod2.game.rooms.minigames.GameUserPlayer;
import habbod2.game.rooms.minigames.GameTeam.ETeam;
import habbod2.game.rooms.minigames.GameUserPlayer.GameValues;

public class BallGame extends Game
{
	public BallGame(TimerItem timer)
	{
		super(timer);
	}
	
	@Override
	protected void onStart()
	{
		
	}
	
	@Override
	protected void onStop(GameTeam winner)
	{
		// Stop now
		this.setAlive(false);
		
		// Update stats
		Database db = Instance.get().getDb();
		for(GameTeam team : this.getTeams())
		{
			for(GameUserPlayer player : team)
			{
				if(player.getScore() > 0)
				{
					db.execute("UPDATE users SET bg_goals=bg_goals + " + player.getScore() + " WHERE id=" + player.getUser().getInfo().id);
				}
			}
		}
	}
	
	@Override
	protected void onPulse()
	{
		// Check all the balls
		for(RoomFloorItem item : this.getItems())
		{
			// This a ball?
			if(item instanceof BallItem)
			{
				// Recently shot by actor?
				BallItem ball = (BallItem)item;
				RoomActor actor = ball.getLastActor();
				if(actor != null)
				{
					// Ball in goal?
					for(RoomFloorItem tmp : ball.getTile().getItems())
					{
						if(tmp instanceof GoalItem)
						{
							GoalItem goal = (GoalItem)tmp;
							
							// Handle actor/user scoring
							GameTeam team = this.getTeam(goal.getTeamId());
							if(actor instanceof RoomUser)
							{
								RoomUser user = (RoomUser)actor;
								
								// Update team (user is in team last scored for)
								team.addPlayer(user);
								
								// Score!
								user.getPlayer().addScore(1);
								user.wave();
							}
							else
							{
								actor.chat(_I("GOAL!"), true);
								this.getTeam(goal.getTeamId()).addScore(1);
							}
							
							// Processed!
							ball.clearLastActor();
						}
					}
				}
			}
		}
	}


	@Override
	public int getEffectOffset()
	{
		return 0;
	}
	
	@Override
	public GameValues createValuesForPlayer()
	{
		return null;
	}
	
	public static final ETeam getOpponent(ETeam team)
	{
		if(team == ETeam.Red)
		{
			return ETeam.Blue;
		}
		else if(team == ETeam.Blue)
		{
			return ETeam.Red;
		}
		else if(team == ETeam.Green)
		{
			return ETeam.Yellow;
		}
		else if(team == ETeam.Yellow)
		{
			return ETeam.Green;
		}
		else
		{
			return team;
		}
	}
}
