package habbod2.game.sessions.local;

import habbod2.game.GameManager;
import habbod2.game.sessions.Session;
import habbod2.game.sessions.SessionManager;
import habbod2.game.users.User;
import habbod2.storage.DatabaseResult;

public class LocalSessionManager extends GameManager.Component implements Runnable
{
	public LocalSessionManager(GameManager gameMgr)
	{
		super(gameMgr);
	}
	
	public void start()
	{
		this.getSystem().scheduleTask(this, 1000);
	}
	
	public Session addSession()
	{
		// Get a user
		User user = this.selectUser();
		if(user == null)
		{
			return null;
		}
		
		// Join fake session
		LocalSession session = (LocalSession)this.getGameMgr().getSessionMgr().startSession(null);
		if(session.authenticate(user))
		{
			return session;
		}
		else
		{
			return null;
		}
	}
	
	private User selectUser()
	{
		// Select user
		try(DatabaseResult result = this.getDb().query("SELECT * FROM users WHERE allow_local=1 AND NOT(figure = 'ha-1015-62.hr-115-42.ch-878-62-62.hd-180-1.lg-270-64') ORDER BY RAND() LIMIT 1"))
		{
			if(result.row())
			{
				return this.getGameMgr().getUserMgr().processUserFromRow(result);
			}
		}
		
		return null;
	}

	@Override
	public void run()
	{
		// Get real session manager
		SessionManager sessionMgr = this.getGameMgr().getSessionMgr();
		
		// Process local sessions
		int localCount = 0;
		for(Session session : sessionMgr.getAuthedSessions())
		{
			if(session.isAuthenticated() && session instanceof LocalSession)
			{
				localCount++;
				
				try
				{
					((LocalSession)session).pulse();
				}
				catch(Exception ex)
				{
					this.getLogger().error(this, "Error pulsing a local session.", ex);
				}
			}
		}
		
		// No local sessions?
		if(!this.getConfig().getBool("sessions.local.enabled"))
		{
			return;
		}
		
		// Count total online users and total online 'real users'
		int totalCount = sessionMgr.authedSessions();
		int nonLocalCount = totalCount - localCount;
		
		// Get configured base and multiplier
		int base = this.getConfig().getInt("sessions.local.base");
		float multiplier = this.getConfig().getFloat("sessions.local.multiplier");
		int targetLocalCount = (int) (base + (nonLocalCount * multiplier));
		if(localCount < targetLocalCount)
		{
			this.addSession();
		}
	}
}
