package habbod2.game.users.subscriptions;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import habbod2.game.GameManager;
import habbod2.storage.DatabaseResult;
import habbod2.storage.SQLHelper;

public class SubscriptionManager extends GameManager.Component
{
	private Map<String, Subscription> types;
	
	public SubscriptionManager(GameManager gameMgr)
	{
		super(gameMgr);
		this.types = Collections.emptyMap();
	}
	
	public void reload()
	{
		// Create new collection
		Map<String, Subscription> tmpTypes = new HashMap<String, Subscription>(0);
		
		// Load all subscriptions from database
		try(DatabaseResult result = this.getDb().query("SELECT * FROM subscriptions"))
		{
			while(result.row())
			{
				// Parse subscription info
				Subscription sub = new Subscription();
				sub.id = result.getString("id");
				sub.name = result.getString("name");
				sub.pricePerMonth = result.getInt("price");
				sub.specialRightsLevel = result.getByte("specialrightslevel");
				
				// Get subscription rights
				try(DatabaseResult result2 = this.getDb().query("SELECT `right` FROM subscription_rights WHERE subscription_id='" + SQLHelper.escape(sub.id) + "'"))
				{
					while(result2.row())
					{
						sub.rights.add(result2.getString("right"));
					}
					
					// Get subscription badges
					result2.query("SELECT badge FROM subscription_badges WHERE subscription_id='" + SQLHelper.escape(sub.id) + "'");
					while(result2.row())
					{
						sub.badges.add(result2.getString("badge"));
					}
				}
				
				// Add it!
				tmpTypes.put(sub.id, sub);
				this.getLogger().info(this, String.format("Loaded '%s': %d right(s) and %d badge(s).", sub.id, sub.rights.size(), sub.badges.size()));
			}
		}
		
		// Set instance field
		this.types = Collections.unmodifiableMap(tmpTypes);
	}
	
	public List<UserSubscription> getSubscriptionsForUser(int userId)
	{
		List<UserSubscription> subs = new ArrayList<UserSubscription>(0);
		
		try(DatabaseResult result = this.getDb().query("SELECT subscription_id,activated_at,expires_at FROM user_subscriptions WHERE user_id=" + userId))
		{
			while(result.row())
			{
				UserSubscription sub = new UserSubscription();
				sub.type = this.getSubscriptionType(result.getString("subscription_id"));
				if(sub.type != null)
				{
					sub.activatedAt = result.getTime("activated_at");
					sub.expiresAt = result.getTime("expires_at");
					subs.add(sub);
				}
			}
		}
		
		return subs;
	}
	
	public Subscription getSubscriptionType(String subscriptionId)
	{
		return this.types.get(subscriptionId);
	}
}
