package habbod2.game.users.updaters;

import static habbod2.core.Locale._I;
import habbod2.game.GameManager;
import habbod2.game.sessions.Session;
import habbod2.game.users.User;
import habbod2.util.ScheduledTask;

public class AutoCreditsUpdater extends GameManager.Component implements Runnable
{
	private int minutes;
	private ScheduledTask task;
	
	public AutoCreditsUpdater(GameManager gameMgr)
	{
		super(gameMgr);
	}

	public void reschedule()
	{
		// Cancel task?
		if(this.task != null)
		{
			this.task.stop();
			this.task = null;
		}
		
		// Schedule new task
		this.minutes = this.getConfig().getInt("users.autocredits.minutes");
		if(this.minutes > 0)
		{
			this.task = this.getSystem().scheduleTask(this, this.minutes * 60 * 1000);
		}
	}
	
	@Override
	public void run()
	{
		try
		{
			this.update();
		}
		catch(Exception ex)
		{
			this.getLogger().error(this, "Error during update.", ex);
		}
	}
	
	private void update()
	{
		// Get configuration
		int add = this.getConfig().getInt("users.autocredits.add");
		int max = this.getConfig().getInt("users.autocredits.max");
		boolean socialBoost = this.getConfig().getBool("users.autocredits.socialboost");
		boolean notify = this.getConfig().getBool("users.autocredits.notify");
		
		// Enabled?
		int userAmount = 0;
		int totalAmount = 0;
		if(add > 0)
		{
			// Go through all online users
			for(Session session : this.getGameMgr().getSessionMgr().getAuthedSessions())
			{
				// Get user
				User user = session.getUser();
				
				// Amount to add
				int toAdd = add;
				
				// User is alone in room?
				boolean isWithOthers = (session.inRoom() && session.getRoom().getInfo().usersNow > 1);
				
				// Social boost enabled?
				if(socialBoost && isWithOthers)
				{
					toAdd = toAdd * 2;
				}
			
				// Impose limit?
				if(max > 0 && (user.credits + toAdd) > max)
				{
					toAdd = max - user.credits;
				}
				
				// Anything to add?
				if(toAdd > 0)
				{
					// Add it
					user.credits += toAdd;
					user.saveCurrencies();
					session.sendCreditBalance();
					
					// Social boost hint?
					if(socialBoost && !isWithOthers)
					{
						session.sendNotif(_I("This time you have received the normal amount (%d) of credits. Play in a room with other users to receive twice as much.", add));
					}
					
					// General notification?
					if(notify)
					{
						session.sendNotif(_I("You have received %d credits!", toAdd));
					}
					
					// Processed a user!
					userAmount++;
					totalAmount += toAdd;
				}
			}
		}
		
		// Done!
		if(userAmount > 0)
		{
			this.getLogger().debug(this, String.format("%d user(s) received a total of %d credits.", userAmount, totalAmount));
		}
		
		// Configuration changed?
		int minutes = this.getConfig().getInt("users.autocredits.minutes");
		if(minutes != this.minutes)
		{
			this.reschedule();
		}
	}
}
