package habbod2.game.users.updaters;

import java.util.Calendar;

import habbod2.game.GameManager;
import habbod2.game.sessions.Session;
import habbod2.game.users.User;
import habbod2.util.StringUtil;
import habbod2.communication.messages.outgoing.handshake.UserObjectMessageComposer;

public class AutoRespectUpdater extends GameManager.Component implements Runnable
{
	public AutoRespectUpdater(GameManager gameMgr)
	{
		super(gameMgr);
	}
	
	private Calendar getTodayMidnight()
	{
		Calendar c = Calendar.getInstance();
		c.setTimeInMillis(System.currentTimeMillis());
		c.set(Calendar.HOUR_OF_DAY, 0);
		c.set(Calendar.MINUTE, 0);
		c.set(Calendar.SECOND, 0);
		c.set(Calendar.MILLISECOND, 0);
			
		return c;
	}

	public void scheduleDailyAtMidnight()
	{
		Calendar firstTime = this.getTodayMidnight();
		this.getSystem().scheduleTaskWithFirstTime(this, 24 * 60 * 60 * 1000, firstTime.getTimeInMillis());
	}
	
	@Override
	public void run()
	{
		// Get current time
		Calendar now = Calendar.getInstance();
		now.setTimeInMillis(System.currentTimeMillis());
		
		// Get last update time
		Calendar lastUpdate = Calendar.getInstance();
		lastUpdate.setTimeInMillis(StringUtil.parseDateTime(this.getConfig().get("users.autorespect.lastupdate")));
		
		// Supposed to run now?
		int daysSinceUpdate = Math.abs(now.get(Calendar.DAY_OF_YEAR) - lastUpdate.get(Calendar.DAY_OF_YEAR));
		if(daysSinceUpdate == 0 && (now.get(Calendar.HOUR_OF_DAY) != 0 || now.get(Calendar.MINUTE) != 0))
		{
			this.getLogger().info(this, "Already updated today, will run again at midnight.");
			return;
		}
		
		// Get reset amounts
		int userRespectPoints = this.getConfig().getInt("users.autorespect.users.amount");
		int petRespectPoints = this.getConfig().getInt("users.autorespect.pets.amount");
		
		// Update it in the database
		this.getDb().execute("UPDATE users SET respect_points=" + userRespectPoints + ",petrespect_points=" + petRespectPoints);
		
		// Update + refresh online users
		for(Session session : this.getGameMgr().getSessionMgr().getAuthedSessions())
		{
			User user = session.getUser();
			user.respectPoints = userRespectPoints;
			user.petRespectPoints = petRespectPoints;
			session.send(UserObjectMessageComposer.compose(user));
		}
		
		// Done!
		lastUpdate = this.getTodayMidnight();
		this.getConfig().set("users.autorespect.lastupdate", StringUtil.formatDateTime(lastUpdate.getTime()));
		this.getLogger().info(this, String.format("Updated respect points for all users. (users=%d, pets=%d)", userRespectPoints, petRespectPoints));
	}
}
