package habbod2.game.rooms.items.floor;

import habbod2.communication.messages.outgoing.room.engine.SlideObjectBundleMessageComposer;
import habbod2.game.rooms.geometry.Direction;
import habbod2.game.rooms.instance.RoomActor;
import habbod2.game.rooms.instance.RoomMap;
import habbod2.game.rooms.instance.RoomTile;
import habbod2.game.rooms.instance.RoomUser;
import habbod2.game.rooms.items.RoomFloorItem;
import habbod2.game.rooms.items.minigame.banzai.GameTileItem;
import habbod2.game.rooms.items.minigame.banzai.PuckItem;

public class BallItem extends RoomFloorItem
{
	/*
	 * bal uit doel trekken: ervoor staan (facing), toggle
voor de bal staan, toggle: rol max 4 vooruit in richting van facing
over de bal heenrennen, einde van pad? schiet max 6 vooruit in richting van facing. Niet einde van pad? dribbelen, 1 vakje vooruit
Vooruit rollen: botsing? Reverse direction (opposite), tel verder vanaf daar
	 */
	
	private RoomActor lastActor;
	
	@Override
	public void onActorStepOn(RoomActor actor)
	{
		super.onActorStepOn(actor);
		if(actor.getTile() == actor.goal)
		{
			this.shoot(actor, 6);
		}
		else
		{
			this.shoot(actor, 1);
		}
	}
	
	@Override
	public void onToggle(RoomUser user, int instruction)
	{
		if(user.touches(this))
		{
			this.shoot(user, 1);
		}
	}
	
	public void shoot(RoomActor actor, final int maxTiles)
	{
		// Record begin position
		RoomTile from = this.getTile();
		RoomTile to = from;
		RoomMap map = this.getRoom().getMap();
		
		// Shoot max x tiles in front of actor, reverse and resume direction at obstacles
		int moved = 0;
		Direction dir = actor.bodyDirection;
		while(moved < maxTiles)
		{
			RoomTile tile = map.getTile(to.x + dir.modX, to.y + dir.modY);
			if(tile != null && tile.isOpen())
			{
				to = tile;
				if(this instanceof PuckItem)
				{
					if(tile.topItem instanceof GameTileItem)
					{
						tile.topItem.onActorStepOn(actor);
					}
				}
			}
			else
			{
				dir = dir.invert();
			}
			moved++;
		}
		
		// Movable?
		if(to != from)
		{
			// Update item
			float fromZ = this.z;
			this.getRoom().moveItem(this.id, to.x, to.y, this.rotation, false);
			
			// Set state
			int state;
			if(this instanceof PuckItem)
			{
				state = 0;
				if(actor instanceof RoomUser)
				{
					RoomUser user = (RoomUser)actor;
					if(user.isPlayer())
					{
						state = user.getPlayer().getTeam().getId().ordinal() + 1;
					}
				}
			}
			else
			{
				state = moved;
			}
			this.setTempState(state);
			
			// Record last actor
			this.lastActor = actor;
			
			// OK!
			this.getRoom().broadcast(SlideObjectBundleMessageComposer.compose(this, from, fromZ, null));
		}
	}
	
	public RoomActor getLastActor()
	{
		return this.lastActor;
	}
	
	public void clearLastActor()
	{
		this.lastActor = null;
	}
}
