package habbod2.game.rooms.items.floor;

import habbod2.Instance;
import habbod2.game.rooms.instance.RoomUser;
import habbod2.game.util.GameUtil;

public class VendingMachineItem extends UsableFloorItem
{
	@Override
	protected void onToggle(RoomUser user, int instruction)
	{
		// User operating?
		if(user == null)
		{
			return;
		}
		
		// Get to the vending machine! (if possible)
		if(user.canMove() && !user.at(this.getTileInFront()))
		{
			user.setGoal(this.getTileInFront());
			return;
		}
		
		// In use?
		if(this.inUse())
		{
			return;
		}
		
		// Start using the vending machine
		user.lockMovement();
		user.lookTo(this.getTile(), false);
				
		// Vending in 2 pulses!
		this.setUser(user);
		this.setTimeout(2);
	}

	@Override
	protected void onTimeout()
	{
		// In use?
		if(this.inUse())
		{
			// Determine a vending ID
			Integer vendingId = -1;
			try
			{
				vendingId = Integer.parseInt(GameUtil.getRandomElement(this.definition.data.split(",")));
			}
			catch(Exception ex)
			{
				Instance.get().getLogger().warning("VendingMachineItem", String.format("Invalid vending IDs classdata for '%s': \"%s\", VendingMachineItems should specify atleast one numeric vending ID, and the classdata must be delimited by commas. Example: \"1,6,15,23\"", this.definition.sprite, this.definition.data));
			}
			finally
			{
				// Vend the item
				RoomUser user = this.getUser();
				user.carryItem(vendingId);
				user.unlockMovement();
				this.clearUser();
			}
		}
	}
}