package habbod2.game.rooms.items.minigame;

import java.util.ArrayList;
import java.util.List;

import habbod2.game.rooms.instance.RoomActor;
import habbod2.game.rooms.instance.RoomTile;
import habbod2.game.rooms.instance.RoomUser;
import habbod2.game.rooms.items.RoomFloorItem;
import habbod2.game.rooms.minigames.Game;
import habbod2.game.rooms.minigames.GameTeam;
import habbod2.game.rooms.minigames.GameTeam.ETeam;
import habbod2.game.util.GameUtil;

public class GateItem extends RoomFloorItem
{	
	public static final int MAX_PLAYERS = 5;
	
	private String gameType;
	private ETeam teamId;
	
	private boolean isClosed;
	
	@Override
	public void onRoomAdd()
	{
		String[] data = this.definition.data.split(" ");
		this.gameType = data[0];
		if(data.length >= 2)
		{
			this.teamId = ETeam.valueOf(data[1]);
		}
		this.setState(0);
	}
	
	@Override
	public void onActorStepOn(RoomActor actor)
	{
		super.onActorStepOn(actor);
		
		// Get user
		if(!(actor instanceof RoomUser))
		{
			return;
		}
		RoomUser user = (RoomUser)actor;
		
		// Quit or join?
		if(user.isPlayer())
		{
			user.setPlayer(null);
		}
		else
		{
			// Validate game
			Game game = this.getRoom().getGame();
			if(game != null && game.getType().equals(this.gameType))
			{
				if(this.teamId != null)
				{
					// Attempt to join the specified team
					GameTeam team = game.getTeam(this.teamId);
					if(team != null && !team.isFull())
					{
						team.addPlayer(user);
					}
				}
				else
				{
					// Get list of teams with capacity
					List<GameTeam> teams = new ArrayList<GameTeam>(ETeam.values().length);
					for(GameTeam team : game.getTeams())
					{
						if(!team.isFull())
						{
							teams.add(team);
						}
					}
					
					// Select random team
					GameTeam team = GameUtil.getRandomElement(teams);
					if(team != null)
					{
						team.addPlayer(user);
					}
				}
			}
		}
	}
	
	public ETeam getTeamId()
	{
		return this.teamId;
	}
	
	public boolean isCompatibleWith(Game game)
	{
		return (this.gameType != null && this.gameType.equals(game.getType()));
	}
	
	public void setPlayerCount(int count)
	{
		this.setState(count);
	}
	
	public void setClosed(boolean isClosed)
	{
		this.isClosed = isClosed;
		this.onGenerateMapTile(this.getTile());
	}
	
	@Override
	public void onGenerateMapTile(RoomTile tile)
	{
		super.onGenerateMapTile(tile);
		tile.state = (this.isClosed ? RoomTile.CLOSED : RoomTile.OPEN);
	}
}
