package habbod2.game.rooms.items.minigame;

import habbod2.game.rooms.instance.RoomUser;
import habbod2.game.rooms.items.RoomFloorItem;
import habbod2.game.rooms.items.minigame.banzai.ApparatusItem;
import habbod2.game.rooms.minigames.game.BanzaiGame;
import habbod2.game.rooms.minigames.game.FreezeGame;
import habbod2.game.rooms.minigames.game.BallGame;
import habbod2.game.util.GameUtil;
import habbod2.util.StringUtil;

public class TimerItem extends RoomFloorItem
{
	private int time;
	private boolean isRunning;
	private boolean isStarted;
	
	public boolean isRunning()
	{
		return this.isRunning;
	}
	
	public int getTime()
	{
		return this.time;
	}
	
	public void setTime(int seconds)
	{
		this.time = seconds;
		this.setState(this.time);
	}
	
	public void start()
	{
		// Already running?
		if(this.isRunning)
		{
			return;
		}
		
		// Reset time?
		if(this.time == 0)
		{
			this.reset();
		}
		
		// Resume from pause?
		if(this.isStarted)
		{
			this.isRunning = true;
			this.setTimeout(GameUtil.secondsToRoomPulses(1));
		}
		else
		{
			// Process apparatus items
			boolean beginNow = true;
			for(RoomFloorItem item : this.getRoom().getFloorItems())
			{
				if(item instanceof ApparatusItem)
				{
					((ApparatusItem)item).enable();
					beginNow = false;
				}
			}
			
			// Initialize game
			this.getRoom().getGame().init();
			
			// Wait for apparatus?
			if(beginNow)
			{
				this.begin();
			}
		}
	}
	
	public void begin()
	{
		if(!this.isStarted)
		{
			this.isRunning = true;
			this.isStarted = true;
			this.setTimeout(GameUtil.secondsToRoomPulses(1));
			if(this.getRoom().getGame() != null)
			{
				this.getRoom().getGame().begin();
			}
		}
	}
	
	public void stop()
	{
		if(this.isRunning || this.getRoom().getGame().isShuttingDown())
		{
			this.isRunning = false;
			if(this.isStarted)
			{
				this.isStarted = false;
				this.getRoom().getGame().end();
				for(RoomFloorItem item : this.getRoom().getFloorItems())
				{
					if(item instanceof ApparatusItem)
					{
						((ApparatusItem)item).disable();
						item.setTempState(item.state);
					}
				}
			}
		}
	}
	
	public void pause()
	{
		this.isRunning = false;
		this.clearTimeout();
	}
	
	public void reset()
	{
		// Stop!
		this.stop();
		
		// Default time set? Set it
		int time = StringUtil.parseInt(this.data);
		if(time > 0)
		{
			this.setTime(time);
		}
		else
		{
			this.setTime(30);
		}
	}
	
	@Override
	public void onRoomAdd()
	{
		// Create game instance
		String type = this.definition.data;
		if(type.equals("BanzaiGame"))
		{
			this.getRoom().setGame(new BanzaiGame(this));
		}
		else if(type.equals("FreezeGame"))
		{
			this.getRoom().setGame(new FreezeGame(this));
		}
		else if(type.equals("BallGame"))
		{
			this.getRoom().setGame(new BallGame(this));
		}
		else
		{
			this.getRoom().setGame(null);
		}
		
		// Reset to initial time
		this.reset();
	}
	
	@Override
	protected void onRoomRemove()
	{
		this.getRoom().setGame(null);
	}
	
	@Override
	protected void onToggle(RoomUser user, int instruction)
	{
		// Only for room rights
		if(user != null && !user.hasRoomRights)
		{
			return;
		}
		
		// Temporarily unusable?
		if(this.getRoom().getGame().isShuttingDown())
		{
			return;
		}
		
		// Start/pause
		if(instruction <= 1)
		{
			if(this.isRunning)
			{
				this.pause();
			}
			else
			{
				this.start();
			}
		}
		
		// Add time
		else if(instruction == 2)
		{
			// Stop
			this.stop();
			
			// Determine new time
			int newTime;
			if(this.time >= 0 && this.time < 30)
			{
				newTime = 30;
			}
			else if(this.time >= 30 && this.time < 60)
			{
				newTime = 60;
			}
			else if(this.time >= 60 && this.time < 120)
			{
				newTime = 120;
			}
			else if(this.time >= 120 && this.time < 180)
			{
				newTime = 180;
			}
			else if(this.time >= 180 && this.time < 240)
			{
				newTime = 240;
			}
			else if(this.time >= 240 && this.time < 300)
			{
				newTime = 300;
			}
			else if(this.time >= 300 && this.time < 600)
			{
				newTime = 600;
			}
			else
			{
				newTime = 30;
			}
			
			// Set time and memorize as 'default' time
			this.setTime(newTime);
			this.data = Integer.toString(this.time);
		}
	}	

	@Override
	public void onTimeout()
	{
		if(this.isRunning && this.time > 0)
		{
			this.setTime(this.time - 1);
			if(this.time > 0)
			{
				this.setTimeout(GameUtil.secondsToRoomPulses(1));
			}
			else
			{
				this.stop();
			}
		}
	}
	
	@Override
	public boolean requiresPulse()
	{
		return true;
	}
}
