package habbod2.game.rooms.items.minigame.banzai;

import java.util.ArrayList;
import java.util.List;

import habbod2.game.rooms.geometry.Direction;
import habbod2.game.rooms.instance.RoomActor;
import habbod2.game.rooms.instance.RoomUser;
import habbod2.game.rooms.items.RoomFloorItem;
import habbod2.game.util.GameUtil;

public class RandomWarpTileItem extends RoomFloorItem
{
	private boolean isWaiting;
	
	@Override
	public void onActorStepOn(RoomActor actor)
	{
		// Only usable by RoomUsers
		if(actor instanceof RoomUser)
		{
			// Not warping already/landing from warp?
			RoomUser user = (RoomUser)actor;
			if(!user.isWarping())
			{
				// Find a random destination
				RandomWarpTileItem other = this.findOther();
				if(other != null)
				{
					// Flash tiles
					this.flash();
					other.flashLater();
					
					// Schedule warp for user, end up with a random direction
					user.warp(other.getTile(), Direction.random());
				}
			}
		}
		
		// Handle other stuff
		super.onActorStepOn(actor);
	}
	
	public RandomWarpTileItem findOther()
	{
		// Aggregrate all the warptiles
		List<RandomWarpTileItem> items = new ArrayList<RandomWarpTileItem>();
		for(RoomFloorItem item : this.getRoom().getFloorItems())
		{
			if(item instanceof RandomWarpTileItem && item != this)
			{
				items.add((RandomWarpTileItem) item);
			}
		}
		
		// Select a random one
		return GameUtil.getRandomElement(items);
	}
	
	public void flash()
	{
		this.setTempState(1);
		this.setTimeout(1);
	}
	
	public void flashLater()
	{
		this.isWaiting = true;
		this.setTimeout(1);
	}
	
	@Override
	public void onTimeout()
	{
		if(this.isWaiting)
		{
			this.flash();
			this.isWaiting = false;
		}
		else
		{
			this.setTempState(0);
		}
	}
}
