package habbod2.game.rooms.items.minigame.freeze;

import habbod2.game.rooms.instance.RoomActor;
import habbod2.game.rooms.instance.RoomUser;
import habbod2.game.rooms.items.RoomFloorItem;
import habbod2.game.rooms.minigames.Game;
import habbod2.game.rooms.minigames.game.FreezeGame;
import habbod2.game.rooms.minigames.game.FreezeGame.FreezeValues;

public class GameTileItem extends RoomFloorItem
{
	@Override
	public void onRoomAdd()
	{
		if(!this.state.isEmpty())
		{
			this.setState(0);
		}
	}
	
	@Override
	public void onActorStepOn(RoomActor actor)
	{
		super.onActorStepOn(actor);
		if(actor instanceof RoomUser)
		{
			RoomUser user = (RoomUser)actor;
			Game game = this.getRoom().getGame();
			if(!user.isPlayer() && game instanceof FreezeGame && game.isRunning())
			{
				GameOverTileItem exit = ((FreezeGame)game).getExit();
				if(exit != null)
				{
					user.warp(exit.getTile());
				}
			}
		}
	}
	
	@Override
	public void onToggle(RoomUser user, int operation)
	{
		// Users need to be on/close to the tile
		if(user != null)
		{
			// Not a player?
			if(!user.isPlayer())
			{
				return;
			}
			
			// Not touching or on the tile?
			if(!user.at(this) && !(user.isMoving() && user.next == this.getTile()))
			{
				return;
			}
		}
		
		// No running FREEZE! game?
		Game game = this.getRoom().getGame();
		if(!(game instanceof FreezeGame) || !game.isRunning())
		{
			return;
		}
		
		// Run user checks
		if(user != null)
		{
			// User cannot throw a ball now?
			FreezeValues values = user.getPlayer().getValues();
			if(values.balls <= 0 || values.isFrozen())
			{
				return;
			}
		}
		
		// Launch the ball!
		((FreezeGame)game).launchBall(this, (user != null) ? user.getPlayer() : null);
	}
}