package habbod2.game.rooms.items.minigame.freeze;

import habbod2.game.rooms.instance.RoomActor;
import habbod2.game.rooms.instance.RoomTile;
import habbod2.game.rooms.instance.RoomUser;
import habbod2.game.rooms.items.RoomFloorItem;
import habbod2.game.rooms.minigames.game.FreezeGame;
import habbod2.game.rooms.minigames.game.FreezeGame.EPowerUp;

public class PowerUpBoxItem extends RoomFloorItem
{
	private EPowerUp powerUp;
	
	@Override
	public void onRoomAdd()
	{
		this.setPowerUp(null);
	}
	
	@Override
	public void onGenerateMapTile(RoomTile tile)
	{
		super.onGenerateMapTile(tile);
		tile.state = this.isCrushed() ? RoomTile.OPEN : RoomTile.CLOSED;
	}
	
	@Override
	public void onToggle(RoomUser user, int operation)
	{
		for(RoomFloorItem item : this.getTile().getItems())
		{
			if(item instanceof GameTileItem)
			{
				((GameTileItem) item).onToggle(user, operation);
				break;
			}
		}
	}
	
	@Override
	public void onActorStepOn(RoomActor actor)
	{
		super.onActorStepOn(actor);
		
		// Has powerup?
		if(this.powerUp == EPowerUp.None || !(actor instanceof RoomUser))
		{
			return;
		}
		
		// User is player?
		RoomUser user = (RoomUser)actor;
		if(user.isPlayer() && user.getPlayer().getGame() instanceof FreezeGame)
		{
			// Game running?
			FreezeGame game = (FreezeGame) user.getPlayer().getGame();
			if(game.isRunning())
			{
				game.applyPowerUp(user.getPlayer(), this.powerUp);
				this.setPowerUp(EPowerUp.None);
			}
		}
	}
	
	public boolean isCrushed()
	{
		return (this.powerUp != null);
	}
	
	public EPowerUp getPowerUp()
	{
		return this.powerUp;
	}
	
	public void setPowerUp(EPowerUp powerUp)
	{
		// Update visualization
		if(powerUp == EPowerUp.None && this.isCrushed())
		{
			this.setState(PowerUpBoxItem.generateState(this.powerUp, true));
			//this.state = PowerUpBoxItem.generateState(powerUp, false);
		}
		else
		{
			this.setState(PowerUpBoxItem.generateState(powerUp, false));
		}
		
		// Update data
		this.powerUp = powerUp;
		if(this.getRoom().isContentLoaded())
		{
			this.onGenerateMapTile(this.getTile());
		}
	}
	
	public static String generateState(EPowerUp powerUp, boolean fadeAway)
	{
		int num = 0;
		if(powerUp != null)
		{
			num = (powerUp.ordinal() + 1) * 1000;
			if(fadeAway)
			{
				num += 10000;
			}
		}
		return Integer.toString(num);
	}
}
