package habbod2.game.rooms.items.wired.conditions;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;

public abstract class TimestampCondition extends TextValueCondition
{
	private static final DateFormat timeFormat = new SimpleDateFormat("HH:mm:ss");
	private static final DateFormat datetimeFormat = new SimpleDateFormat("dd-MM-yyyy HH:mm:ss");
	
	protected long timestamp;
	
	@Override
	protected void onSettingsUpdate()
	{
		String text = this.getText();
		if(text.isEmpty())
		{
			this.timestamp = 0;
		}
		else
		{
			try
			{
				// Parse the specified timestamp
				Calendar timestamp = Calendar.getInstance();
				timestamp.setTime((text.indexOf(' ') == -1 ? timeFormat : datetimeFormat).parse(text));
				
				// Day/month/year omitted? Assume current date/month/year
				if(timestamp.get(Calendar.YEAR) == 1970)
				{
					Calendar now = Calendar.getInstance();
					timestamp.set(Calendar.DAY_OF_MONTH, now.get(Calendar.DAY_OF_MONTH));
					timestamp.set(Calendar.MONTH, now.get(Calendar.MONTH));
					timestamp.set(Calendar.YEAR, now.get(Calendar.YEAR));
				}
				
				// Extract timestamp time
				this.timestamp = timestamp.getTimeInMillis();
			}
			catch (ParseException e)
			{
				this.timestamp = -1;
			}
		}
	}
	
	public long getTimestamp()
	{
		return this.timestamp;
	}
}