package prinzip.accounts.actions;

import java.util.Date;

import org.apache.log4j.Logger;

import prinzip.accounts.Account;
import twitter4j.Twitter;
import twitter4j.TwitterException;
import twitter4j.User;

public class FollowAction extends TargetedAccountAction
{
	private static final Logger logger = Logger.getLogger(FollowAction.class);

	public FollowAction(Account account, long targetId)
	{
		super(account, targetId);
	}

	@Override
	protected void execute() throws TwitterException
	{
		// Resolve account and twitter
		Account account = super.getAccount();
		Twitter twitter = account.getTwitter();

		// Attempt to follow the target
		User target = twitter.createFriendship(super.getTargetId(), false);
		
		// Following!
		logger.info(String.format("@%s is now following @%s", account.username, target.getScreenName()));

		// Record for a check
		account.toFollowCheck.put(target.getId(), new Date());
	}
	
	protected void onCompletion()
	{
		// Save
		Account account = this.getAccount();
		account.toFollow.remove(this.getTargetId());
		account.save();
		
		// Resume next
		account.nextFollowAction(this);
		account.nextCheckAction(null);
	}
	
	@Override
	protected void onOtherTwitterApiException(TwitterException tex)
	{
		// User actually found?
		if(tex.resourceNotFound())
		{
			logger.warn(String.format("@%s could not follow non-existant user #%d", this.getAccount().username, this.getTargetId()));
		}
		
		// Could not follow for whatever reason?
		else if(tex.getStatusCode() == TwitterException.FORBIDDEN)
		{
			logger.warn(String.format("@%s could not follow #%d: %s (#%d)", this.getAccount().username, this.getTargetId(), tex.getErrorMessage(), tex.getErrorCode()));
		}
		
		// Hmm...
		else
		{
			super.onUnhandledTwitterApiException(tex);
		}
	}

	@Override
	protected String getDescription()
	{
		return String.format("follow(%d)", this.getTargetId());
	}
}
