package prinzip.accounts.actions;

import org.apache.log4j.Logger;

import prinzip.accounts.Account;

import twitter4j.Relationship;
import twitter4j.Twitter;
import twitter4j.TwitterException;

public class FollowedBackCheckAction extends TargetedAccountAction
{
	private static final Logger logger = Logger.getLogger(FollowedBackCheckAction.class);
	
	private boolean doUnfollow;
	
	public FollowedBackCheckAction(Account account, long targetId)
	{
		super(account, targetId);
	}

	@Override
	protected void execute() throws TwitterException
	{
		// Resolve account and twitter
		Account account = super.getAccount();
		Twitter twitter = account.getTwitter();
		
		// Evaluate whether the user is followed back by now
		Relationship rel = twitter.showFriendship(twitter.getId(), super.getTargetId());
		if(rel.isTargetFollowingSource())
		{
			// Success!
			account.followedBack.add(super.getTargetId());
			logger.info(String.format("@%s is followed back by @%s, success!", account.username, rel.getTargetUserScreenName()));
		}
		else
		{
			// Too bad
			account.notFollowedBack.add(super.getTargetId());
			logger.info(String.format("@%s is not followed back by @%s", account.username, rel.getTargetUserScreenName()));
			this.doUnfollow = true;
		}
	}
	
	@Override
	protected void onCompletion()
	{
		// Schedule next check (only force when this one wasn't rescheduled)
		Account account = this.getAccount();
		account.toFollowCheck.remove(super.getTargetId());
		account.nextCheckAction(this);
		
		// Unfollow this user in the near future?
		if(this.doUnfollow)
		{
			account.toUnfollow.add(super.getTargetId()); // conflicting in pendingActions
			account.nextUnfollowAction(null);
		}
		
		// Save state
		account.save();	
	}
	
	@Override
	protected void onOtherTwitterApiException(TwitterException tex)
	{
		// User actually found?
		if(tex.resourceNotFound())
		{
			logger.warn(String.format("@%s could not unfollow non-existant user #%d", this.getAccount().username, this.getTargetId()));
		}
		
		// Could not follow for whatever reason?
		else if(tex.getStatusCode() == TwitterException.FORBIDDEN)
		{
			logger.warn(String.format("@%s could not unfollow #%d: %s (#%d)", this.getAccount().username, this.getTargetId(), tex.getErrorMessage(), tex.getErrorCode()));
		}
		
		// Hmm...
		else
		{
			super.onUnhandledTwitterApiException(tex);
		}
	}

	@Override
	protected String getDescription()
	{
		return String.format("follow_check(%d)", super.getTargetId());
	}
}
