package prinzip.util;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map.Entry;
import java.util.concurrent.TimeUnit;

import org.apache.log4j.Logger;

import prinzip.accounts.Account;
import twitter4j.Twitter;
import twitter4j.TwitterException;
import twitter4j.User;

import com.google.common.base.Joiner;
import com.google.common.collect.Lists;

public class CompatUpdate
{
	private static final Logger logger = Logger.getLogger(CompatUpdate.class);

	public static void update(Account account)
	{
		logger.info("upgrading @" + account.username);

		if(account.email != null)
			account.email = account.email.replace("\"", "");

		if(account.searchQuery != null)
		{
			account.searchQuery = account.searchQuery.replace("\"", "");

			// Add keywords
			for(String keyword : account.searchQuery.split(" OR "))
			{
				account.keywords.add(keyword);
			}
			account.searchQuery = null;

			// Fix times
			for(Entry<Long, Date> check : account.toFollowCheck.entrySet())
			{
				check.getValue().setTime(check.getValue().getTime() - TimeUnit.DAYS.toMillis(3));
			}
		}

		if(account.importUserNames != null)
		{
			account.importUsers = resolveUserIds(account, Joiner.on(",").join(account.importUserNames));
			account.importUserNames = null;
		}

		logger.info("upgraded @" + account.username);

		account.save();
	}

	private static List<Long> resolveUserIds(Account account, String input)
	{
		// Parse usernames
		List<String> userNames = Lists.newArrayList();
		for(String userName : input.split(","))
		{
			userName = userName.replace("\"", "");
			if(!userName.isEmpty())
			{
				userNames.add(userName);
			}
		}

		// Resolve user IDs
		List<Long> userIds = Lists.newArrayList();
		if(!userNames.isEmpty())
		{
			Twitter utility;
			try
			{
				utility = account.getTwitter();
			}
			catch(Exception e)
			{
				utility = TwitterUtilityAccount.getUtilityAccount();
			}

			if(utility == null)
			{
				logger.warn("no utility account available, imports not resolved");
			}
			else
			{
				try
				{
					for(User user : utility.lookupUsers(userNames.toArray(new String[0])))
					{
						userIds.add(user.getId());
					}
				}
				catch(TwitterException e)
				{
					logger.error(String.format("@%s: could not resolve usernames '%s'", account.username, Arrays.toString(account.importUserNames)), e);
				}
			}
		}

		return userIds;
	}
}
