package prinzip.webapi;

import org.simpleframework.http.Method;
import org.simpleframework.http.Request;
import org.simpleframework.http.Response;
import org.simpleframework.http.Status;

public abstract class ResourceRequestHandler<T>
{
	protected T resource;
	
	protected ResourceRequestHandler(T resource)
	{
		this.resource = resource;
	}
	
	public final void handleRequest(Request req, Response res)
	{
		try
		{
			if(this.resource != null)
			{
				switch(req.getMethod())
				{
					case Method.GET:
						this.onRead(req, res); res.setStatus(Status.OK);
						break;
						
					case Method.POST:
						res.setStatus(this.onSave(req, res) ? Status.OK : Status.NOT_MODIFIED);
						break;
						
					case Method.DELETE:
						res.setStatus(this.onDelete(req, res) ? Status.GONE : Status.NOT_MODIFIED);
						break;
						
					default:
						res.setStatus(Status.METHOD_NOT_ALLOWED);
				}
			}
			else if(req.getMethod().equals(Method.PUT))
			{
				res.setStatus(this.onCreate(req, res) ? Status.CREATED : Status.NOT_ACCEPTABLE);
			}
			else
			{
				res.setStatus(Status.NOT_FOUND);
			}
		}
		catch(Exception ex)
		{
			res.setStatus(Status.INTERNAL_SERVER_ERROR);
		}
	}
	
	protected abstract boolean onCreate(Request req, Response res) throws Exception;
	protected abstract void onRead(Request req, Response res) throws Exception;
	protected abstract boolean onSave(Request req, Response res) throws Exception;
	protected abstract boolean onDelete(Request req, Response res) throws Exception;
}
